#### $^O will be used to determine which OS the perl program is running on
#### This will determine if any special system calls or commands need to be ran
#### Otherwise, a generic command that works on all platforms will be used
#### MSWin32 = Windows platforms
#### linux = Linux platforms
#### aix = AIX
#### hpux = HP-UX
#### solaris or sunos = Solaris platforms

#Set up some vars for discarding output
if ($^O eq "MSWin32")
{
  $NULL_DEVICE		= "NUL";
}
else
{
  $NULL_DEVICE		= "/dev/null";
}

$PDINFO_LOC		=	"$PDBASE_VAR/etc/pdinfo";
$CONFIG_FILE		=	"$PDINFO_LOC/pdinfo.cfg";

### If the user isn't root or has suid'd to root then exit
### No need to worry about for windows
if( !($^O eq "MSWin32") && (($> != 0) || ($> != 0)) )
{
  print "Must be root to run PDInfo.\n";
  die;
}

#### Load the config file if it exists
if(-e $CONFIG_FILE)
{
  do "$CONFIG_FILE";
}
else
{
  print "Unable to locate config file ($CONFIG_FILE).\n";
  die;
}

#Get the name of the file they want as output
if($^O eq "MSWin32")
{
  print("Please enter the name (full path) of the desired output file eg. c:\\tmp\\output.tar: ");
}
else
{
  print("Please enter the name (full path) of the desired output file eg. /tmp/output.tar.gz: ");
}

chomp($output_file = <STDIN>);

#We need to check to see if the directory they are putting it in exists
#So we need to check to see if it is a full path
$slash_pos = rindex $output_file,"\\";
#if there were no normal Windows slashes, check for unix ones
if ( $slash_pos eq -1 )
{
  $slash_pos = rindex $output_file,"/";
}

#Now go and check to see if the directory exists
if ( !($slash_pos eq -1) )
{
  $OUTPUT_DIR = substr $output_file, 0, $slash_pos;
  if (! (-e $OUTPUT_DIR) )
  {
    print("Output directory must exist.\n");
    die;
  }
}

# ALWAYS write to the current directory, then move to the final destination
# after tar etc.
$OUTPUT_DIR = "./pdinfo_output";


if($^O eq "MSWin32")
{

  #Replace all the wrong slashes with the right ones
  $output_file =~ s/\//\\/g;
  $OUTPUT_DIR  =~ s/\//\\/g;
}

print("Starting Information Gathering..\n");
### Rename any existing output
if(-e $OUTPUT_DIR.old)
{
  system("rm -rf '$OUTPUT_DIR.old' 1>$NULL_DEVICE 2>&1");
}
if(-e $OUTPUT_DIR)
{
  if($^O eq "MSWin32")
  {
    system("move '$OUTPUT_DIR' '$OUTPUT_DIR.old' 1>$NULL_DEVICE 2>&1");
  }
  else
  {
    system("mv '$OUTPUT_DIR' '$OUTPUT_DIR.old' 1>$NULL_DEVICE 2>&1");
  }
}

system("mkdir $OUTPUT_DIR 1>$NULL_DEVICE 2>&1");

#### Start gathering system information
if($GATHER_MACHINE_INFO)
{
  writeToLog("Machine Info\n");
  writeToLog("================================================\n");
  writeToLog("Name\n");
  writeToLog("---------------\n");
  machineName();
  writeToLog(" \n");

  writeToLog("O/S\n");
  writeToLog("---------------\n");
  OS();
  writeToLog(" \n");

  writeToLog("Architecture\n");
  writeToLog("---------------\n");
  architecture();
  writeToLog(" \n");

  writeToLog("Processors\n");
  writeToLog("---------------\n");
  processors();
  writeToLog(" \n");

  writeToLog("RAM\n");
  writeToLog("---------------\n");
  RAM();
  writeToLog(" \n");

  writeToLog("Virtual Memory Stats\n");
  writeToLog("---------------\n");
  vmstat();
  writeToLog(" \n");

  writeToLog("Disk Usage\n");
  writeToLog("---------------\n");
  HDD();
  writeToLog(" \n");

  writeToLog("System Uptime\n");
  writeToLog("---------------\n");
  uptime();
  writeToLog(" \n");
}

if($GATHER_NETWORK_INFO)
{
  writeToLog("Network Info\n");
  writeToLog("================================================\n");
  writeToLog("Interfaces\n");
  writeToLog("---------------\n");
  interfaces();
  writeToLog(" \n");

  writeToLog("ARP\n");
  writeToLog("---------------\n");
  arp();
  writeToLog(" \n");

  writeToLog("Routing\n");
  writeToLog("---------------\n");
  routing();
  writeToLog(" \n");

  writeToLog("Sockets\n");
  writeToLog("---------------\n");
  sockets();
  writeToLog(" \n");

  writeToLog("Per-protocol stats: netstat -s");
  writeToLog("---------------\n");
  protstats();

  writeToLog("STREAMS stats: netstat -m\n");
  writeToLog("---------------\n");
  streamstats();

  if ($^O eq "aix")
  {
    writeToLog("AIX: netstat -v");
    writeToLog("---------------\n");
    vstats();
  }
}

if($GATHER_PROCESS_INFO)
{
  writeToLog("Current Processes\n");
  writeToLog("================================================\n");
  processes();
  writeToLog(" \n");
}

if($GATHER_INSTALLED_INFO)
{
  writeToLog("Installed Software\n");
  writeToLog("================================================\n");
  installedSoftware();
  writeToLog(" \n");
}


if($GATHER_PD_INFO)
{
  writeToLog("PD Related Information\n");
  writeToLog("================================================\n");

  writeToLog("Servers Status\n");
  writeToLog("---------------\n");
  pdstatus();

  writeToLog(" \n");
}

if($EXECUTE_BLADES)
{ 
  writeToLog("Executing Blades\n");
  writeToLog("================================================\n");
  print("Executing PDRTE Blade\n");
  copyTraceAuditFiles();
  copyLogFiles();
  copyDBFiles();
  copyConfFiles();
  genAndCopyCoreFiles();

  executeBlades();
}

tarAndGzipOutput();

print("...Information Gathering Complete.\n");


#####################################################################
sub writeToLog
{
  open FH, ">>$OUTPUT_DIR/$OUTPUT_FILE";
  foreach $stringToWrite (@_)
  {    
    print FH $stringToWrite 
  }
  close FH;
}

sub machineName
{
  #If it is windows, we use hostname instead
  if($^O eq "MSWin32")
  {
    system("hostname  2>> \"$OUTPUT_DIR\\$ERROR_FILE\" >> \"$OUTPUT_DIR\\$OUTPUT_FILE\"");
  }
  else
  {
    system("uname -n  2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub OS
{
  if($^O eq "MSWin32")
  {
    @osVersion = Win32::GetOSVersion;
    if ($osVersion[4] eq "0")
    {
      writeToLog("Win32 Version: $osVersion[1].$osVersion[2] Build: $osVersion[3]\n");       
    }
    elsif ($osVersion[4] eq "1")
    {
      writeToLog("Windows9X Version: $osVersion[1].$osVersion[2] Build: $osVersion[3]\n");
    }
    else
    {
       writeToLog("WindowsNT Version: $osVersion[1].$osVersion[2] Build: $osVersion[3]\n");
    }
  }
  elsif($^O eq "solaris")
  {
     system("uname -srv 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
     writeToLog("OS Patches\n");
     writeToLog("---------------\n");
     system("showrev -p 2>>$OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  else
  {
    system("uname -srv 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }  
}


sub architecture
{
  #If it is windows use $ENV{PROCESSOR_ARCHITECTURE}
  if($^O eq "MSWin32")
  {
    writeToLog("$ENV{PROCESSOR_ARCHITECTURE}\n");
  }
  elsif ($^O eq "solaris" || $^O eq "sunos")
  {
    system("uname -ipm 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  elsif ($^O eq "aix" || $^O eq "sunos")
  {
    system("uname -M 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  else #HP-UX and linux
  {
    system("uname -m 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}

sub processors
{
  if($^O eq "MSWin32")
  {
    #Get the details of all processors    
    writeToLog(pdinfoSysCalls::getProcessors());
  }
  elsif ($^O eq "solaris" || $^O eq "sunos")
  {

	# sun4u machines have /usr/platform/sun4u/prtdiag, which gives the
	# information we're after here.
	if (`uname -m` eq "sun4u") {
		system("/usr/platform/sun4u/sbin/prtdiag -v 2>>$OUTPUT_DIR/$ERROR_FILE >>$OUTPUT_DIR/$OUTPUT_FILE");
	} else {
		# Can't rely on prtdiag existing.  If Workshop is installed,
		# /opt/SUNWspro/bin/fpversion should exist, and we can use that.
		if (-e "/opt/SUNWspro/bin/fpversion") {
			system("/opt/SUNWspro/bin/fpversion 2>>$OUTPUT_DIR/$ERROR_FILE >>$OUTPUT_DIR/$OUTPUT_FILE");
		} else {
			# Don't know any other way to find this information.
			writeToLog("Processor information is not available on this architecture\n");
		}
	}
  }
  elsif ($^O eq "aix" || $^O eq "sunos")
  {
    system("lscfg -vp 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  elsif ($^O eq "linux")
  {
    system("cat /proc/cpuinfo 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  elsif ($^O eq "hpux")
  {
    writeToLog(pdinfoSysCalls::getProcessorCount());
  }
}

sub RAM
{
  if($^O eq "MSWin32")
  {
    writeToLog(pdinfoSysCalls::getMemoryInfo());
    writeToLog("\n");
  }
  elsif ($^O eq "solaris" || $^O eq "sunos")
  {    
    writeToLog("RAM:\n");
    writeToLog("  Free:\n");
    writeToLog(pdinfoSysCalls::getFreeMemory());
    writeToLog("  Total:\n");
    writeToLog(pdinfoSysCalls::getTotalMemory());
    writeToLog("VM:\n");
    system("swap -s 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  elsif ($^O eq "aix" || $^O eq "sunos")
  {
    #If vmstat exists, use that, otherwise just use lsps -a
    writeToLog("VM:\n");
    if(-e "/usr/bin/vmstat")
    {
      system("pstat -s 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
    }
    else
    {
      system("lsps -a 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
    }
    writeToLog("RAM (kB):\n");
    system("bootinfo -r  2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
    writeToLog("\n");
  }
  elsif ($^O eq "linux")
  {
    writeToLog("VM Stats:\n");
    system("vmstat 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
    writeToLog("Physical Available:\n");
    writeToLog(pdinfoSysCalls::getFreeMemory());
    writeToLog("Physical Total:");
    writeToLog(pdinfoSysCalls::getTotalMemory());
    writeToLog("Virtual Available:");
    writeToLog(pdinfoSysCalls::getFreeVM());
    writeToLog("Virtual Total:");
    writeToLog(pdinfoSysCalls::getTotalVM());
  }
  elsif ($^O eq "hpux")
  {
    writeToLog("VM in 4096b pages:\n");
    system("svmon 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
    writeToLog("Free Memory Available:\n");
    writeToLog(pdinfoSysCalls::getFreeMemory());
    writeToLog("Total Memory Available:\n");
    writeToLog(pdinfoSysCalls::getTotalMemory());
  }
}

sub uptime
{
  if($^O eq "MSWin32")
  {
    writeToLog("Feature not available on Windows.\n");
  }
  else #HP-UX and linux
  {
    system("uptime 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}

sub vmstat
{
  if ($^O eq "aix")
  {
   writeToLog("See RAM section.\n");
  }
  elsif ($^O eq "MSWin32")
  {
    writeToLog("See: Ram\n");
  }
  else
  {
    system("vmstat 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub interfaces
{
  if($^O eq "MSWin32")
  {
    system("ipconfig /all 2>> \"$OUTPUT_DIR\\$ERROR_FILE\" >> \"$OUTPUT_DIR\\$OUTPUT_FILE\""); 
  }
  else
  {
    system("netstat -in 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub arp
{
  if ($^O eq "MSWin32") {
    system("arp -a 2>> \"$OUTPUT_DIR\\$ERROR_FILE\" >> \"$OUTPUT_DIR\\$OUTPUT_FILE\""); 
  } elsif (($^O eq "solaris") || ($^O eq "sunos")) {
    system("arp -a 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE"); 
  } else {
      system("arp -an 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub routing
{
  if ($^O eq "MSWin32") {
    system("netstat -rn  2>> \"$OUTPUT_DIR\\$ERROR_FILE\" >> \"$OUTPUT_DIR\\$OUTPUT_FILE\"");
  } else {
    system("netstat -rn  2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub protstats
{
  if ($^O eq "MSWin32") {
    system("netstat -s 2>> \"$OUTPUT_DIR\\$ERROR_FILE\" >> \"$OUTPUT_DIR\\$OUTPUT_FILE\"");
  } else {
    system("netstat -s 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub streamstats
{
  if( $^O eq "MSWin32")
  {
    writeToLog("Unavailable on Windows\n");
  }
  elsif( $^O eq "linux")
  {
      system("netstat -v 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  else
  {
    system("netstat -m 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub vstats
{
  if( $^O eq "MSWin32") {
    system("netstat -v 2>> \"$OUTPUT_DIR\\$ERROR_FILE\" >> \"$OUTPUT_DIR\\$OUTPUT_FILE\"");
  } else {
    system("netstat -v 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub processes
{
  if($^O eq "MSWin32")
  {
    @version = Win32::GetOSVersion();
    if(! ($version[1] eq "4") )
    {
      writeToLog(pdinfoSysCalls::getProcesses()); 
    }
    # Can't get a system snapshot in windows NT - only 2000.
  }
  else
  {
    system("ps -elf 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub HDD
{
  if($^O eq "MSWin32")
  {
    writeToLog(pdinfoSysCalls::getDiskSpace());
  }
  else
  {
    system("df -k 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub installedSoftware
{
  if($^O eq "MSWin32")
  {
    writeToLog(pdinfoSysCalls::getInstalledSW());
  }
  elsif ($^O eq "solaris" || $^O eq "sunos")
  {
    system("pkginfo -l 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  elsif ($^O eq "aix" || $^O eq "sunos")
  {
    system("lslpp -L 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  elsif ($^O eq "linux")
  {
    system("rpm -qa 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  elsif ($^O eq "hpux")
  {
    system("swlist 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub sockets
{
  if($^O eq "MSWin32") {
    system("netstat -a 2>> \"$OUTPUT_DIR\\$ERROR_FILE\" >> \"$OUTPUT_DIR\\$OUTPUT_FILE\"");
  } else {
    system("netstat -a 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub pdstatus
{
  if($^O eq "MSWin32")
  {
    writeToLog("View active processes to determine if servers are running.\n");
  }
  elsif ($^O eq "solaris" || $^O eq "sunos")
  {
    system("/etc/init.d/iv status 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  elsif ($^O eq "aix" || $^O eq "sunos")
  {
    system("/opt/PolicyDirector/bin/pd_start status 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
  elsif ($^O eq "linux")
  {
    writeToLog("Currently not required as there are no servers for linux!\n");
  }
  elsif ($^O eq "hpux")
  {
    system("/sbin/init.d/iv status 2>> $OUTPUT_DIR/$ERROR_FILE >> $OUTPUT_DIR/$OUTPUT_FILE");
  }
}


sub executeBlades
{
  opendir (DIR, $PDINFO_LOC) or print "Cannot open $PDINFO_LOC directory.";
  foreach my $file (readdir(DIR))
  {
    #We don't want to execute this script again, or any directories
    if( !($file eq ".") && !($file eq "..") && !($file eq "pdinfo_pdrte_blade.pl") && !($file eq "pdinfo.cfg") && !($file eq ""))
    {
      writeToLog("Executing $file\n");
      writeToLog("---------------\n");
      do "$PDINFO_LOC/$file";
    }
  }
  closedir (DIR);
}

sub tarAndGzipOutput
{

  if($^O eq "MSWin32")
  {
    # gzip function doesn't work on windows
    pdinfoSysCalls::tarFile("$OUTPUT_DIR");
    system("move \"$OUTPUT_DIR.tar\" \"$output_file\" 1>$NULL_DEVICE 2>&1");
    system("rm -rf $OUTPUT_DIR 1>$NULL_DEVICE 2>&1");
  }
  else
  {
    pdinfoSysCalls::tarFile("$OUTPUT_DIR");
    pdinfoSysCalls::gzipFile("$OUTPUT_DIR.tar");
    system("mv $OUTPUT_DIR.tar.gz $output_file 1>$NULL_DEVICE 2>&1");
    system("rm -rf $OUTPUT_DIR $OUTPUT_DIR.tar 1>$NULL_DEVICE 2>&1");
  }
}

##############################################################################################################
#PDRTE Blade stuff#

sub copyTraceAuditFiles
{
  if($^O eq "MSWin32")
  {
    system("mkdir \"$OUTPUT_DIR\\trace+audit\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\audit\" \"$OUTPUT_DIR\\trace+audit\" 1>$NULL_DEVICE 2>&1");
  }
  else
  {
    system("mkdir $OUTPUT_DIR/trace+audit 1>$NULL_DEVICE 2>&1"); 
    system("cp -r /var/PolicyDirector/audit/ $OUTPUT_DIR/trace+audit/ 1>$NULL_DEVICE 2>&1");
  }
}

sub copyLogFiles
{
  if($^O eq "MSWin32")
  {
    system("mkdir \"$OUTPUT_DIR\\log\" 2>> \"$OUTPUT_DIR\\$ERROR_FILE\" 1>$NULL_DEVICE 2>&1");
    system("mkdir \"$OUTPUT_DIR\\log\\wwwlog\" 2>> \"$OUTPUT_DIR\\$ERROR_FILE\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\log\\error.log\" \"$OUTPUT_DIR\\log\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\log\\fatal.log\" \"$OUTPUT_DIR\\log\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\log\\notice.log*\" \"$OUTPUT_DIR\\log\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\log\\warning.log\" \"$OUTPUT_DIR\\log\" 1>$NULL_DEVICE 2>&1");
  }
  else
  {
    system("mkdir $OUTPUT_DIR/log 1>$NULL_DEVICE 2>&1");
    system("mkdir $OUTPUT_DIR/log/wwwlog 1>$NULL_DEVICE 2>&1");
    system("cp /var/PolicyDirector/log/error.log $OUTPUT_DIR/log 1>$NULL_DEVICE 2>&1");
    system("cp /var/PolicyDirector/log/fatal.log $OUTPUT_DIR/log 1>$NULL_DEVICE 2>&1");
    system("cp /var/PolicyDirector/log/notice.log* $OUTPUT_DIR/log 1>$NULL_DEVICE 2>&1");
    system("cp /var/PolicyDirector/log/warning.log $OUTPUT_DIR/log 1>$NULL_DEVICE 2>&1");
  }
}

sub copyDBFiles()
{
  if($^O eq "MSWin32")
  {
    system("mkdir \"$OUTPUT_DIR\\db\" 1>$NULL_DEVICE 2>&1");
	system("mkdir \"$OUTPUT_DIR\\db\\junction\" 1>$NULL_DEVICE 2>&1");
  }
  else
  {
    system("mkdir $OUTPUT_DIR/db 1>$NULL_DEVICE 2>&1");
	system("mkdir $OUTPUT_DIR/db/junction 1>$NULL_DEVICE 2>&1");
  }
}

sub copyConfFiles()
{
  if($^O eq "MSWin32")
  {
    system("mkdir \"$OUTPUT_DIR\\conf\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\etc\\pd.conf\" \"$OUTPUT_DIR\\conf\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\etc\\migrate.conf\" \"$OUTPUT_DIR\\conf\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\etc\\ldap.conf\" \"$OUTPUT_DIR\\conf\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\etc\\domino.conf\" \"$OUTPUT_DIR\\conf\" 1>$NULL_DEVICE 2>&1");
    system("copy \"$PDBASE_VAR\\etc\\routing\" \"$OUTPUT_DIR\\conf\" 1>$NULL_DEVICE 2>&1");
  }
  else
  {
    system("mkdir $OUTPUT_DIR/conf 1>$NULL_DEVICE 2>&1");
    system("cp /opt/PolicyDirector/etc/pd.conf $OUTPUT_DIR/conf 1>$NULL_DEVICE 2>&1");
    system("cp /opt/PolicyDirector/etc/migrate.conf $OUTPUT_DIR/conf 1>$NULL_DEVICE 2>&1");
    system("cp /opt/PolicyDirector/etc/ldap.conf $OUTPUT_DIR/conf 1>$NULL_DEVICE 2>&1");
    system("cp /opt/PolicyDirector/etc/domino.conf $OUTPUT_DIR/conf 1>$NULL_DEVICE 2>&1");
    system("cp /opt/PolicyDirector/etc/routing $OUTPUT_DIR/conf 1>$NULL_DEVICE 2>&1");
  }
}

sub genAndCopyCoreFiles()
{
  if(! ($^O eq "MSWin32") )
  {
    system("mkdir $OUTPUT_DIR/core 1>$NULL_DEVICE 2>&1");
  }
}
