#ifndef lint
static char sccsid[]="@(#)89 1.4 src/ivmgrd/pdadminapi/pdadminapi_demo/pdadminapi_demo.c, pd.admin, pd390, 020405b 10/31/01 10:43:27";
#endif
/*
 * Component Name: ivadmin adk
 * File Name: ivadminapi_demo.c
 *
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ivadminapi.h"



/*
 *
 * This program demonstrates the use of the ivadminapi for various
 * management tasks
 *
 */

/*
 * Global constants
 * These should be changed to match your environment before compiling and
 * running the program
 */
char *sec_masterpwd = "password";     /* the password for the sec_master user */
char *ldap_suffix = "o=tivoli,c=us";  /* the LDAP suffix for Policy Director users */

/*****************************************************************************
 * Prints the response text from an ivadmin API command.
 *****************************************************************************/

void printResponse(ivadmin_response rsp, char *api)
{
   unsigned long i=0;

   if (rsp == NULL) {
      fprintf(stderr,"   %s : failed\n", api);
   }

   if (ivadmin_response_getok(rsp)) {
      fprintf(stderr,"   %s : succeeded\n", api);
   } else {
      for (i=0; i<ivadmin_response_getcount(rsp); i++) {
         fprintf(stderr,"   %s : %s\n", api, ivadmin_response_getmessage(rsp, i));
      }
   }
}

int main(int argc, char ** argv)
{
    int success = 1;
    unsigned long status;
    ivadmin_context ctx;
    ivadmin_response rsp;
    const char *username = "testuser";
    char *userdn;
    char *groupname = "testgroup";
    char *groupdn;

    /*
     * Before doing and management operations it is necessary to create
     * a context for communication with the Policy Director Management server
     */
    printf("Creating context for communication with the Policy Director Management Server\n");
    status = ivadmin_context_createdefault("sec_master", sec_masterpwd, &ctx, &rsp);
    if (status != IVADMIN_TRUE) {
        printResponse(rsp, "ivadmin_context_createdefault");
        return 0; /* The context create call failed so we should just exit */
    }
    ivadmin_free(rsp);

    /*
     * Create a Policy Director User
     */
    userdn = (char *)malloc(strlen(ldap_suffix) + strlen(username) + 5);
    sprintf(userdn, "cn=%s,%s", username, ldap_suffix);
    printf("Creating Policy Director user: %s\n", username);
    status = ivadmin_user_create3(ctx, username, userdn,
                                  username, username, "ChangeMe1", 0, NULL,
                                  IVADMIN_FALSE, IVADMIN_FALSE, &rsp);
    if (status != IVADMIN_TRUE) {
        printResponse(rsp, "ivadmin_user_create3");
    }
    free(userdn);
    ivadmin_free(rsp);

    /*
     * Set the account to valid for the new user
     */
    printf("Setting the account to valid for user: %s\n", username);
    status = ivadmin_user_setaccountvalid(ctx, username, IVADMIN_TRUE, &rsp);
    if (status != IVADMIN_TRUE) {
        printResponse(rsp, "ivadmin_user_setaccountvalid");
    }
    ivadmin_free(rsp);

    /*
     * Change the password of the new user
     */
    printf("Changing the password for user: %s\n", username);
    status = ivadmin_user_setpassword(ctx, username, "ChangeMe2", &rsp);
    if (status != IVADMIN_TRUE) {
        printResponse(rsp, "ivadmin_user_setpassword");
    }
    ivadmin_free(rsp);

    /*
     * Create a new group
     */
    groupdn = (char *)malloc(strlen(ldap_suffix) + strlen(groupname) + 5);
    sprintf(groupdn, "cn=%s,%s", groupname, ldap_suffix);
    printf("Creating a new group: %s\n", groupname);
    status = ivadmin_group_create2(ctx, groupname, groupdn, groupname, NULL, &rsp);
    if (status != IVADMIN_TRUE) {
        printResponse(rsp, "ivadmin_group_create2");
    }
    free(groupdn);
    ivadmin_free(rsp);

    /*
     * Add the new user to the new group
     */
    printf("Adding user: %s to group: %s\n", username, groupname);

    status = ivadmin_group_addmembers(ctx, groupname, 1, &username, &rsp);
    if (status != IVADMIN_TRUE) {
        printResponse(rsp, "ivadmin_group_addmembers");
    }
    ivadmin_free(rsp);

    /*
     * Delete the group
     */
    printf("Deleting group: %s\n", groupname);
    status = ivadmin_group_delete2(ctx, groupname, IVADMIN_TRUE, &rsp);
    if (status != IVADMIN_TRUE) {
        printResponse(rsp, "ivadmin_group_delete2");
    }
    ivadmin_free(rsp);

    /*
     * Delete the user
     */
    printf("Deleting user: %s\n", username);
    status = ivadmin_user_delete2(ctx, username, IVADMIN_TRUE, &rsp);
    if (status != IVADMIN_TRUE) {
        printResponse(rsp, "ivadmin_user_delete2");
    }
    ivadmin_free(rsp);

    /*
     * Delete the context
     */
    printf("Deleting the context to the Policy Director Management Server\n");
    status = ivadmin_context_delete(ctx, &rsp);
    if (status != IVADMIN_TRUE) {
        printResponse(rsp, "ivadmin_context_delete");
    }
    ivadmin_free(rsp);
    ivadmin_free(ctx);


    return 1;
}





