/*
 * 5 95 1.5 src/com/tivoli/pd/jadmin/demo/PDAdminDemo.java, pd.jadmin, pd390, 020405b 2/25/02 16:50:30
 *
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001, 2002
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * File Name: PDAdminDemo.java
 */


import java.util.*;
import java.net.URL;
import java.io.*;

import com.tivoli.pd.jutil.PDContext;
import com.tivoli.pd.jutil.PDMessage;
import com.tivoli.pd.jutil.PDMessages;
import com.tivoli.pd.jutil.PDRgyUserName;
import com.tivoli.pd.jutil.PDRgyGroupName;
import com.tivoli.pd.jutil.PDException;

import com.tivoli.pd.jadmin.*;

import com.tivoli.pd.nls.pdbjamsg;

/**
 * This class demonstrates the use of the Java PDAdmin API classes.
 *
 */

public class PDAdminDemo
{
    private static final String sCodeID = "$Id: @(#)95  1.5 src/com/tivoli/pd/jadmin/demo/PDAdminDemo.java, pd.jadmin, pd390, 020405b 02/02/25 16:50:30 $";
    public static final String COPYRIGHT =
   "Licensed Materials - Property of IBM\n" +
   "5748-XX8\n" +
   "(c) Copyright International Business Machines Corp. 2001, 2002\n" +
   "All Rights Reserved.\n"
   +
   "US Government Users Restricted Rights - Use, duplication or disclosure\n"
    +
   "restricted by GSA ADP Schedule Contract with IBM Corp.\n";


   static class ConsoleEraser extends Thread 
   {  
      private boolean running = true;

      public void setRunning(boolean r)
      {
         running = r;
      }
      public void run() 
      {
         while (running) 
         {
            System.out.print("\b ");
         }
      }
   }


   static void processMsgs(PDMessages msgs)
   {
      if (msgs.size() > 0)
      {
         System.out.println("Msgs are: " + msgs + "\n");
         msgs.clear();
      }
   }

   static void handlePDException(Exception e)
   {
      PDException pd = (PDException)e;
      PDMessages msgs = pd.getMessages();
      if (msgs != null)
      {
         Iterator pdi = msgs.iterator();
         PDMessage msg = null;
         int msgCode = 0;
         while (pdi.hasNext())
         {
            msg = (PDMessage)pdi.next();
            msgCode = msg.getMsgCode();
            /*---------------------------------------------------------------
             * Here are examples of generic Policy Director message codes.  
             * They are available by importing the com.tivoli.pd.nls.pdbjamsg
             * class. These message codes are documented in the 
             * Error Message Reference. 
             *---------------------------------------------------------------
             */  
            switch (msgCode)
            {
               case pdbjamsg.bja_invalid_msgs:
                  System.out.println ("*** Invalid PDMessage Parameter ***");
                  System.out.println ("Message text is: " + msg.getMsgText() + "\n");
                  break;
               case pdbjamsg.bja_invalid_ctxt:
                  System.out.println ("*** Invalid Context Parameter ***");
                  System.out.println ("Message text is: " + msg.getMsgText() + "\n");
                  break;
               case pdbjamsg.bja_cannot_contact_server:
                  System.out.println ("*** The Server cannot be contacted ***");
                  System.out.println ("Message text is: " + msg.getMsgText() + "\n");
                  break;
               default:
                  break;
            }
         }
      }
   }
   
   public static void main(String [] args)
      throws IOException
   {
      String prog = "PDAdminDemo";

      /*------------------------------------------------------------------
       * A PDMessages instance is passed as an output parameter in
       * PDAdmin API functions, and can also be returned as part of a
       * PDException.  The PDMessages returned in the output parameter
       * as a result of a PDAdmin API call can contain warning and 
       * informational messages, whereas the PDMessages returned in a 
       * PDException can contain warning, informational, and error messages.
       *------------------------------------------------------------------
       */

      PDMessages msgs = new PDMessages();


     /*------------------------------------------------------------------
      * Parse the command-line arguments.  Three arguments are required
      * and an additional argument is required for installations using
      * LDAP as a user registry: 
      *
      * adminUser	a Policy Director user with administrative
      *			capabilities (for example, sec_master).
      * configURL	a URL for the configuration file created by the
      *			SvrSslCfg configuration utility.  This URL must
      *			specify the file: protocol (for example,
      *			file:///C:/tmp/pd/pdadmin.properties).
      * rgySuffix	(LDAP installations only) the LDAP registry suffix
      *			under which Policy Director users and groups will
      *			be created (for example, o=tivoli,c=us).
      *------------------------------------------------------------------
      */

      if ((args.length != 2) && (args.length != 3)) {
          System.out.println("Usage:");
          System.out.println("\tPDAdminDemo <adminUser> <configURL> [<rgySuffix>]");
          System.out.println("where");
          System.out.println("\t<adminUser>\ta Policy Director user with administrative");
          System.out.println("\t           \tcapabilities (for example, sec_master).");
          System.out.println("\t<configURL>\ta URL for the configuration file created by the");
          System.out.println("\t           \tSvrSslCfg configuration utility.  This URL must");
          System.out.println("\t           \tspecify the file: protocol (for example,");
          System.out.println("\t           \tfile:///C:/tmp/pd/pdadmin.properties).");
          System.out.println("\t<rgySuffix>\t(LDAP installations only) the LDAP registry suffix");
          System.out.println("\t           \tunder which Policy Director users and groups will");
          System.out.println("\t           \tbe created (for example, o=tivoli,c=us).");
          System.exit(1);
      }

      String adminName = args[0];
      String configURLStr = args[1];

      String rgySuffix = "o=tivoli,c=us";
      if (args.length == 3) 
      {
          rgySuffix = args[2];
      }

      Locale locale = new Locale("ENGLISH", "US");
      URL configURL = null;


      /*------------------------------------------------------------------
       * Create a ConsoleEraser while prompting for the password so that
       * that characters do not echo to the console.
       *------------------------------------------------------------------
       */ 
      ConsoleEraser consoleEraser = new ConsoleEraser();
      System.out.print("Password: ");
      BufferedReader stdin = 
         new BufferedReader(new InputStreamReader(System.in));
      consoleEraser.start();                       
      char [] adminPassword = stdin.readLine().toCharArray();
      consoleEraser.setRunning(false);
      System.out.print("\n");

      try 
      {  

         /*------------------------------------------------------------------
          * Initialize the PDAdmin API.  This initialization is required
          * before the PDAdmin API can be used.
          *------------------------------------------------------------------
          */ 
         System.out.println("Initializing PDAdmin...\n");
         PDAdmin.initialize(prog, msgs);
         processMsgs(msgs);


         configURL = new URL(configURLStr);

         /*------------------------------------------------------------------
          * Create a PDContext object.  The PDContext object is required for
          * all subsequent PDAdmin API calls which interface to the
          * Management Server.
          *------------------------------------------------------------------
          */
         System.out.println("Creating a context...\n");
         PDContext ctxt = new PDContext(locale, 
                                        adminName, 
                                        adminPassword, 
                                        configURL);

         /*------------------------------------------------------------------
          * Clear out the password since it's no longer needed 
          *------------------------------------------------------------------
          */
          for (int i = 0; i < adminPassword.length; i++) {
              adminPassword[i] = '\0';
          }

         /*------------------------------------------------------------------
          * Create a group, using the PDGroup.createGroup() static method.  
          * This will send a request to the Management Server to add the 
          * group.  All administration object creation is accomplished via 
          * static create methods, which always create these objects at the 
          * Management Server.
          *------------------------------------------------------------------
          */
         String group = "TestGroup1";
         String rgyGroup = "cn=" + group + "," + rgySuffix; 
         PDRgyGroupName pdRgyGroupName = new PDRgyGroupName(rgyGroup);

         System.out.println("Creating a group...\n");
         PDGroup.createGroup(ctxt,
                             group,
                             pdRgyGroupName,
                             null, // description
                             null, // container
                             msgs);
         processMsgs(msgs);

         /*------------------------------------------------------------------
          * Create a user, using the PDUser.createUser() static method, and 
          * assign the user to the group created above.  This will send a 
          * request to the Management Server to create the user.  All 
          * administration object creation is accomplished via static create
          * methods, which always attempt to create these objects at the
          * Management Server.
          *------------------------------------------------------------------
          */

         // Set up all of the user's attributes
         String name = "DemoUser";
         String firstName = "Demo";
         String lastName = "User";
         String password = "DemoPassword";
         String description = "Demo Description";
         String rgyName = "cn=" + name + "," + rgySuffix;
         PDRgyUserName pdRgyUserName = 
            new PDRgyUserName(rgyName, firstName, lastName);
         boolean ssoUser = false;
         boolean pwdPolicy = true;
         ArrayList groupList = new ArrayList();
         groupList.add(group);

         System.out.println("Creating a user...\n");
         PDUser.createUser(ctxt,
                           name,
                           pdRgyUserName,
                           description,
                           password.toCharArray(),
                           groupList,                     
                           ssoUser,
                           pwdPolicy,
                           msgs);


         /*------------------------------------------------------------------
          * The preceding createUser call is an example of a successful 
          * Management Server call that returns informational or warning 
          * messages.  "Description" is not yet a supported parameter, so 
          * the API places a message to that effect in msgs.  An attempt 
          * should be made after every PDAdmin API call to handle any msgs 
          * that may have been returned, regardless of success or failure.
          *------------------------------------------------------------------
          */
         processMsgs(msgs);


         /*------------------------------------------------------------------
          * Get all of the user's data from the Management Server.  To get an
          * object's data from the Management Server, call the constructor 
          * representing the class of the object, and pass it the name of 
          * the object's instance (supplied in the "name" parameter during 
          * creation).  This fetches the data for the object from the 
          * Management Server and places it in a local instance.
          *------------------------------------------------------------------
          */
         System.out.println("Getting a user...\n");
         PDUser pdUser = new PDUser(ctxt,
                                    name,
                                    msgs);


         processMsgs(msgs);

         /*------------------------------------------------------------------
          * Print the isAccountValid attribute of the user.  This calls
          * a local instance method, which gets the attribute from the 
          * local copy of the object, not from the Management Server.  
          * All "gets" operate on local object copies.  To get an object's 
          * data from the Management Server, use the object constructor.
          *------------------------------------------------------------------
          */
         System.out.println("User " + name + ".isAccountValid: " + 
                            pdUser.isAccountValid() + "\n");


         /*------------------------------------------------------------------
          * Set the account to valid using an instance method.  All set
          * instance methods set both the attributes
          * in the local copy and those at the Management Server.
          *------------------------------------------------------------------
          */

         System.out.println("Setting the user's account to valid using the instance set method ...\n");
         pdUser.setAccountValid(ctxt, true, msgs);
         processMsgs(msgs);

         // Change the user's password
         String pwd = "New1Password";
         System.out.println("Changing the user's password using the instance set method ...\n");
         pdUser.setPassword(ctxt, pwd.toCharArray(), msgs);
         processMsgs(msgs);


         /*------------------------------------------------------------------
          * Get a fresh copy of this PDUser object from the Management Server,
          * and print the user's attributes.  This should show the 
          * isAccountValid attribute as having been set to true.
          *------------------------------------------------------------------
          */
         System.out.println("Getting the user...\n");
         pdUser = new PDUser(ctxt,
                             name,
                             msgs);
         processMsgs(msgs);
         System.out.println(pdUser);

         /*------------------------------------------------------------------
          * Set the account to invalid using a static method.  All set
          * static methods set the attributes at the Management Server.
          *------------------------------------------------------------------
          */

         System.out.println("Setting the user's account to invalid using the static set method ...\n");
         PDUser.setAccountValid(ctxt, name, false, msgs);
         processMsgs(msgs);


         /*------------------------------------------------------------------
          * Get a fresh copy of this PDUser object from the Management Server,
          * and print the user's attributes.  This should show the 
          * isAccountValid attribute as having been set to false.
          *------------------------------------------------------------------
          */
         System.out.println("Getting the user...\n");
         pdUser = new PDUser(ctxt,
                             name,
                             msgs);
         processMsgs(msgs);
         System.out.println("User " + name + ".isAccountValid: " + 
                            pdUser.isAccountValid() + "\n");


         /*------------------------------------------------------------------
          * Delete the group.  This will attempt to delete the group at the
          * Management Server.  Deletions of administration objects are 
          * accomplished using the static delete methods.
          *------------------------------------------------------------------
          */
         System.out.println("Deleting the group...\n");
         PDGroup.deleteGroup(ctxt, group, true, msgs); 
         processMsgs(msgs);


         /*------------------------------------------------------------------
          * Delete the user.  This will attempt to delete the user at the
          * Management Server.  Deletions of administration objects are 
          * accomplished using the static delete methods.
          *------------------------------------------------------------------
          */
         System.out.println("Deleting the user...\n");
         PDUser.deleteUser(ctxt, name, true, msgs);
         processMsgs(msgs);

         /*------------------------------------------------------------------
          * List the users.  This will get the list of users from the 
          * Management Server.  The listing of administration objects is 
          * accomplished using the static list methods.  The listing should
          * show that the user created in this program has been deleted.
          *------------------------------------------------------------------
          */

         System.out.println("Listing the users...\n");
         ArrayList alst = 
            PDUser.listUsers(ctxt, 
                             PDUser.PDUSER_ALLPATTERN,  // match all users
                             PDUser.PDUSER_MAXRETURN,  
                                                // maximum number to return
                             false,             // don't match by registry
                                                //    names, match by Policy
                                                //    Director names
                             msgs);

         processMsgs(msgs);
         System.out.println(alst);

         /*------------------------------------------------------------------
          * The next section intentionally creates an exception condition
          * to illustrate how the PDMessages contained in an exception can
          * be traversed to perform actions based on embedded reason codes
          * A null context parameter is passed to the PDUser constructor,
          * and handlePDException handles the resulting reason code.
          *------------------------------------------------------------------
          */
         try
         {
            PDUser dummyUser = new PDUser(null,
                                        "dummyUser",
                                        msgs);
            processMsgs(msgs);

         }
         catch (PDException e)
         {
            System.out.println("Expected exception caught:");
            handlePDException(e);        
         }      



         /*------------------------------------------------------------------
          * Shut down PDAdmin.  This call is required for proper cleanup
          * of the PDAdmin facility
          *------------------------------------------------------------------
          */
         System.out.println("Shutting down PDAdmin...\n");
         PDAdmin.shutdown(msgs);
         processMsgs(msgs);
      }
      catch (PDException e)
      {
         /*------------------------------------------------------------------
          * The PDAdmin API throws a PDException when it encounters an error.
          * PDException subclasses Exception, and contains a PDMessages member
          * variable.  PDException's printStackTrace() will print out any
          * existing PDMessages assigned to the variable.
          *------------------------------------------------------------------
          */
         e.printStackTrace();
         /*------------------------------------------------------------------
          * Loop through PDMessages to process specific reason codes.
          *------------------------------------------------------------------
          */
         handlePDException(e);
      }
      catch (Exception e)
      {
         System.out.println("*****Error in PDAdminDemo*****");
         /*------------------------------------------------------------------
          * Just print the stack trace for a non-Policy Director exception 
          *------------------------------------------------------------------
          */
         e.printStackTrace();
      }
   }
}
