/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef IVADMINAPI_H
#define IVADMINAPI_H

#if defined(WIN32) || defined(_WIN32)
#define IVADMIN_CALLTYPE       __cdecl
#if defined(IVADMINDLL)
#define IVADMIN_DECLSPEC       __declspec( dllexport )
#else
#if defined(IVADMIN_LINK_STATIC)
#define IVADMIN_DECLSPEC
#else
#define IVADMIN_DECLSPEC       __declspec( dllimport )
#endif /* IVADMIN_LINK_STATIC */
#endif /* IVADMINDLL */
#else /* WIN32 */
#define IVADMIN_CALLTYPE
#define IVADMIN_DECLSPEC
#endif /* WIN32 */

/*
   This now pre-reqs the AZN ADK.
*/
#include <ogauthzn.h>

/* basic return type */
/*
   taken out because of collisions on typedef.  bad idea anyways.
   typedef long unsigned unsigned32;
*/

/* boolean */
#define IVADMIN_FALSE     0
#define IVADMIN_TRUE      1

#define IVADMIN_AUDIT_NONE (0)
#define IVADMIN_AUDIT_PERMIT (1)
#define IVADMIN_AUDIT_DENY (2)
#define IVADMIN_AUDIT_ERROR (4)
#define IVADMIN_AUDIT_ADMIN (8)
#define IVADMIN_AUDIT_ALL (15)

#define IVADMIN_TIME_LOCAL (0)
#define IVADMIN_TIME_UTC (1)
#define IVADMIN_TOD_ANY (0)
#define IVADMIN_TOD_SUN (1)
#define IVADMIN_TOD_MON (2)
#define IVADMIN_TOD_TUE (4)
#define IVADMIN_TOD_WED (8)
#define IVADMIN_TOD_THU (16)
#define IVADMIN_TOD_FRI (32)
#define IVADMIN_TOD_SAT (64)
#define IVADMIN_TOD_ALL (127)
#define IVADMIN_TOD_WEEKDAY (62)
#define IVADMIN_TOD_WEEKEND (65)
#define IVADMIN_TOD_MINUTES (60)
#define IVADMIN_TOD_OCLOCK (3600)

#ifdef __cplusplus
    extern "C" {
#endif

typedef struct sivadmin_context* ivadmin_context;
typedef struct sivadmin_response* ivadmin_response;
typedef struct sivadmin_ldapgroup* ivadmin_ldapgroup;
typedef struct sivadmin_ldapuser* ivadmin_ldapuser;
typedef struct sivadmin_protobj* ivadmin_protobj;
typedef struct sivadmin_acl* ivadmin_acl;
typedef struct sivadmin_action* ivadmin_action;
typedef struct sivadmin_ssoweb* ivadmin_ssoweb;
typedef struct sivadmin_ssogroup* ivadmin_ssogroup;
typedef struct sivadmin_ssocred* ivadmin_ssocred;
typedef struct sivadmin_pop* ivadmin_pop;

/*
   General Notes

   This API set provides a set of functions for the administration
   of Policy Director objects.  The list of objects include:
      Users (in an LDAP registry)
      Groups (in an LDAP registry)
      Protected Objects
      Actions (Permissions)
      ACLs
      POPs
      SSO Web Resources
      SSO Group Resources
      SSO Credentials
      AZN Server

   For each object type a common set of routines exist to manage it.
   These typically include:  List, Create, Delete, Get, Set, and Accessors.
   The objects are presented in opaque structures and the necessary
   accessor functions are provided as well.  To see all of the user
   data for example, use the "list" function to get all of the user IDs,
   then get each user using the "get" function, and then use the "accessor"
   functions to see all of the data.

   In addition, two other types are defined.  First a context represents
   a connection to the management server and must be passed to any function
   that requires a remote connection.  Secondly a response type has been defined
   and it represents the success or failure result from the management server.

   All of the functions are completely re-entrant, however you cannot use
   a context on two different threads at the same time.  If you share a
   context across threads, you must serialize its use.

   All functions (other than accessors) return a success (IVADMIN_TRUE)
   or failure response (IVADMIN_FALSE).  If a failure has occurred the
   response structure will contain the error information.  Although usually
   programming errors, invalid Inputs, typically NULL pointer inputs, will
   generate a failure and appropriate response information.

   Memory allocation errors will result in NULL pointer outputs being
   returned as well as a failure return.  A NULL pointer will also be
   returned for the response structure because of the need to allocate them.
   A free routine is provided and should be used to free all memory
   allocated by these API's.  The memory management requirements are
   identified with each function description.

   Below is a list of each of the functions provided for each object type:


   Context

   The context represents a connection to the management server.
   This is done using Secure Sockets Layer.  The context functions
   are as follows:

   ivadmin_context_create             - Create a context.
   ivadmin_context_createdefault      - Create context using default SSL configuration.
   ivadmin_context_delete             - Delete the connection.
   ivadmin_context_setdelcred         - Set the context delegated credential.
   ivadmin_context_cleardelcred       - Clear the context delegated credential.
   ivadmin_context_getuserreg         - Get user registry type.
   ivadmin_context_setminpwdlen       - Set minimum password length.
   ivadmin_context_getminpwdlen       - Get minimum password length.
   ivadmin_context_setmaxpwdage       - Set maximum password age.
   ivadmin_context_getmaxpwdage       - Get maximum password age.
   ivadmin_context_setaccexpdate      - Set account expiration date.
   ivadmin_context_getaccexpdate      - Get account expiration date.
   ivadmin_context_setmaxlgnfails     - Set maximum number of failed logins allowed.
   ivadmin_context_getmaxlgnfails     - Get maximum number of failed logins allowed.
   ivadmin_context_setdisabletimeint  - Set disable time interval after failed logins exceeded.
   ivadmin_context_getdisabletimeint  - Get disable time interval after failed logins exceeded.
   ivadmin_context_setmaxpwdrepchars  - Set maximum repeated characters allowed.
   ivadmin_context_getmaxpwdrepchars  - Get maximum repeated characters allowed.
   ivadmin_context_setminpwdalphas    - Set minimum alphabetic characters allowed.
   ivadmin_context_getminpwdalphas    - Get minimum alphabetic characters allowed.
   ivadmin_context_setminpwdnonalphas - Set minimum non-alphabetic characters allowed.
   ivadmin_context_getminpwdnonalphas - Get minimum non-alphabetic characters allowed.
   ivadmin_context_setpwdspaces       - Set spaces allowed in passwords.
   ivadmin_context_getpwdspaces       - Get spaces allowed in passwords.
   ivadmin_context_settodaccess       - Set time of day access for accounts
   ivadmin_context_gettodaccess       - Get time of day access for accounts
   ivadmin_context_getmgmtsrvrhost    - Get the Management Server host name
   ivadmin_context_getmgmtsrvrport    - Get the Management Server TCP/IP port

   User

   Users are stored in an LDAP directory by the management server.
   DCE users are NOT administerable from these API's.  Each user
   is uniquely identified its ID.

   ivadmin_user_create3             - Create user.
   ivadmin_user_import2             - Import existing LDAP user.
   ivadmin_user_setdescription      - Set user description.
   ivadmin_user_setpassword         - Set user password.
   ivadmin_user_setauthmech         - Set user authentication mechanism.
   ivadmin_user_setaccountvalid     - Enable/disable account.
   ivadmin_user_setpasswordvalid    - Expire password.
   ivadmin_user_setssouser          - Enable/disable SSO capability.
   ivadmin_user_delete2             - Delete user.
   ivadmin_user_getmemberships      - Get groups in which user is a member.
   ivadmin_user_get                 - Get user.
   ivadmin_user_getbydn             - Get user using distinguished name.
   ivadmin_user_list                - List users.
   ivadmin_user_listbydn            - List users using distinguished names.
   ivadmin_user_getid               - Accessor to return user ID.
   ivadmin_user_getdescription      - Accessor to return user description.
   ivadmin_user_getauthmech         - Accessor to return user authentication mechanism.
   ivadmin_user_getaccountvalid     - Accessor to return user account enabled indicator.
   ivadmin_user_getpasswordvalid    - Accessor to return user password expired indicator.
   ivadmin_user_getssouser          - Accessor to return user SSO enabled indicator.
   ivadmin_user_getdn               - Accessor to return user LDAP distinguished name.
   ivadmin_user_getcn               - Accessor to return user LDAP common name attribute.
   ivadmin_user_getsn               - Accessor to return user LDAP surname attribute.
   ivadmin_user_setminpwdlen        - Set minimum password length.
   ivadmin_user_getminpwdlen        - Get minimum password length.
   ivadmin_user_setmaxpwdage        - Set maximum password age.
   ivadmin_user_getmaxpwdage        - Get maximum password age.
   ivadmin_user_setaccexpdate       - Set account expiration date.
   ivadmin_user_getaccexpdate       - Get account expiration date.
   ivadmin_user_setmaxlgnfails      - Set maximum number of failed logins allowed.
   ivadmin_user_getmaxlgnfails      - Get maximum number of failed logins allowed.
   ivadmin_user_setdisabletimeint   - Set disable time interval after failed logins exceeded.
   ivadmin_user_getdisabletimeint   - Get disable time interval after failed logins exceeded.
   ivadmin_user_setmaxpwdrepchars   - Set maximum repeated characters allowed.
   ivadmin_user_getmaxpwdrepchars   - Get maximum repeated characters allowed.
   ivadmin_user_setminpwdalphas     - Set minimum alphabetic characters allowed.
   ivadmin_user_getminpwdalphas     - Get minimum alphabetic characters allowed.
   ivadmin_user_setminpwdnonalphas  - Set minimum non-alphabetic characters allowed.
   ivadmin_user_getminpwdnonalphas  - Get minimum non-alphabetic characters allowed.
   ivadmin_user_setpwdspaces        - Set spaces allowed in passwords.
   ivadmin_user_getpwdspaces        - Get spaces allowed in passwords.
   ivadmin_user_settodaccess        - Set the time of day access for the account
   ivadmin_user_gettodaccess        - Get the time of day access for users




   Groups are stored in an LDAP directory by the management server.
   DCE groups are NOT administerable from these API's.  Each group
   is uniquely identified by its ID.

   ivadmin_group_create2            - Create group.
   ivadmin_group_import2            - Import existing LDAP group.
   ivadmin_group_setdescription     - Set group description.
   ivadmin_group_addmembers         - Add users to group.
   ivadmin_group_removemember       - Remove user from group.
   ivadmin_group_delete2            - Delete group.
   ivadmin_group_get                - Get group.
   ivadmin_group_getbydn            - Get group using distinguished name.
   ivadmin_group_list               - List groups.
   ivadmin_group_listbydn           - List groups using distinguished names.
   ivadmin_group_getmembers         - List members of the group.
   ivadmin_group_getid              - Accessor to return group ID.
   ivadmin_group_getdescription     - Accessor to return group description.
   ivadmin_group_getdn              - Accessor to return group LDAP distinguished name.
   ivadmin_group_getcn              - Accessor to return group LDAP common name attribute.


   Protected Object

   Protected objects are objects in the protected namespace
   that have an ACL attached to them.  Each is uniquely
   identified by its namespace ID.

   ivadmin_protobj_list2            - DEPRECATED.
   ivadmin_protobj_list3            - List all protected objects.
   ivadmin_protobj_listbyacl        - List all protected objects with specific attached ACL.
   ivadmin_protobj_get              - DEPRECATED.
   ivadmin_protobj_get2             - Get protected object.
   ivadmin_protobj_attachacl        - Attach ACL to protected object.
   ivadmin_protobj_detachacl        - Detach ACL from protected object.
   ivadmin_protobj_getid            - Accessor to return protected object ID.
   ivadmin_protobj_gettype          - Accessor to return protected object type.
   ivadmin_protobj_getacl           - Accessor to return protected object attached ACL.
   ivadmin_protobj_create           - Create a protected object
   ivadmin_protobj_delete           - Delete a protected object
   ivadmin_protobj_attr_put         - Put an extended attribute on an object
   ivadmin_protobj_attr_get         - Get an extended attribute from an object
   ivadmin_protobj_attr_delete_key  - Delete an extended attribute from an object
   ivadmin_protobj_attr_delete_val  - Delete a specific value from an extended attr on an object
   ivadmin_protobj_attr_list        - List all extended attributes for an object
   ivadmin_protobj_desc_modify      - Modify the description of the protected object
   ivadmin_protobj_name_modify      - Modify the name of the protected object
   ivadmin_protobj_ispolicyattachablemodify - Modify the isPolicyAttachable bit


   Object Space

   Object Spaces are the containers that contain protected objects
   in the protected name space.  Each is uniquely identified by
   its namespace ID.

   ivadmin_objectspace_create       - Creates an object space
   ivadmin_objectspace_delete       - Deletes an object space
   ivadmin_objectspace_list         - Lists all object spaces


   Protected Object Policy(POP)

   ivadmin_pop_create               - Creates a POP
   ivadmin_pop_delete               - Deletes a POP
   ivadmin_pop_list                 - Lists all POPs
   ivadmin_pop_attach               - Attaches a POP to a protected object
   ivadmin_pop_detach               - Detaches a POP from a protected object
   ivadmin_pop_find                 - Lists objects where a POP is attached
   ivadmin_pop_show                 - Shows the attributes of a POP
   ivadmin_pop_get                  - gets a handle for a POP
   ivadmin_pop_get_auditlevel       - Accessor to return audit level
   ivadmin_pop_get_description      - Accessor to return description
   ivadmin_pop_get_qop              - Accessor to return qop level
   ivadmin_pop_get_tod              - Accessor to return Time of Day restrictions
   ivadmin_pop_get_warnmode         - Accessor to get Warning Mode value
   ivadmin_pop_set_auditlevel       - Sets the audit level for the POP
   ivadmin_pop_set_description      - Sets the description for a POP
   ivadmin_pop_set_qop              - Sets the qop level for a POP
   ivadmin_pop_set_tod              - Sets the time of day restrictions for a POP
   ivadmin_pop_set_warnmode         - Sets the warning mode value for the POP
   ivadmin_pop_attr_put             - Puts an extended attribute on a POP
   ivadmin_pop_attr_get             - Gets an extended attribute from a POP
   ivadmin_pop_attr_delete_key      - Deletes an extended attribute from a POP
   ivadmin_pop_attr_delete_val      - Deletes a specific value from an extended attr on a POP
   ivadmin_pop_attr_list            - Lists all extended attributes for a POP

   Action (Permission)

   Actions are the available permission definitions.  It is possible
   to customize the default set using these functions.  Each action
   is uniquely identified by the single letter ID given to it.  Additional
   groups of actions can be created and

   ivadmin_action_create            - Create action.
   ivadmin_action_create_in_group   - Create action in action group.
   ivadmin_action_delete            - Delete action.
   ivadmin_action_delete_from_group - Delete action from action group.
   ivadmin_action_list              - List actions.
   ivadmin_action_list_in_group     - List actions in action group.
   ivadmin_action_group_list        - List action groups.
   ivadmin_action_group_create      - Create an action group.
   ivadmin_action_group_delete      - Delete action group.
   ivadmin_action_getid             - Accessor to return action ID
   ivadmin_action_getdescription    - Accessor to return action description.
   ivadmin_action_gettype           - Accessor to return action type.


   ACL

   An ACL is list of ACL entries.  Each entry corresponds to a particular
   subject:  user, group, any-other, and unauthenticated.  Each entry
   also contains the permissions of that subject.  Each ACL is uniquely
   identified by its ID.

   ivadmin_acl_create               - Create ACL.
   ivadmin_acl_delete               - Delete ACL.
   ivadmin_acl_list                 - List all ACLs.
   ivadmin_acl_get                  - Get ACL.
   ivadmin_acl_setdescription       - Set ACL description.
   ivadmin_acl_removeuser           - Remove ACL user permissions.
   ivadmin_acl_removegroup          - Remove ACL group permissions.
   ivadmin_acl_removeanyother       - Remove ACL any-other permissions.
   ivadmin_acl_removeunauth         - Remove ACL unauthenticated permissions.
   ivadmin_acl_setuser              - Set/Add ACL user permissions.
   ivadmin_acl_setgroup             - Set/Add ACL group permissions.
   ivadmin_acl_setanyother          - Set ACL any-other permissions.
   ivadmin_acl_setunauth            - Set ACL unauthenticated permissions.
   ivadmin_acl_getid                - Accessor to return ACL ID.
   ivadmin_acl_getdescription       - Accessor to return ACL description.
   ivadmin_acl_listusers            - Accessor to list ACL user entries.
   ivadmin_acl_listgroups           - Accessor to list ACL group entries.
   ivadmin_acl_getuser              - Accessor to return ACL user permissions.
   ivadmin_acl_getgroup             - Accessor to return ACL group permissions.
   ivadmin_acl_getanyother          - Accessor to return ACL any-other permissions.
   ivadmin_acl_getunauth            - Accessor to return ACL unauthenticated permissions.
   ivadmin_acl_attr_get             - Get an extended attribute of an ACL
   ivadmin_acl_attr_put             - Add a value to an extended attribute key
   ivadmin_acl_attr_list            - List the extedend attribute values for an ACL
   ivadmin_acl_attr_del_val         - Delete an extended attrbut value from an extended
                                    attribute key on an ACL
   ivadmin_acl_attr_del_key         - Delete and extended attribute key from an ACL


   SSO Web Resource

   An SSO web resource is a junctioned web server that WebSEAL will pass SSO authentication information to.
   Each SSO web resource is uniquely identifed by its ID.

   ivadmin_ssoweb_list              - List all SSO web resources.
   ivadmin_ssoweb_get               - Get SSO web resource.
   ivadmin_ssoweb_create            - Create SSO web resource.
   ivadmin_ssoweb_delete            - Delete SSO web resource.
   ivadmin_ssoweb_getid             - Accessor to return SSO web resource ID.
   ivadmin_ssoweb_getdescription    - Accessor to return SSO web resource description.


   SSO Group Resource

   An SSO group resource is a group of SSO web resources.  Each SSO group is uniquely
   identified by its ID.

   ivadmin_ssogroup_list            - List all SSO group resources.
   ivadmin_ssogroup_get             - Get SSO group resource.
   ivadmin_ssogroup_create          - Create SSO group resource.
   ivadmin_ssogroup_delete          - Delete SSO group resource.
   ivadmin_ssogroup_addres          - Add SSO resource to SSO group.
   ivadmin_ssogroup_removeres       - Remove SSO resource to SSO group.
   ivadmin_ssogroup_getid           - Accessor to return SSO group resource ID.
   ivadmin_ssogroup_getdescription  - Accessor to return SSO group resource description.
   ivadmin_ssogroup_getresources    - Accessor to return SSO group members.


   SSO Credential

   An SSO credential is a user's user ID and password that is associated with with an SSO web or SSO group resource.
   Each SSO credential is uniqely identified by the user it is associated with, the SSO resource ID it is associated with,
   and the resource type, web, or group, that is associated with.  SSO web resources IDs and SSO group resource IDs can
   be the same.

   ivadmin_ssocred_list             - List all SSO credentials.
   ivadmin_ssocred_get              - Get SSO credential.
   ivadmin_ssocred_create           - Create SSO credential.
   ivadmin_ssocred_delete           - Delete SSO credential.
   ivadmin_ssocred_set              - Set SSO credential.
   ivadmin_ssocred_getid            - Accessor to return SSO credential resource ID.
   ivadmin_ssocred_gettype          - Accessor to return SSO credential resource type.
   ivadmin_ssocred_getuser          - Accessor to return SSO credential user.
   ivadmin_ssocred_getssouser       - Accessor to return SSO credential SSO user ID.
   ivadmin_ssocred_getssopassword   - Accessor to return SSO credential SSO password.


   Response

   Response objects are returned from most function calls.  Response
   objects can contain more than one message.

   ivadmin_response_getcount        - Accessor to return number of messages in response.
   ivadmin_response_getreason       - Accessor to return reason.
   ivadmin_response_getcode         - Accessor to return message code.
   ivadmin_response_getmodifier     - Accessor to return message modifier.
   ivadmin_response_getmessage      - Accessor to return message text.

   Config

    Configuration APIs.

      ivadmin_cfg_configureserver       - Configure a server
      ivadmin_cfg_unconfigureserver     - Unconfigure
      ivadmin_cfg_setport               - Change server port number
      ivadmin_cfg_setlistening          - Enable or disable listening
      ivadmin_cfg_renewservercert       - Renew server certificate
      ivadmin_cfg_setkeyringpwd         - Change keyring password
      ivadmin_cfg_setssltimeout         - Change SSL session timeout value
      ivadmin_cfg_setapplicationcert    - Replace the application's certificate
      ivadmin_cfg_addreplica            - Add replica
      ivadmin_cfg_chgreplica            - Change replica
      ivadmin_cfg_rmvreplica            - Remove replica



   Server

   A server is a consumer of the AZN API.

   ivadmin_server_performtask       - Send a command to an AZN server.

*/

/*
   Frees memory allocated by this API set.  All memory allocated
   by these API's must be freed by this function.

   Possible reasons returned:  N/A
*/
IVADMIN_DECLSPEC void IVADMIN_CALLTYPE
   ivadmin_free(void *p);

/*
   The context represents a connection to the management server.  To successfully create a context the
   management server must be available and the authentication must be successful.

   Possible reasons returned:  None

   keyringfile      - Fully qualified path name to the SSL keyring file which contains the
                      management server's public key.
   keyringstashfile - Fully qualified path name to the stash file which contains the password used to access
                      the keyring file.  You must either specify a keyring stash file or keyring file password.
                      If you specify both, the password will be used.  If you specify neither, an invalid input
                      error will be returned.
   keyringpassword  - Password used to access the keyring file.  You must either specify a keyring stash
                      file or keyring file password.  If you specify both, the password will be used.  If you
                      specify neither, an invalid input error will be returned.
   userid           - Administrator user ID to authenticate as.  This user must be a member of the
                      "cn=iv-admin,cn=SecurityGroups,secauthority=default" LDAP group.
   pwd              - Administrator password.
   serverdn         - Management server certificate distinguished name used to authenticate management server.
                      This parameter is optional.  If you do not wish to authenticate the management server you
                      may specify NULL or an empty string.
   serverhost       - Management server hostname or IP address.
   port             - Management server listening port number.
   ctx              - Returned context.  This is used to send administration requests to the management server.
                      This should be freed when no longer needed.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_create(const char *keyringfile,
                          const char *keyringstashfile,
                          const char *keyringpassword,
                          const char *userid,
                          const char *pwd,
                          const char *serverdn,
                          const char *serverhost,
                          unsigned long port,
                          ivadmin_context *ctx,
                          ivadmin_response *rsp);

/*
   The context represents a connection to the management server.  The location of the management server
   and SSL information is retrieved from the the current runtime configuration.
   To successfully create a context the management server must be available and the
   authentication must be successful.

   Possible reasons returned:  None

   userid           - Administrator user ID to authenticate as.  This user must be a member of the
                      "cn=iv-admin,cn=SecurityGroups,secauthority=default" LDAP group.
   pwd              - Administrator password.
   ctx              - Returned context.  This is used to send administration requests to the management server.
                      This should be freed when no longer needed.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_createdefault(const char *userid,
                                 const char *pwd,
                                 ivadmin_context *ctx,
                                 ivadmin_response *rsp);

/*
   Closes the connection with the management server.  This must be called prior program exit.  Deleting the connection
   allows the client and management server to free SSL resources.  The context is no longer useable and the context memory
   should be freed following this call.

   Possible reasons returned:  None

   ctx         - Context to delete.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_delete(ivadmin_context ctx,
                          ivadmin_response *rsp);

/*
   Set the delegated credential for the context based on the specified PAC.
   Only one credential can be delegated at a time.  If a delegated
   credential already exists for this context it will be overwritten.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   pacValue    - The credential PAC data.
   pacLength   - The credential PAC length.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setdelcred(ivadmin_context ctx,
                              const unsigned char* pacValue,
                              const unsigned long pacLength,
                              ivadmin_response *rsp);

/*
   Clear the delegated credential for the context.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_cleardelcred(ivadmin_context ctx,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: admin show configuration

   This returns an indicator of which type of user registry is configured for this management server.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   registry    - Pointer to a registry type indicator (refer to the #defines below)
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.

*/
#define IVADMIN_CONTEXT_DCEUSERREG  0
#define IVADMIN_CONTEXT_LDAPUSERREG 1
#define IVADMIN_CONTEXT_ADUSERREG 2
#define IVADMIN_CONTEXT_DOMINOUSERREG 3
#define IVADMIN_CONTEXT_MULTIDOMAIN_ADUSERREG 4
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getuserreg(ivadmin_context ctx,
                              unsigned long *registry,
                              ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set min-password-length {unset|<number>}

   Set the minimum password length for all user accounts.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   length      - Minimum allowed password length to be set.
   unset       - Policy ignored and not enforced if set to true. If set to false
                 the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setminpwdlen(ivadmin_context ctx,
                                unsigned long length,
                                unsigned long unset,
                                ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: policy get min-password-length

   Get the minimum password length for all user accounts.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   length      - Returned minimum allowed password length.
   unset       - Policy ignored and not enforced if set to true. If set to false
                 the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getminpwdlen(ivadmin_context ctx,
                                unsigned long *length,
                                unsigned long *unset,
                                ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: policy set max-password-age {unset|<relative-time>}

   Set the maximum password age for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   seconds      - Maximum lifetime before expiration of password in seconds.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setmaxpwdage(ivadmin_context ctx,
                                unsigned long seconds,
                                unsigned long unset,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get max-password-age

   Get the maximum password age for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   seconds      - Returned maximum lifetime before expiration of password in seconds.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getmaxpwdage(ivadmin_context ctx,
                                unsigned long *seconds,
                                unsigned long *unset,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set account-expiry-date {unlimited|<absolute-time>|unset}

   Set the account expiration date for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   seconds      - Date and time of the expiration of all user accounts.  This is the number of seconds
                  since 00:00:00 Universal Time, January 1, 1970 (Same as time_t).
   unlimited    - Do not expire accounts and ingnore "seconds" parameter if set to true. (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setaccexpdate(ivadmin_context ctx,
                                 unsigned long seconds,
                                 unsigned long unlimited,
                                 unsigned long unset,
                                 ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get account-expiry-date

   Get the account expiration date for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   seconds      - Returned date and time of the expiration of all user accounts.  This is the number of seconds
                  since 00:00:00 Universal Time, January 1, 1970 (Same as time_t).
   unlimited    - Returned account expiration not restricted indicator. (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getaccexpdate(ivadmin_context ctx,
                                 unsigned long *seconds,
                                 unsigned long *unlimited,
                                 unsigned long *unset,
                                 ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set max-login-failures <number|unset>

   Set the maximum allowed login failures for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   failures     - Maximum login failures allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setmaxlgnfails(ivadmin_context ctx,
                                  unsigned long failures,
                                  unsigned long unset,
                                  ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get max-login-failures

   Get the maximum allowed login failures for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   failures     - Maximum login failures allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getmaxlgnfails(ivadmin_context ctx,
                                  unsigned long *failures,
                                  unsigned long *unset,
                                  ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set disable-time-interval {<number>|unset|disable}

   Set the time to disable all user accounts if maximum login failures exceeded.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   seconds      - Disable the user account for the specified number of seconds if the
                  maximum login failures exceeded.
   disable      - Disable the user account if maximum login failures exceeded.  Administrator
                  action is required to enable the account. (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setdisabletimeint(ivadmin_context ctx,
                                     unsigned long seconds,
                                     unsigned long disable,
                                     unsigned long unset,
                                     ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get disable-time-interval

   Get the time to disable all user accounts if maximum login failures exceeded.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   seconds      - Disable the user account for the specified number of seconds if the
                  maximum login failures exceeded.
   disable      - Disable the user account if maximum login failures exceeded.  Administrator
                  action is required to enable the account. (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getdisabletimeint(ivadmin_context ctx,
                                     unsigned long *seconds,
                                     unsigned long *disable,
                                     unsigned long *unset,
                                     ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set max-password-repeated-chars <number|unset>

   Set the maximum allowed repeated characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   chars        - Maximum repeated characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setmaxpwdrepchars(ivadmin_context ctx,
                                     unsigned long chars,
                                     unsigned long unset,
                                     ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get max-password-repeated-chars

   Get the maximum allowed repeated characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   chars        - Maximum repeated characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getmaxpwdrepchars(ivadmin_context ctx,
                                     unsigned long *chars,
                                     unsigned long *unset,
                                     ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set min-password-alphas {unset|<number>}

   Set the minimum allowed alphabetic characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   chars        - Minimum alphabetic characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setminpwdalphas(ivadmin_context ctx,
                                   unsigned long chars,
                                   unsigned long unset,
                                   ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get min-password-alphas

   Get the minimum allowed alphabetic characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   chars        - Minimum alphabetic characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getminpwdalphas(ivadmin_context ctx,
                                   unsigned long *chars,
                                   unsigned long *unset,
                                   ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set min-password-non-alphas {unset|<number>}

   Set the minimum allowed non-alphabetic characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   chars        - Minimum non-alphabetic characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setminpwdnonalphas(ivadmin_context ctx,
                                      unsigned long chars,
                                      unsigned long unset,
                                      ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get min-password-non-alphas

   Get the minimum allowed non-alphabetic characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   chars        - Minimum non-alphabetic characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getminpwdnonalphas(ivadmin_context ctx,
                                      unsigned long *chars,
                                      unsigned long *unset,
                                      ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set password-spaces {yes|no|unset}

   Set whether spaces are allowed in passwords for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   allowed      - Indicates whether spaces should be allowed in passwords (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setpwdspaces(ivadmin_context ctx,
                                unsigned long allowed,
                                unsigned long unset,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get password-spaces

   Get whether spaces are allowed in passwords for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   allowed      - Indicates whether spaces should be allowed in passwords (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getpwdspaces(ivadmin_context ctx,
                                unsigned long *allowed,
                                unsigned long *unset,
                                ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: policy set todaccess <todaccess string>

   Sets the global Time of Day access policy

   Possible reasons returned:  None

   ctx         - Context to communcicate with management server
   days        - A bit map of the days for the time of day policy
   start       - the minutes after midnight for the start of the time range
   end         - the minutes after midnight for the end of the time range
   reference   - the time zone, utc or local
   unset       - If true the policy is removed.  If False the policy is set
                 as specified.
   rsp         - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_settodaccess(ivadmin_context ctx,
                                unsigned long days,
                                unsigned long start,
                                unsigned long end,
                                unsigned long reference,
                                unsigned long unset,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get todaccess

   Gets the global Time of Day access policy

   Possible reasons returned:  None

   ctx         - Context to communcicate with management server
   days        - A bit map of the days for the time of day policy
   start       - the minutes after midnight for the start of the time range
   end         - the minutes after midnight for the end of the time range
   reference   - the time zone, utc or local
   unset       - If true the policy is notset.  If False the policy is set.
   rsp         - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_gettodaccess(ivadmin_context ctx,
                                unsigned long *days,
                                unsigned long *start,
                                unsigned long *end,
                                unsigned long *reference,
                                unsigned long *unset,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: none

   Gets the Management server's host name

   Possible reasons returned:  None

   return      - Host name. This should NOT be freed.
                 This is data maintained in the context structure.

   ctx         - Context to communcicate with management server
*/
IVADMIN_DECLSPEC const char *  IVADMIN_CALLTYPE
   ivadmin_context_getmgmtsvrhost(ivadmin_context ctx);

/*
   IVADMIN command line equivalent: none

   Gets the Management server's TCP/IP port

   Possible reasons returned:  None

   ctx         - Context to communcicate with management server
*/
IVADMIN_DECLSPEC unsigned long  IVADMIN_CALLTYPE
   ivadmin_context_getmgmtsvrport(ivadmin_context ctx);

/*
   IVADMIN command line equivalent: user create [-gsouser] [-no-password-policy] user-name dn cn sn pwd

   Creates a user in the LDAP directory used by the management server.
   Accounts are created invalid by default.  You must use
   ivadmin_user_setaccountvalid to enable the account.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - A user already exists with the specified userid.

   ctx         - Context to communicate with management server.
   userid      - Policy Director user ID.
   dn          - LDAP directory Distinguished Name.
   sn          - LDAP directory attribute surname.
   cn          - LDAP directory attribute commonname.
   pwd         - LDAP directory attribute password.
   groupid     - The initial LDAP group to which the user will belong, can be NULL.
   ssouser     - User will be capable of having sso credentials. (IVADMIN_TRUE/IVADMIN_FALSE).
   nopwdpolicy - Password policy will not be enforced during creation.
                 This has no effect on password policy enforcement after
                 user creation. (IVADMIN_TRUE/IVADMIN_FALSE)
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
/******************************************
 * DEPRECATED....USE ivadmin_user_create3 *
 ******************************************
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_create2(ivadmin_context ctx,
                        const char *userid,
                        const char *dn,
                        const char *cn,
                        const char *sn,
                        const char *pwd,
                        const char *groupid,
                        unsigned long ssouser,
                        unsigned long nopwdpolicy,
                        ivadmin_response *rsp);
*/
/*
   IVADMIN command line equivalent: user create [-gsouser] [-no-password-policy] user-name dn cn sn pwd

   Creates a user in the LDAP directory used by the management server.
   Accounts are created invalid by default.  You must use
   ivadmin_user_setaccountvalid to enable the account.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - A user already exists with the specified userid.

   ctx         - Context to communicate with management server.
   userid      - Policy Director user ID.
   dn          - LDAP directory Distinguished Name.
   sn          - LDAP directory attribute surname.
   cn          - LDAP directory attribute commonname.
   pwd         - LDAP directory attribute password.
   group_count - The number of groups to which the user will belong
   groups    -   The initial groups to which the user will belong, can be NULL.
   ssouser     - User will be capable of having sso credentials. (IVADMIN_TRUE/IVADMIN_FALSE).
   nopwdpolicy - Password policy will not be enforced during creation.
                 This has no effect on password policy enforcement after
                 user creation. (IVADMIN_TRUE/IVADMIN_FALSE)
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_create3(ivadmin_context ctx,
                        const char *userid,
                        const char *dn,
                        const char *cn,
                        const char *sn,
                        const char *pwd,
                        unsigned long group_count,
                        const char **groups,
                        unsigned long ssouser,
                        unsigned long nopwdpolicy,
                        ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user import [-gsouser] user-name dn

   Make a user that already exists in the LDAP directory a Policy Director user.  Accounts are
   created invalid by default.  You must use ivadmin_user_setaccountvalid to enable the account.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   dn          - LDAP directory Distinguished Name.
   groupid     - The initial LDAP group to which the user will belong, can be NULL.
   ssouser     - User will be capable of having sso credentials.  (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_import2(ivadmin_context ctx,
                        const char *userid,
                        const char *dn,
                        const char *groupid,
                        unsigned long ssouser,
                        ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user modify user-name description description

   Modify the user description.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   description - New description.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setdescription(ivadmin_context ctx,
                               const char *userid,
                               const char *description,
                               ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user modify user-name password password

   Modify the user password.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   pwd         - New password.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setpassword(ivadmin_context ctx,
                            const char *userid,
                            const char *pwd,
                            ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user modify user-name authentication-mechanism mech

   Modify the user authentication mechanism.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   authmech    - New authentication mechanism (IVADMIN_USER_AUTHMETHDCE/IVADMIN_USER_AUTHMETHLDAP).
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
#define IVADMIN_USER_DCEAUTHMETH  0
#define IVADMIN_USER_LDAPAUTHMETH 1
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setauthmech(ivadmin_context ctx,
                            const char *userid,
                            unsigned long authmech,
                            ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user modify user-name account-valid {yes | no}

   Enable/disable the Policy Director account.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   valid       - Boolean indicator of account validity. (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setaccountvalid(ivadmin_context ctx,
                                const char *userid,
                                unsigned long valid,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user modify user-name password-valid {yes | no}

   Expire the Policy Director account password.  This will force the user to change his password when he attempts to log on.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   valid       - Expire IVADMIN_FALSE, or unexpire IVADMIN_TRUE.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setpasswordvalid(ivadmin_context ctx,
                                 const char *userid,
                                 unsigned long valid,
                                 ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user modify user-name gsouser {yes | no}

   Enable/disable the sso capabilities of the Policy Director user.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   ssouser     - User will be capable of having sso credentials. (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setssouser(ivadmin_context ctx,
                           const char *userid,
                           unsigned long ssouser,
                           ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user delete [-registry] user-name

   Delete the user.  This will delete all Policy Director information about the user and optionally
   will delete the user registry contents.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   registry    - Delete user from the user registry as well as from Policy Director. (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_delete2(ivadmin_context ctx,
                        const char *userid,
                        unsigned long registry,
                        ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user show-groups username

   This will return the list of groups of which this user is a member.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   count       - Number of group IDs returned.
   groupids    - Array of pointers to group IDs.  You should free each groupID character pointer and the array of pointers when
                 no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getmemberships(ivadmin_context ctx,
                               const char *userid,
                               unsigned long *count,
                               char ***groupids,
                               ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user show user-name

   This will return a Policy Director user.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   user        - Returned user.  You should free this memory when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_get(ivadmin_context ctx,
                    const char *userid,
                    ivadmin_ldapuser *user,
                    ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user show-dn dn

   This will return a Policy Director user using the LDAP directory DN for identification.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   dn          - LDAP directory distinguished name of user.
   user        - Returned user.  You should free this memory when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getbydn(ivadmin_context ctx,
                        const char *dn,
                        ivadmin_ldapuser *user,
                        ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user list pattern max-return

   This will return the list of users IDs whose ID matches the pattern specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   pattern     - Pattern match for user IDs.  IVADMIN_ALLPATTERN indicates all users.
   maxreturn   - Maximum number to return.  IVADMIN_MAXRETURN indicates unlimited.  This number can also be limited by the
                 LDAP server so the maximum returned is really the minimum of the server configuration and this value.
   count       - Number of user IDs returned.
   userids     - Array of pointers to user IDs.  You should free each userID character pointer and the array of pointers when
                 no longer needed.  The order returned is the order created.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
#define IVADMIN_MAXRETURN 0
#define IVADMIN_ALLPATTERN "*"
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_list(ivadmin_context ctx,
                     const char *pattern,
                     unsigned long maxreturn,
                     unsigned long *count,
                     char ***userids,
                     ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  user list-dn pattern max-return

   This will return the list of LDAP directory distinguished names whose LDAP directory cn attribute matches the pattern specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   pattern     - Pattern match for cn attribute.  IVADMIN_ALLPATTERN indicates all users.
   maxreturn   - Maximum number to return.  IVADMIN_MAXRETURN indicates unlimited.  This number can also be limited by the
                 LDAP server so the maximum returned is really the minimum of the server configuration and this value.
   count       - Number of user dn's returned.
   dns         - Array of pointers to dns.  You should free each dn character pointer and the array of pointers when
                 no longer needed.  The order returned is the order created.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_listbydn(ivadmin_context ctx,
                         const char *pattern,
                         unsigned long maxreturn,
                         unsigned long *count,
                         char ***dns,
                         ivadmin_response *rsp);

/*
   Returns user ID.

   Possible reasons returned:  N/A

   user        - Pointer to user structure.

   return      - User ID.  This should NOT be freed.  This is data maintained in the user structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getid(ivadmin_ldapuser user);

/*
   Returns user description.

   Possible reasons returned:  N/A

   user        - Pointer to user structure.

   return      - Description.  This should NOT be freed.  This is data maintained in the user structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getdescription(ivadmin_ldapuser user);

/*
   Returns authentication mechanism.

   Possible reasons returned:  N/A

   user        - Pointer to user structure.

   return      - Authentication mechanism. (IVADMIN_USER_AUTHMETHDCE/IVADMIN_USER_AUTHMETHLDAP)
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getauthmech(ivadmin_ldapuser user);

/*
   Returns account valid indicator.

   Possible reasons returned:  N/A

   user        - Pointer to user structure.

   return      - Account valid indicator. (IVADMIN_TRUE/IVADMIN_FALSE).
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getaccountvalid(ivadmin_ldapuser user);

/*
   Returns password valid indicator.

   Possible reasons returned:  N/A

   user        - Pointer to user structure.

   return      - Password expired indicator. (IVADMIN_TRUE/IVADMIN_FALSE).
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getpasswordvalid(ivadmin_ldapuser user);

/*
   Returns SSO capable indicator.

   Possible reasons returned:  N/A

   user        - Pointer to user structure.

   return      - SSO capable indicator. (IVADMIN_TRUE/IVADMIN_FALSE).
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getssouser(ivadmin_ldapuser user);

/*
   Returns distinguished name(dn).

   Possible reasons returned:  N/A

   user        - Pointer to user structure.

   return      - Distinguished name(dn).  This should NOT be freed.  This is data maintained in the user structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getdn(ivadmin_ldapuser user);

/*
   Returns common name(cn) attribute.

   Possible reasons returned:  N/A

   user        - Pointer to user structure.

   return      - Common name(cn) attribute.  This should NOT be freed.  This is data maintained in the user structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getcn(ivadmin_ldapuser user);

/*
   Returns surname(sn) attribute.

   Possible reasons returned:  N/A

   user        - Pointer to user structure.

   return      - Surname (sn) attribute.  This should NOT be freed.  This is data maintained in the user structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getsn(ivadmin_ldapuser user);

/*
   IVADMIN command line equivalent: policy set account-expiry-date {unlimited|<absolute-time>|unset} [-user <user-name>]
   Set the account expiration date for specified user.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   seconds      - Date and time of the expiration of specified user account.  This is the number of seconds
                  since 00:00:00 Universal Time, January 1, 1970 (Same as time_t).
   unlimited    - Do not expire specified user account and ingnore "seconds" parameter if set to true. (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setaccexpdate(ivadmin_context ctx,
                              const char *userid,
                              unsigned long seconds,
                              unsigned long unlimited,
                              unsigned long unset,
                              ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get account-expiry-date [-user <user-name>]

   Get the account expiration date for specified user.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   seconds      - Returned date and time of the expiration for specified user account.  This is the number of seconds
                  since 00:00:00 Universal Time, January 1, 1970 (Same as time_t).
   unlimited    - Returned account expiration not restricted indicator. (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getaccexpdate(ivadmin_context ctx,
                              const char *userid,
                              unsigned long *seconds,
                              unsigned long *unlimited,
                              unsigned long *unset,
                              ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set todaccess <todaccess string> -user <userid>

   Sets the user specific Time of Day access policy

   Possible reasons returned:  None

   ctx         - Context to communcicate with management server
   userid      - ldap username
   days        - A bit map of the days for the time of day policy
   start       - the minutes after midnight for the start of the time range
   end         - the minutes after midnight for the end of the time range
   reference   - the time zone, utc or local
   unset       - If true the policy is removed.  If False the policy is set
                 as specified.
   rsp         - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_settodaccess(ivadmin_context ctx,
                             const char *userid,
                             unsigned long days,
                             unsigned long start,
                             unsigned long end,
                             unsigned long reference,
                                unsigned long unset,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get todaccess -user <userid>

   Gets the user specific Time of Day access policy

   Possible reasons returned:  None

   ctx         - Context to communcicate with management server
   userid      - ldap username
   days        - A bit map of the days for the time of day policy
   start       - the minutes after midnight for the start of the time range
   end         - the minutes after midnight for the end of the time range
   reference   - the time zone, utc or local
   unset       - If true the policy is notset.  If False the policy is set.
   rsp         - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_gettodaccess(ivadmin_context ctx,
                             const char *userid,
                             unsigned long *days,
                             unsigned long *start,
                             unsigned long *end,
                             unsigned long *reference,
                                unsigned long *unset,
                                ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: policy set min-password-length {unset|<number>} [-user <user-name>]

   Set the minimum password length for all user accounts.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   length      - Minimum allowed password length to be set.
   unset       - Policy ignored and not enforced if set to true. If set to false
                 the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setminpwdlen(ivadmin_context ctx,
                             const char *userid,
                             unsigned long length,
                             unsigned long unset,
                             ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: policy get min-password-length [-user <user-name>]

   Get the minimum password length for all user accounts.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   userid      - User ID.
   length      - Returned minimum allowed password length.
   unset       - Policy ignored and not enforced if set to true. If set to false
                 the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getminpwdlen(ivadmin_context ctx,
                             const char *userid,
                             unsigned long *length,
                             unsigned long *unset,
                             ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: policy set max-password-age {unset|<relative-time>} [-user <user-name>]

   Set the maximum password age for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   seconds      - Maximum lifetime before expiration of password in seconds.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setmaxpwdage(ivadmin_context ctx,
                             const char *userid,
                             unsigned long seconds,
                             unsigned long unset,
                             ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get max-password-age [-user <user-name>]

   Get the maximum password age for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   seconds      - Returned maximum lifetime before expiration of password in seconds.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getmaxpwdage(ivadmin_context ctx,
                             const char *userid,
                             unsigned long *seconds,
                             unsigned long *unset,
                             ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set max-login-failures <number|unset> [-user <user-name>]

   Set the maximum allowed login failures for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   failures     - Maximum login failures allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setmaxlgnfails(ivadmin_context ctx,
                               const char *userid,
                               unsigned long failures,
                               unsigned long unset,
                               ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get max-login-failures [-user <user-name>]

   Get the maximum allowed login failures for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   failures     - Maximum login failures allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getmaxlgnfails(ivadmin_context ctx,
                               const char *userid,
                               unsigned long *failures,
                               unsigned long *unset,
                               ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set disable-time-interval {<number>|unset|disable} [-user <user-name>]

   Set the time to disable all user accounts if maximum login failures exceeded.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   seconds      - Disable the user account for the specified number of seconds if the
                  maximum login failures exceeded.
   disable      - Disable the user account if maximum login failures exceeded.  Administrator
                  action is required to enable the account. (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setdisabletimeint(ivadmin_context ctx,
                                  const char *userid,
                                  unsigned long seconds,
                                  unsigned long disable,
                                  unsigned long unset,
                                  ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get disable-time-interval [-user <user-name>]

   Get the time to disable all user accounts if maximum login failures exceeded.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   seconds      - Disable the user account for the specified number of seconds if the
                  maximum login failures exceeded.
   disable      - Disable the user account if maximum login failures exceeded.  Administrator
                  action is required to enable the account. (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getdisabletimeint(ivadmin_context ctx,
                                  const char *userid,
                                  unsigned long *seconds,
                                  unsigned long *disable,
                                  unsigned long *unset,
                                  ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set max-password-repeated-chars <number|unset> [-user <user-name>]

   Set the maximum allowed repeated characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   chars        - Maximum repeated characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setmaxpwdrepchars(ivadmin_context ctx,
                                  const char *userid,
                                  unsigned long chars,
                                  unsigned long unset,
                                  ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get max-password-repeated-chars [-user <user-name>]

   Get the maximum allowed repeated characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   chars        - Maximum repeated characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getmaxpwdrepchars(ivadmin_context ctx,
                                  const char *userid,
                                  unsigned long *chars,
                                  unsigned long *unset,
                                  ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set min-password-alphas {unset|<number>} [-user <user-name>]

   Set the minimum allowed alphabetic characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   chars        - Minimum alphabetic characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setminpwdalphas(ivadmin_context ctx,
                                const char *userid,
                                unsigned long chars,
                                unsigned long unset,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get min-password-alphas

   Get the minimum allowed alphabetic characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   chars        - Minimum alphabetic characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getminpwdalphas(ivadmin_context ctx,
                                const char *userid,
                                unsigned long *chars,
                                unsigned long *unset,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set min-password-non-alphas {unset|<number>} [-user <user-name>]

   Set the minimum allowed non-alphabetic characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   chars        - Minimum non-alphabetic characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setminpwdnonalphas(ivadmin_context ctx,
                                   const char *userid,
                                   unsigned long chars,
                                   unsigned long unset,
                                   ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get min-password-non-alphas [-user <user-name>]

   Get the minimum allowed non-alphabetic characters in a password for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   chars        - Minimum non-alphabetic characters allowed.
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getminpwdnonalphas(ivadmin_context ctx,
                                   const char *userid,
                                   unsigned long *chars,
                                   unsigned long *unset,
                                   ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy set password-spaces {yes|no|unset} [-user <user-name>]

   Set whether spaces are allowed in passwords for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   allowed      - Indicates whether spaces should be allowed in passwords (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setpwdspaces(ivadmin_context ctx,
                             const char *userid,
                             unsigned long allowed,
                             unsigned long unset,
                             ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: policy get password-spaces [-user <user-name>]

   Get whether spaces are allowed in passwords for all user accounts.

   Possible reasons returned:  None

   ctx          - Context to communicate with management server.
   userid       - User ID.
   allowed      - Indicates whether spaces should be allowed in passwords (IVADMIN_TRUE/IVADMIN_FALSE).
   unset        - Policy ignored and not enforced if set to true. If set to false
                  the policy is set as specified (IVADMIN_TRUE/IVADMIN_FALSE).
   rsp          - Response.  Indicates success or failure.  Free when no longer needed.

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getpwdspaces(ivadmin_context ctx,
                             const char *userid,
                             unsigned long *allowed,
                             unsigned long *unset,
                             ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group create groupname dn cn

   Creates a group in the LDAP directory used by the management server.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - A group already exists with the specified groupid.

   ctx             - Context to communicate with management server.
   groupid         - Group ID.
   dn              - LDAP directory Distinguished Name.
   cn              - LDAP directory common name attribute.
   group_container - Container object within the management object space.
                     Can be NULL to indiciate that it will be at the root level.
   rsp             - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_create2(ivadmin_context ctx,
                         const char *groupid,
                         const char *dn,
                         const char *cn,
                         const char *group_container,
                         ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group import groupname dn

   Make a group that already exists in the LDAP directory a Policy Director group.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupid     - Group ID.
   dn          - LDAP directory Distinguished Name.
   group_container - Container object within the management object space.  Can
               be NULL to indiciate that it will be at the root level.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_import2(ivadmin_context ctx,
                         const char *groupid,
                         const char *dn,
                         const char *group_container,
                         ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group modify groupname description description

   Change the group description.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupid     - Group ID.
   description - New description.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_setdescription(ivadmin_context ctx,
                                const char *groupid,
                                const char *description,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group modify groupname add user-name

   Add a user to the group.  Policy Director does not support a group as a group member.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupid     - Group ID.
   userid      - New member user ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/

/**********************************************
 * DEPRECATED....USE ivadmin_group_addmembers *
 **********************************************
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_addmember(ivadmin_context ctx,
                           const char *groupid,
                           const char *userid,
                           ivadmin_response *rsp);
*/


/*
   IVADMIN command line equivalent:  group modify groupname add user1,user2...

   Add users to the group.  Policy Director does not support a group as a group member.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupid     - Group ID.
   users      -  New member user IDs.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_addmembers(ivadmin_context ctx,
                           const char *groupid,
                           unsigned long userCount,
                           const char **users,
                           ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group modify groupname remove user-name

   Remove a user from the group.  

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupid     - Group ID.
   userid      - Member user ID to remove.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/

/**********************************************
 * DEPRECATED....USE ivadmin_group_removemembers *
 **********************************************

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_removemember(ivadmin_context ctx,
                              const char *groupid,
                              const char *userid,
                              ivadmin_response *rsp);
*/

/*
   IVADMIN command line equivalent:  group modify groupname remove user1,user2...

   Remove users from the group.  

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupid     - Group ID.
   users      -  Member user ID to remove.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_removemembers(ivadmin_context ctx,
                           const char *groupid,
                           unsigned long userCount,
                           const char **users,
                           ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group delete [-registry] groupname

   Delete the group.  This will delete all Policy Director information about the group and optionally
   will delete the user registry contents.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupid     - Group ID.
   registry    - Delete group from the user registry as well as from Policy Director. (IVADMIN_TRUE/IVADMIN_FALSE).   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_delete2(ivadmin_context ctx,
                         const char *groupid,
                         unsigned long registry,
                         ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group show groupname

   This will return the group specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupid     - Group ID.
   group       - Returned group.  You should free this memory when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_get(ivadmin_context ctx,
                     const char *groupid,
                     ivadmin_ldapgroup *group,
                     ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group show-dn dn

   This will return a group user using the LDAP directory DN for identification.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   dn          - LDAP directory distinguished name of group.
   group       - Returned group.  You should free this memory when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_getbydn(ivadmin_context ctx,
                         const char *dn,
                         ivadmin_ldapgroup *group,
                         ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group list pattern max-return

   This will return the list of group IDs whose ID matches the pattern specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   pattern     - Pattern match for group IDs.  IVADMIN_ALLPATTERN indicates all groups.
   maxreturn   - Maximum number to return.  IVADMIN_MAXRETURN indicates unlimited.  This number can also be limited by the
                 LDAP server so the maximum returned is really the minimum of the server configuration and this value.
   count       - Number of group IDs returned.
   groupids    - Array of pointers to group IDs.  You should free each groupID character pointer and the array of pointers when
                 no longer needed.  The order returned is the order created.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_list(ivadmin_context ctx,
                      const char *pattern,
                      unsigned long maxreturn,
                      unsigned long *count,
                      char ***groupids,
                      ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group list-dn pattern max-return

   This will return the list of LDAP directory distinguished names whose LDAP directory cn attribute matches the pattern specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   pattern     - Pattern match for cn attribute.  IVADMIN_ALLPATTERN indicates all users.
   maxreturn   - Maximum number to return.  IVADMIN_MAXRETURN indicates unlimited.  This number can also be limited by the
                 LDAP server so the maximum returned is really the minimum of the server configuration and this value.
   count       - Number of group dn's returned.
   dns         - Array of pointers to dns.  You should free each dn character pointer and the array of pointers when
                 no longer needed.  The order returned is the order created.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_listbydn(ivadmin_context ctx,
                          const char *pattern,
                          unsigned long maxreturn,
                          unsigned long *count,
                          char ***dns,
                          ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  group show-members groupname

   This will return the list of user IDs who are members of the group specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupid     - Group ID.
   count       - Number of user IDs returned.
   userids     - Array of pointers to user IDs.  You should free each user ID character pointer and the array of pointers when
                 no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_getmembers(ivadmin_context ctx,
                            const char *groupid,
                            unsigned long *count,
                            char ***userids,
                            ivadmin_response *rsp);

/*
   Returns the group ID.

   group       - Pointer to group structure.

   return      - Group ID.  This should NOT be freed.  This is data maintained in the group structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_group_getid(ivadmin_ldapgroup group);

/*
   Returns the group description.

   group       - Pointer to group structure.

   return      - Group description.  This should NOT be freed.  This is data maintained in the group structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_group_getdescription(ivadmin_ldapgroup group);

/*
   Returns the group distinguished name(dn).

   group       - Pointer to group structure.

   return      - Group distinguished name(dn).  This should NOT be freed.  This is data maintained in the group structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_group_getdn(ivadmin_ldapgroup group);

/*
   Returns the group common name(cn) attribute.

   group       - Pointer to group structure.

   return      - Common name(cn) attribute.  This should NOT be freed.  This is data maintained in the group structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_group_getcn(ivadmin_ldapgroup group);

/*
   IVADMIN command line equivalent: objectspace create objectspace-name

   This function creates an objectspace

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - An object space already exists
                                       with the specified objspaceid.

   ctx          - Context to communicate with management server
   objspaceid   - name of objecspace to create
   type         - the type of objectspace to create
   description  - a desctiption for the object space
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_objectspace_create(ivadmin_context ctx,
                              const char *objspaceid,
                              unsigned long type,
                              const char *description,
                              ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: objectspace list

   This command lists all the objectspaces

   Possible reasons returned:  None

   ctx          - Context to communicate with management server
   count        - the number of object spaces
   objspace_list    - list of objectspaces, should be freed
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_objectspace_list(ivadmin_context ctx,
                            unsigned long *count,
                            char ***objspace_list,
                            ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: objectspace delete objectspace-name

   This function deletes an object space

   Possible reasons returned:  None

   ctx          - Context to communicate with the management server
   objspaceid   - The name of the objectspace to delete
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_objectspace_delete(ivadmin_context ctx,
                              const char *objspaceid,
                              ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: object create object-name

   This will create a protceted object

   Possible reasons returned:  None

   ctx          - Context to communicate with management server
   objid        - The name of the object to create
   type         - The type of object to create
   description  - A short description of the object
   isLeaf       - Is this a leaf object
   rsp          - Response.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_create(ivadmin_context ctx,
                          const char *objid,
                          unsigned long type,
                          const char *description,
                          ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: object delete object-name

   This will delete a protected object

   Possible reasons returned:  None

   ctx          - Context to communicate with the management server
   objid        - The name of the obhect to delete
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_delete(ivadmin_context ctx,
                          const char *objid,
                          ivadmin_response *rsp);


/*
   IVADMIN command line equivalent: object modify object-name name new-name conflict-resolution resolution-modifer

   Possible reasons returned:  None

   ctx          - Context to communicate with the management server
   old_objid    - The old object name
   new_objid    - The new object name
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_setname(ivadmin_context ctx,
                               const char *old_objid,
                               const char *new_objid,
                               ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: object modify object-name desctiption new-description

   This will modify the description field of a protected object

   Possible reasons returned:  None

   ctx          - Context to communicate with the management server
   objid        - The object to work with
   description  - Then new description for the object
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_setdesc(ivadmin_context ctx,
                               const char *objid,
                               const char *description,
                               ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: object modify object-name type new-type

   This will modify the type field of a protected object

   Possible reasons returned:  None

   ctx          - Context to communicate with the management server
   objid        - The object to work with
   type         - Then new type for the object
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_settype(ivadmin_context ctx,
                               const char *objid,
                               unsigned long type,
                               ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: object modify onject-name isPolicyAttachable yes|no

   This will modify whether policy can be attached to the object

   Possible reasons returned:  None

   ctx          - Context to communicate with the management server
   objid        - The name of the object to work with
   flag         - The flag to set the isPolicyAttachable Vaule 1=yes 0=no
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_setpolicyattachable(ivadmin_context ctx,
                                           const char *objid,
                                           unsigned long flag,
                                           ivadmin_response *rsp);

/*
   IVADMIN command line equivalent: object object-name put attr-name attr-value

   This will create/update an attribute/value pair for the object.

   Possible reasons returned:  None

   ctx          - Context to communicate with the management server
   objid        - The name of the object to work with
   attr_name    - The name of the extended attribute
   attr_value   - The value for the extended attribute
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attrput(ivadmin_context ctx,
                            const char *objid,
                            const char *attr_name,
                            const char *attr_value,
                            ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: object object-name get attr-name

   This will retrieve the value associated with the extended attribute for this
   object

   Possible reasons returned:  None

   protobj      - ivadmin protected object structure
   attr_key     - The extended attribute to look up
   count        - the number of values returned
   attr_value   - The list of values returned, should be freed
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attrget(ivadmin_protobj protobj,
                            const char *attr_key,
                            unsigned long *count,
                            char ***attr_value);

/*
   IVADMIN command line equivalent: object object-name delete attr-name

   This will delete the extended attribute pair associated with attr-name
   from the object

   Possible reasons returned:  None

   ctx          - Context to communicate with the management server
   objid        - The object to work with
   attr_name    - The name of the extened attribute to delete
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attrdelkey(ivadmin_context ctx,
                               const char *objid,
                               const char *attr_name,
                               ivadmin_response *rsp);

/*
   Deletes a specific value from an extended attribute key

   Possible reasons returned:  None

   ctx          - Context to talk to the management server
   popid        - the name of the protected object
   attr_key     - the extended attr_key
   attr_value   - the extended attr value to delete from the key
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_protobj_attrdelval(ivadmin_context ctx,
                                                                         char *objid,
                                                                         char *attr_key,
                                                                         char *attr_value,
                                                                         ivadmin_response *rsp);
/*
   IVADMIN command line equivalent: object object-name listextendedattributes

   This will list all of the extended attributes associated with a protected
   object

   Possible reasons returned:  None

   ctx          - Context to communicate with the management server
   objid        - The object to work with
   attrs_list   - The list of attributes returned, should be freed
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attrlist(ivadmin_protobj protobj,
                            unsigned long *count,
                            char ***attrs_list);

/*
   DEPRECATED - This API has been deprecated.  It is only maintained
                for binary-backward compatibility.  It will be removed
                in the future.  Use the ivadmin_protobj_list3 api instead.

   IVADMIN command line equivalent:  object list <object-name>

   This will return the protected objects under the specified directory.  It only returns objects
   immediately under this directory and does not include subdirectories.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   objid       - Parent directory.
   count       - Number of protected objects returned.
   objs        - Array of pointers to protected objects.  You should free each protected object
                 and the array of pointers when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_list2(ivadmin_context ctx,
                        const char *objid,
                        unsigned long *count,
                        char ***objs,
                        ivadmin_response *rsp);
*/

/*
   IVADMIN command line equivalent:  object list <object-name>

   This will return the protected objects under the specified
   directory.  It only returns objects immediately under this
   directory and does not include subdirectories. This API will
   replace ivadmin_protobj_list2 and ivadmin_protobj_list2 will
   be deprecated.

   Possible reasons returned:

   IVADMIN_REASON_AZNADMINSVC_FAILURE - Although The AZN Administration
       Service implementer was successfully contacted, he failed to
       perform the requested task.  Additional information will be
       returned by the implementer in the result strings and/or outdata.

   ctx         - Context to communicate with management server.

   objid       - Parent object name.

   indata      - Pass through data that would allow a custom user
                 interface to communicate additional information
                 to the server.  A valid address of an azn_attrlist_h_t
                 is expected on input however if NULL is specified
                 the parameter is ignored.  It is also assumed the
                 caller has created this azn_attrlist_h_t using
                 azn_attrlist_create() prior to making this API
                 call. When the data is no longer required, the
                 caller should free the associated memory with
                 azn_attrlist_delete().

   objcount    - Number of object names returned.

   objs        - Object names.  This is the list of object names
                 that exist directly below the specified parent object.
                 You should free each object name character pointer
                 and the array of pointers when  no longer needed
                 using ivadmin_free().

   outdata     - Pass through data that would allow a custom user
                 interface to communicate additional information
                 to the server.  A valid address of an azn_attrlist_h_t
                 is expected on input however if NULL is specified
                 the parameter is ignored.  It is also assumed the
                 caller has created this azn_attrlist_h_t using
                 azn_attrlist_create() prior to making this API
                 call. When the data is no longer required, the
                 caller should free the associated memory with
                 azn_attrlist_delete().

   resultcount - Count of result strings returned by the server.

   results     - Result strings.  These are the message strings that
                 are returned by the task.  These would typically show
                 up as output on a CLI interface or log output and
                 contain information about the success or failure
                 of the task.  You should free each result character
                 pointer and the array of pointers when  no longer
                 needed using ivadmin_free.

   rsp         - Success or failure.  This should be freed when no
                 longer needed using ivadmin_free().

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
ivadmin_protobj_list3(ivadmin_context ctx,
                      const char *objid,
                      azn_attrlist_h_t *indata,
                      unsigned long *objcount,
                      char ***objs,
                      azn_attrlist_h_t *outdata,
                      unsigned long *resultcount,
                      char ***results,
                      ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl find acl-name

   This will return a list of protected objects which have the specified ACL attached.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - Attached ACL ID.
   count       - Number of protected objects IDs returned.
   objids      - Array of pointers to protected objects IDs.  You should free each
                 protected object ID pointer and the array of pointers when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_listbyacl(ivadmin_context ctx,
                             const char *aclid,
                             unsigned long *count,
                             char ***objids,
                             ivadmin_response *rsp);

/*
   DEPRECATED - This API has been deprecated.  It is only maintained
                for binary-backward compatibility.  It will be removed
                in the future.  Use the ivadmin_protobj_get2 api instead.

   IVADMIN command line equivalent:  object show object-name

   This will return the protected object specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   objid       - Protected object ID.
   obj         - Returned protected object.  You should free this memory when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_get(ivadmin_context ctx,
                       const char *objid,
                       ivadmin_protobj *obj,
                       ivadmin_response *rsp);
*/

/*
   IVADMIN command line equivalent:  object show <object name>.

   This will return the protected object specified.  This API
   will replace ivadmin_protobj_get and ivadmin_protobj_get
   will be deprecated.

   Possible reasons returned:

   IVADMIN_REASON_AZNADMINSVC_FAILURE - Although The AZN Administration
       Service implementer was successfully contacted, he failed to
       perform the requested task.  Additional information will be
       returned by the implementer in the result strings and/or outdata.

   ctx         - Context to communicate with management server.

   objid       - Name of protected object to return

   indata      - Pass through data that would allow a custom user
                 interface to communicate additional information
                 to the server.  A valid address of an azn_attrlist_h_t
                 is expected on input however if NULL is specified
                 the parameter is ignored.  It is also assumed the
                 caller has created this azn_attrlist_h_t using
                 azn_attrlist_create() prior to making this API
                 call. When the data is no longer required, the
                 caller should free the associated memory with
                 azn_attrlist_delete().

   obj         - Returned protected object.

   outdata     - Pass through data that would allow a custom user
                 interface to communicate additional information
                 to the server.  A valid address of an azn_attrlist_h_t
                 is expected on input however if NULL is specified
                 the parameter is ignored.  It is also assumed the
                 caller has created this azn_attrlist_h_t using
                 azn_attrlist_create() prior to making this API
                 call. When the data is no longer required, the
                 caller should free the associated memory with
                 azn_attrlist_delete().

   resultcount - Count of result strings returned by the server.

   results     - Result strings.  These are the message strings that
                 are returned by the task.  These would typically show
                 up as output on a CLI interface or log output and
                 contain information about the success or failure
                 of the task.  You should free each result character
                 pointer and the array of pointers when  no longer
                 needed using ivadmin_free.

   rsp         - Success or failure.  This should be freed when no
                 longer needed using ivadmin_free().
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
ivadmin_protobj_get2(ivadmin_context ctx,
                     const char *objid,
                     azn_attrlist_h_t *indata,
                     ivadmin_protobj *obj,
                     azn_attrlist_h_t *outdata,
                     unsigned long *resultcount,
                     char ***results,
                     ivadmin_response *rsp);
/*
   IVADMIN command line equivalent:  acl attach obj-name acl-name

   Attach the specifed ACL to the object.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   objid       - Object ID.
   aclid       - ACL ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attachacl(ivadmin_context ctx,
                             const char *objid,
                             const char *aclid,
                             ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl detach obj-name

   Detach the ACL from the protected object.  Since there can only be one ACL
   attached at a time to an object, the currently attached ACL is detached..

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   objid       - Object ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_detachacl(ivadmin_context ctx,
                             const char *objid,
                             ivadmin_response *rsp);

/*
   Returns the protected object ID.

   Possible reasons returned:  N/A

   protobj     - Pointer to protected object structure.

   return      - Protected object ID.  This should NOT be freed.  This is data maintained in
                 the protected object structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_getid(ivadmin_protobj protobj);

   /* protected object types */
#define IVADMIN_PROTOBJ_TYPE_UNKNOWN           0
#define IVADMIN_PROTOBJ_TYPE__DOMAIN           1
#define IVADMIN_PROTOBJ_TYPE__FILE             2
#define IVADMIN_PROTOBJ_TYPE__PROGRAM          3
#define IVADMIN_PROTOBJ_TYPE__DIR              4
#define IVADMIN_PROTOBJ_TYPE__JNCT             5
#define IVADMIN_PROTOBJ_TYPE__WEBSEAL_SVR      6
#define IVADMIN_PROTOBJ_TYPE__NETSEAL_SVR      7
#define IVADMIN_PROTOBJ_TYPE__EXTERN_AUTH_SVR  8
#define IVADMIN_PROTOBJ_TYPE__HTTP_SVR         9
#define IVADMIN_PROTOBJ_TYPE__NON_EXIST_OBJ    10
#define IVADMIN_PROTOBJ_TYPE__CONTAINER        11
#define IVADMIN_PROTOBJ_TYPE__LEAF             12
#define IVADMIN_PROTOBJ_TYPE__PORT             13
#define IVADMIN_PROTOBJ_TYPE__APP_CONTAINER    14
#define IVADMIN_PROTOBJ_TYPE__APP_LEAF         15
#define IVADMIN_PROTOBJ_TYPE__MGMT_OBJ         16
#define IVADMIN_PROTOBJ_TYPE__NETSEAL_NET      17

/*
   Returns the protected object type.

   Possible reasons returned:  N/A

   protobj     - Pointer to protected object structure.

   return      - Protected object type.  The possible types are listed above.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_gettype(ivadmin_protobj protobj);

/*
   Returns the protected object ACL.

   Possible reasons returned:  N/A

   protobj     - Pointer to protected object structure.

   return      - Protected object ACL.  This SHOULD be freed when no longer needed.
*/
IVADMIN_DECLSPEC ivadmin_acl IVADMIN_CALLTYPE
   ivadmin_protobj_getacl(ivadmin_protobj protobj);

/*
   Returns the protected object POP

   Possible reasons returned:  N/A

   protobj      - protected object structure

   return       - POP structure, this SHOULD be freed when no longer needed
*/
IVADMIN_DECLSPEC ivadmin_pop IVADMIN_CALLTYPE
   ivadmin_protobj_getpop(ivadmin_protobj protobj);

/*
   Gets the description of a protected object

   Possible reasons returned:  N/A

   protobj      - protected object structure

   return       - The description, this should not be FREED
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_getdesc(ivadmin_protobj protobj);

/*
   Gets the isPolicyAttachable attribute of a protected object

   Possible reasons returned:  N/A

   protobj      - protected object structure

   returns      - 1 if isPolicyAttachable is true, 0 otherwise
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_getpolicyattachable(ivadmin_protobj protobj);

/*
   Creates a POP (Protected Object Policy) Object

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - A pop already exists with the
                                       specified popid.

   ctx          - context to communicate with the management server
   popid        - the name of the POP to create
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_create(ivadmin_context ctx,
                                    const char *popid,
                                    ivadmin_response *rsp);
/*
   Delete a POP Object

   Possible reasons returned:  None

   ctx          - context to communicate with the management server
   popid        - the name of the POP to delete
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_delete(ivadmin_context ctx,
                                               const char *popid,
                                               ivadmin_response *rsp);

/*
   Lists all POP objects

   Possible reasons returned:  None

   ctx          - context to communicate with the management server
   poplist      - list of POPs returned
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_list(ivadmin_context ctx,
                                                                 unsigned long *count,
                                             char ***poplist,
                                             ivadmin_response *rsp);
/*
   Attaches a POP to a protected object

   Possible reasons returned:  None

   ctx          - context to communicte with the management server
   popid        - the name of the POP to attach
   objid        - the name of the protected object
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_attach(ivadmin_context ctx,
                                                  char *popid,
                                                  char *objid,
                                                  ivadmin_response *rsp);

/*
   Detaches a POP from a protected object

   Possible reasons returned:  None

   ctx          - context to communicate with the management server
   objid        - the protected object to detach from
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_detach(ivadmin_context ctx,
                                                                   char *objid,
                                                                   ivadmin_response *rsp);


/*
   Finds all protected objects where a POP is attached

   Possible reasons returned:  None

   ctx          - context to communicate with the management server
   popid        - the name of the POP to search for
   count        - number of object in the list
   obj_list     - the returned list of objects
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_find(ivadmin_context ctx,
                                                                 char *popid,
                                                                 unsigned long *count,
                                                                 char ***obj_list,
                                                                 ivadmin_response *rsp);
/*
   Gets a POP

   Possible reasons returned:  None

   ctx          - context to communicate with the management server
   popid        - name of the pop to get
   pop          - the pop returned
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_get(ivadmin_context ctx,
                                                                char *popid,
                                                                ivadmin_pop *pop,
                                                                ivadmin_response *rsp);
/*
   Gets the id of a POP

   Possible reasons returned:  N/A

   pop         - the ivadmin_pop object

   returns     - the id of the POP
                 This data should NOT be freed it is maintained in the ivadmin_pop
                 structure.
*/
IVADMIN_DECLSPEC const char* IVADMIN_CALLTYPE ivadmin_pop_getid(ivadmin_pop pop);

/*
   Gets the Description of a POP

   Possible reasons returned:  N/A

   pop         - the ivadmin_pop object

   returns     - string containing the description of the POP
                 This data should NOT be freed it is maintained in the ivadmin_pop
                 structure.
*/
IVADMIN_DECLSPEC const char* IVADMIN_CALLTYPE ivadmin_pop_getdescription(ivadmin_pop pop);

/*
   Sets the description of a POP

   Possible reasons returned:  N/A

   ctx          - context to communicate with the management server
   popid        - the name of the POP
   desc         - the new description for the POP
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_setdescription(ivadmin_context ctx,
                                                                            char *popid,
                                                                            char *desc,
                                                                            ivadmin_response *rsp);

/*
   Gets the time of day range from a POP

   Possible reasons returned:  None

   pop          - the ivadmin_pop object

   days         - a bitmap of the days
   start        - the minutes for the start of the range
   end          - the minutes for the end of the range
   reference    - utc or local
   returns      - IVADMIN_TRUE on success
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_gettod(ivadmin_pop pop,
                                                                 unsigned long *days,
                                                                 unsigned long *start,
                                                                 unsigned long *end,
                                                                 unsigned long *reference);

/*
   Sets the time of day range for a POP

   Possible reasons returned:  None

   ctx           - context to talk to the management server
   popid         - the name of the POP
   days         - a bitmap of the days
   start        - the minutes for the start of the range
   end          - the minutes for the end of the range
   reference    - utc or local
   rsp           - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_settod(ivadmin_context ctx,
                                                                    char *popid,
                                                                    unsigned long days,
                                                                    unsigned long start,
                                                                    unsigned long end,
                                                                    unsigned long reference,
                                                                    ivadmin_response *rsp);

/*
   Gets the anyothernw setting for authentication level from a POP

   Possible reasons returned:  None

   ctx          - context to talk to the management server
   pop_handle   - handle of the POP
   level        - pointer to authentication level returned
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_getanyothernw(ivadmin_pop pop,
                                                                        unsigned long *level);



/*
   Sets the anyothernw setting for authentication level from a POP

   Possible reasons returned:  None

   ctx           - context to talk to the management server
   popid         - the name of the POP
   level         - The authentication level to associate with anyothernw
   rsp           - Response

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_setanyothernw(ivadmin_context ctx,
                                                                        char *popid,
                                                                        unsigned long level,
                                                                        ivadmin_response *rsp);


/*
   Sets the anyothernw setting from a POP to forbidden

   Possible reasons returned:  None

   ctx           - context to talk to the management server
   popid         - the name of the POP
   level         - The authentication level to associate with anyothernw
   rsp           - Response

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_setanyothernw_forbidden(
                        ivadmin_context ctx,
                        char *popid,
                        ivadmin_response *rsp);


/*
   Gets the ipauth settings from a POP

   Possible reasons returned:  None

   pop          - the POP
   count        - returns the number of settings retrieved
   networks     - returns the array of networks
   netmasks     - returns the array of netmasks
   authMethods  - returns the array of authentication methods associated
   rsp          - Response
*/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_getipauth(ivadmin_pop pop,
                                                                      unsigned long *count,
                                                                      unsigned long **networks,
                                                                      unsigned long **netmasks,
                                                                      unsigned long **authMethods);




/*
   Sets the ipauth setting for authentication level from a POP

   Possible reasons returned:  None

   ctx           - context to talk to the management server
   popid         - the name of the POP
   network       - the network address
   netmask       - the netmask address
   authMethod    - the authentication level to associate with the network
   rsp           - Response

*/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_setipauth(ivadmin_context ctx,
                                                                        char *popid,
                                                                        unsigned long network,
                                                                        unsigned long netmask,
                                                                        unsigned long authMethod,
                                                                        ivadmin_response *rsp);


/*
   Sets the ipauth setting for authentication level from a POP to forbidden

   Possible reasons returned:  None

   ctx           - context to talk to the management server
   popid         - the name of the POP
   network       - the network address
   netmask       - the netmask address
   rsp           - Response

*/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_setipauth_forbidden(ivadmin_context ctx,
                                                                        char *popid,
                                                                        unsigned long network,
                                                                        unsigned long netmask,
                                                                        ivadmin_response *rsp);


/*
   Removes the ipauth setting for authentication level from a POP

   Possible reasons returned:  None

   ctx           - context to talk to the management server
   popid         - the name of the POP
   network       - The network address to delete
   rsp           - Response
*/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_removeipauth(ivadmin_context ctx,
                                                                        char *popid,
                                                                        char *network,
                                    char *netmask,
                                                                        ivadmin_response *rsp);
/*
   Gets the warning mode value from a POP

   pop          - the ivadmin_pop object

   returns      - IVADMIN_TRUE if waring mode is True for the POP, IVADMIN_FALSE
                otherwise
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_getwarnmode(ivadmin_pop pop);


/*
   Sets the warning mode for a POP

   Possible reasons returned:  None

   ctx          - context to talk to the management server
   popid        - the name of the POP
   warn_mode    - the new value of warn mode for the POP should be:
                IVADMIN_TRUE or IVADMIN_FALSE
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_setwarnmode(ivadmin_context ctx,
                                                                         char *popid,
                                                                         unsigned long warn_mode,
                                                                         ivadmin_response *rsp);

/*
   Gets the Audit Level from a POP

   Possible reasons returned:  N/A

   pop          - the ivadmin_pop object

   returns      - The auditlevel which can be:
                IVADMIN_AUDIT_NONE (0)
                IVADMIN_AUDIT_PERMIT (1)
                IVADMIN_AUDIT_DENY (2)
                IVADMIN_AUDIT_ERROR (4)
                IVADMIN_AUDIT_ADMIN (8)
                IVADMIN_AUDIT_ALL (15)
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_getauditlevel(ivadmin_pop pop);

/*
   Sets the Audit Level for a POP

   Possible reasons returned:  None

   ctx          - contex for talking to the management server
   popid        - the name of the POP
   audit_level  - the new audit level for the the POP.  This unsigned long should be one of:
                IVADMIN_AUDIT_NONE (0)
                IVADMIN_AUDIT_PERMIT (1)
                IVADMIN_AUDIT_DENY (2)
                IVADMIN_AUDIT_ERROR (4)
                IVADMIN_AUDIT_ADMIN (8)
                IVADMIN_AUDIT_ALL (15)
   rsp          - Reponse
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_setauditlevel(ivadmin_context ctx,
                                                                           char *popid,
                                                                           unsigned long audit_level,
                                                                           ivadmin_response *rsp);

/*
   Gets the Quality of Protection (qop) level from a POP

   Possible reasons returned:  N/A

   pop          - the ivadmin_pop object

   returns      - a string with the qop level which can be:
                [none|integrity|privacy]
                  This data should NOT be freed it is maintained in the ivadmin_pop
                  structure
*/
IVADMIN_DECLSPEC const char* IVADMIN_CALLTYPE ivadmin_pop_getqop(ivadmin_pop pop);

/*
   Sets the Quality of Protection (qop) for a POP

   Possible reasons returned:  None

   ctx          - context for communicating with the management server
   popid        - name of the POP
   qop_level    - the new qop level to set this string should be:
                [none|integrity|privacy]
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_setqop(ivadmin_context ctx,
                                                                    char *popid,
                                                                    char *qop_level,
                                                                    ivadmin_response *rsp);

/*
   Lists the extended attribute keys associated with the POP

   Possible reasons returned:  None

   pop          - ivadmin pop object
   count        - the number of extended attributes
   attr_list    - the extended attributes, should be freed
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_attrlist(ivadmin_pop pop,
                                                                     unsigned long *count,
                                                                     char ***attr_list);

/*
   Gets the extended attribute value for a particular key from a POP

   Possible reasons returned:  N/A

   pop          - ivadmin pop object
   attr_key     - the key to look up
   count        - the number of values returned
   attr_value   - the list of values returned (should be freed)
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_attrget(ivadmin_pop pop,
                                                                     char *attr_key,
                                                                     unsigned long *count,
                                                                     char ***attr_value);

/*
   Sets the extended attribute value for a particular key for a POP

   Possible reasons returned:  None

   ctx          - context to talk to the management server
   popid        - the name of the POP
   attr_key     - the key to set a value for
   attr_value   - the value to set
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_attrput(ivadmin_context ctx,
                                                                     char *popid,
                                                                     char *attr_key,
                                                                     char *attr_value,
                                                                     ivadmin_response *rsp);

/*
   Deletes an extended attribute from a POP

   Possible reasons returned:  None

   ctx          - context to talk to the management server
   popid        - the name of the POP
   attr_key     - the extended attribute to delete
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_attrdelkey(ivadmin_context ctx,
                                                                     char *popid,
                                                                     char *attr_key,
                                                                     ivadmin_response *rsp);


/*
   Deletes a specific value from an extended attribute key

   Possible reasons returned:  None

   ctx          - Context to talk to the management server
   popid        - the name of the POP
   attr_key     - the extended attr_key
   attr_value   - the extended attr value to delete from the key
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_pop_attrdelval(ivadmin_context ctx,
                                                                         char *popid,
                                                                         char *attr_key,
                                                                         char *attr_value,
                                                                         ivadmin_response *rsp);
/*
   IVADMIN command line equivalent:  action create name description action-type

   Defines a new action (permission) code in the "primary" action group.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - A action already exists with the specified actionid.

   ctx         - Context to communicate with management server.
   actionid    - Action ID.  This must be a single letter code that does not conflict
                 with existing permission codes.  The input is left as a string for future
                 expansion.
   description - Description of permission code.  This will appear in the management console.
   type        - Label for action category.  This will appear in the management console.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_create(ivadmin_context ctx,
                         const char *actionid,
                         const char *description,
                         const char *type,
                         ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  action create name description action-type action-group-name

   Defines a new action (permission) code in the specified action group.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - A action already exists with the specified actionid.

   ctx         - Context to communicate with management server.
   actionid    - Action ID.  This must be a single letter code that does not conflict
                 with existing permission codes.  The input is left as a string for future
                 expansion.
   description - Description of permission code.  This will appear in the management console.
   type        - Label for action category.  This will appear in the management console.
   groupname   - name of the action group to create the action in.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_create_in_group(ivadmin_context ctx,
                                  const char *actionid,
                                  const char *description,
                                  const char *type,
                                  const char *groupname,
                                  ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  action delete name

   Deletes an action (permission) code from the "primary" action group.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   actionid    - Action ID.  This must be a single letter code that identifies the
                 permission to delete.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_delete(ivadmin_context ctx,
                         const char *actionid,
                         ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  action delete name action-group-name

   Deletes an action (permission) code from the specified action group.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   actionid    - Action ID.  This must be a single letter code that identifies the
                 permission to delete.
   groupname   - name of the action group to delete the action from.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_delete_from_group(ivadmin_context ctx,
                                    const char *actionid,
                                    const char *groupname,
                                    ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  action list

   This will return all of the defined action (permission) codes from the "primary" action group.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   count       - Number of actions returned.
   actions     - Array of pointers to actions.  You should free each action pointer
                 and the array of pointers when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_list(ivadmin_context ctx,
                       unsigned long *count,
                       ivadmin_action **actions,
                       ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  action list action-group-name

   This will return all of the defined action (permission) codes from the specified action group.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   count       - Number of actions returned.
   actions     - Array of pointers to actions.  You should free each action pointer
                 and the array of pointers when no longer needed.
   groupname   - name of the action group to list.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_list_in_group(ivadmin_context ctx,
                                const char *actiongroup,
                                unsigned long *count,
                                ivadmin_action **actions,
                                ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  action group list

   This will return all of the defined action group names.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   count       - Number of names returned.
   names       - Array of pointers to group name strings.  You should free
                 each group name string and the array of pointers when no
                 longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_group_list(ivadmin_context ctx,
                             unsigned long *count,
                             char ***names,
                             ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  action group delete action-group-name

   This will delete the action group and all it's actions.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupname   - name of the action group to delete.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_group_delete(ivadmin_context ctx,
                               const char *groupname,
                               ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  action group create action-group-name

   This will create a new action group.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   groupname   - name of the action group to delete.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_group_create(ivadmin_context ctx,
                               const char *groupname,
                               ivadmin_response *rsp);
/*
   Returns the action (permission) code.

   Possible reasons returned:  N/A

   action      - Pointer to action.

   return      - Action code.  This should NOT be freed.  This is data maintained in
                 the action structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_action_getid(ivadmin_action action);

/*
   Returns the action (permission) description.

   Possible reasons returned:  N/A

   action      - Pointer to action.

   return      - Action description.  This should NOT be freed.  This is data maintained in
                 the action structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_action_getdescription(ivadmin_action action);

/*
   Returns the action (permission) type.

   Possible reasons returned:  N/A

   action      - Pointer to action.

   return      - Action type.  This should NOT be freed.  This is data maintained in
                 the action structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_action_gettype(ivadmin_action action);

/*
   IVADMIN command line equivalent:  acl create acl-name

   Creates a new ACL.  The default permissions in a new ACL are:

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - A acl already exists with the specified aclid.

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_create(ivadmin_context ctx,
                      const char *aclid,
                      ivadmin_response *rsp);

/*
   Delete ACL

   IVADMIN command line equivalent:  acl delete acl-name

   Deletes the specified ACL.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_delete(ivadmin_context ctx,
                      const char *aclid,
                      ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl list

   This will return the IDs of all of the defined ACLs..

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   count       - Number of ACL IDs returned.
   aclids      - Array of pointers to ACL IDs.  You should free each ACL ID pointer
                 and the array of pointers when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_list(ivadmin_context ctx,
                    unsigned long *count,
                    char ***aclids,
                    ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl show acl-name

   This will return the ACL specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   acl         - Returned ACL.  You should free this memory when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_get(ivadmin_context ctx,
                   const char *aclid,
                   ivadmin_acl *acl,
                   ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl modify acl-name description desc

   Change the ACL description.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   description - New description.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setdescription(ivadmin_context ctx,
                              const char *aclid,
                              const char *description,
                              ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl modify acl-name remove user user-name

   Remove a user ACL entry from the ACL.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   userid      - The ACL entry for this user is removed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_removeuser(ivadmin_context ctx,
                          const char *aclid,
                          const char *userid,
                          ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl modify acl-name remove group group-name

   Remove a group ACL entry from the ACL.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   groupid     - The ACL entry for this group is removed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_removegroup(ivadmin_context ctx,
                           const char *aclid,
                           const char *groupid,
                           ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl modify acl-name remove any-other

   Remove the any-other ACL entry from the ACL.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_removeanyother(ivadmin_context ctx,
                              const char *aclid,
                              ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl modify acl-name remove unauthenticated

   Remove the unauthenticated ACL entry from the ACL.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_removeunauth(ivadmin_context ctx,
                            const char *aclid,
                            ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl modify acl-name set user user-name perms

   Set a user ACL entry in the ACL.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   userid      - The ACL entry for this user is set.
   actions     - The new permissions for this ACL entry.  This is a string consisting of
                 single letter permission codes.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setuser(ivadmin_context ctx,
                       const char *aclid,
                       const char *userid,
                       const char *actions,
                       ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl modify acl-name set group group-name perms

   Set a group ACL entry in the ACL.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   groupid     - The ACL entry for this group is set.
   actions     - The new permissions for this ACL entry.  This is a string consisting of
                 single letter permission codes.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setgroup(ivadmin_context ctx,
                        const char *aclid,
                        const char *groupid,
                        const char *actions,
                        ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl modify acl-name set any-other perms

   Set the any-other ACL entry in the ACL.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   actions     - The new permissions for this ACL entry.  This is a string consisting of
                 single letter permission codes.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setanyother(ivadmin_context ctx,
                           const char *aclid,
                           const char *actions,
                           ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  acl modify acl-name set unauthenticated perms

   Set the unauthenticated ACL entry in the ACL.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   aclid       - ACL ID.
   actions     - The new permissions for this ACL entry.  This is a string consisting of
                 single letter permission codes.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setunauth(ivadmin_context ctx,
                         const char *aclid,
                         const char *actions,
                         ivadmin_response *rsp);

/*
   Returns the ACL indentifier.

   Possible reasons returned:  N/A

   acl         - Pointer to ACL.

   return      - ACL ID.  This should NOT be freed.  This is data maintained in
                 the acl structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getid(ivadmin_acl acl);

/*
   Returns the ACL description.

   Possible reasons returned:  N/A

   acl         - Pointer to ACL.

   return      - ACL description.  This should NOT be freed.  This is data maintained in
                 the acl structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getdescription(ivadmin_acl acl);

/*
   IVADMIN command line equivalent:

   This will return a list of user IDs specified in the ACL.

   Possible reasons returned:  N/A

   ctx         - Context to communicate with management server.
   count       - Number of user IDs returned.
   userids     - Array of pointers to user IDs.  You should free each user ID pointer
                 and the array of pointers when no longer needed.

   return      - IVADMIN_TRUE (Success.)
                 IVADMIN_FALSE (Invalid parameter or memory allocation error.)
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_listusers(ivadmin_acl acl,
                         unsigned long *count,
                         char ***userids);

/*
   IVADMIN command line equivalent:

   This will return a list of group IDs specified in the ACL.

   Possible reasons returned:  N/A

   ctx         - Context to communicate with management server.
   count       - Number of group IDs returned.
   groupids    - Array of pointers to group IDs.  You should free each group ID pointer
                 and the array of pointers when no longer needed.

   return      - IVADMIN_TRUE (Success)
                 IVADMIN_FALSE (Invalid parameter or memory allocation error.)
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_listgroups(ivadmin_acl acl,
                          unsigned long *count,
                          char ***groupids);
/*
   Returns the actions (permissions) defined for a user ACL entry.

   Possible reasons returned:  N/A

   acl         - Pointer to ACL.
   userid      - Return action for this user ID.

   return      - Actions for the user.  This SHOULD be freed when no longer needed.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getuser(ivadmin_acl acl,
                       const char *userid);

/*
   Returns the actions (permissions) defined for a group ACL entry .

   Possible reasons returned:  N/A

   acl         - Pointer to ACL.
   groupid     - Return action for this group ID.

   return      - Actions for the group.  This SHOULD be freed when no longer needed.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getgroup(ivadmin_acl acl,
                        const char *groupid);

/*
   Returns the actions (permissions) defined for the any-other ACL entry.

   Possible reasons returned:  N/A

   acl         - Pointer to ACL.

   return      - Actions for the any-other ACL entry.  This SHOULD be freed when no longer needed.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getanyother(ivadmin_acl acl);

/*
   Returns the actions (permissions) defined for the unauthenticated ACL entry.

   Possible reasons returned:  N/A

   acl         - Pointer to ACL.

   return      - Actions for the unauthenticated ACL entry.  This SHOULD be freed when no longer needed.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getunauth(ivadmin_acl acl);


/*
   Lists the extended attribute keys associated with the ACL

   Possible reasons returned:  N/A

   acl          - the ivadmin acl object
   count        - the number of extended attributes
   attr_list    - the extended attributes, should be freed
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_acl_attrlist(ivadmin_acl acl,
                                                                     unsigned long *count,
                                                                     char ***attr_list);

/*
   Gets the extended attribute value for a particular key from a ACL

   Possible reasons returned:  N/A

   acl          - the ivadmin acl object
   attr_key     - the key to look up
   count        - the number of values returned
   attr_value   - the list of values returned (should be freed)
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_acl_attrget(ivadmin_acl acl,
                                                                     char *attr_key,
                                                                     unsigned long *count,
                                                                     char ***attr_value);

/*
   Sets the extended attribute value for a particular key for a ACL

   Possible reasons returned:  None

   ctx          - context to talk to the management server
   aclid        - the name of the ACL
   attr_key     - the key to set a value for
   attr_value   - the value to set
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_acl_attrput(ivadmin_context ctx,
                                                                     char *aclid,
                                                                     char *attr_key,
                                                                     char *attr_value,
                                                                     ivadmin_response *rsp);

/*
   Deletes an extended attribute key from a ACL

   Possible reasons returned:  None

   ctx          - context to talk to the management server
   aclid        - the name of the ACL
   attr_key     - the extended attribute to delete
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_acl_attrdelkey(ivadmin_context ctx,
                                                                     char *aclid,
                                                                     char *attr_key,
                                                                     ivadmin_response *rsp);
/*
   Deletes a specific value from an extended attribute key

   Possible reasons returned:  None

   ctx          - Context to talk to the management server
   aclid        - the name of the ACL
   attr_key     - the extended attr_key
   attr_value   - the extended attr value to delete from the key
   rsp          - Response
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE ivadmin_acl_attrdelval(ivadmin_context ctx,
                                                                         char *aclid,
                                                                         char *attr_key,
                                                                         char *attr_value,
                                                                         ivadmin_response *rsp);
/*
   IVADMIN command line equivalent:  rsrc list

   This will return a list of all of the SSO web resource IDs.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   count       - Number of SSO web resource IDs returned.
   ssowebids   - Array of pointers to SSO web resource IDs.  You should free each SSO web resource ID pointer
                 and the array of pointers when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssoweb_list(ivadmin_context ctx,
                       unsigned long *count,
                       char ***ssowebids,
                       ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrc show resource-name

   This will return the SSO web resource specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   ssowebid    - SSO web resource ID.
   ssoweb      - Returned SSO web resource.  You should free this memory when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssoweb_get(ivadmin_context ctx,
                      const char *ssowebid,
                      ivadmin_ssoweb *ssoweb,
                      ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrc create resource-name[-desc description]

   This will create an SSO web resource.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - An SSO web resource already exists
                                       with the specified ssowebid.

   ctx         - Context to communicate with management server.
   ssowebid    - SSO web resource ID.
   description - Description of the SSO web resource..
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssoweb_create(ivadmin_context ctx,
                         const char *ssowebid,
                         const char *description,
                         ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrc delete resource-name

   This will delete an SSO web resource.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   ssowebid    - SSO web resource ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssoweb_delete(ivadmin_context ctx,
                         const char *ssowebid,
                         ivadmin_response *rsp);

/*
   Returns the ID of the SSO web resource.

   Possible reasons returned:  N/A

   ssoweb      - Pointer to SSO web resource.

   return      - SSO web resource ID.  This should NOT be freed.  This is data maintained in the SSO web resource structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssoweb_getid(ivadmin_ssoweb ssoweb);

/*
   Returns the description of the SSO web resource.

   Possible reasons returned:  N/A

   ssoweb      - Pointer to SSO web resource.

   return      - SSO web resource description.  This should NOT be freed.  This is data maintained in the SSO web resource structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssoweb_getdescription(ivadmin_ssoweb ssoweb);

/*
   IVADMIN command line equivalent:  rsrcgroup list

   This will return a list of all of the SSO group resource IDs.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   count       - Number of SSO group resource IDs returned.
   ssogroupids - Array of pointers to SSO group resource IDs.  You should free each SSO group resource ID pointer
                 and the array of pointers when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_list(ivadmin_context ctx,
                         unsigned long *count,
                         char ***ssogroupids,
                         ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrcgroup show resource-group-name

   This will return the SSO group resource specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   ssogroupid  - SSO group resource ID.
   ssogroup    - Returned SSO group resource.  You should free this memory when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_get(ivadmin_context ctx,
                        const char *ssogroupid,
                        ivadmin_ssogroup *ssogroup,
                        ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrcgroup create resource-group-name[-desc description]

   This will create an SSO group resource.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - An SSO group resource already exists
                                       with the specified ssogroupid.

   ctx         - Context to communicate with management server.
   ssogroupid  - SSO group resource ID.
   description - Description of the SSO group resource..
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_create(ivadmin_context ctx,
                           const char *ssogroupid,
                           const char *description,
                           ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrcgroup delete resource-group-name

   This will delete an SSO group resource.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   ssogroupid  - SSO group resource ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_delete(ivadmin_context ctx,
                           const char *ssogroupid,
                           ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrcgroup modify resource-group-nameadd rsrcname resource-name

   Add an SSO resource to an SSO resource group.  Policy Director does not support a resource group as a resource group member.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   ssogroupid  - SSO resource group ID.
   ssoid       - New member SSO resource ID.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_addres(ivadmin_context ctx,
                           const char *ssogroupid,
                           const char *ssoid,
                           ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrcgroup modify resource-group-nameremove rsrcname resource-name

   Remove an SSO resource from an SSO resource group.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   ssogroupid  - SSO resource group ID.
   ssoid       - Member SSO resource ID to remove.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_removeres(ivadmin_context ctx,
                              const char *ssogroupid,
                              const char *ssoid,
                              ivadmin_response *rsp);

/*
   Returns the ID of the SSO group resource.

   Possible reasons returned:  N/A

   ssogroup    - Pointer to SSO group resource.

   return      - SSO group resource ID.  This should NOT be freed.  This is data maintained in the SSO group resource structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssogroup_getid(ivadmin_ssogroup ssogroup);

/*
   Returns the description of the SSO group resource.

   Possible reasons returned:  N/A

   ssogroup    - Pointer to SSO group resource.

   return      - SSO group resource description.  This should NOT be freed.  This is data maintained in the SSO group resource structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssogroup_getdescription(ivadmin_ssogroup ssogroup);

/*
   Return list of member SSO resource IDs.

   Possible reasons returned:  N/A

   ssogroup    - Pointer to SSO group resource.
   count       - Number of SSO resource IDs returned.
   ssoids      - Array of pointers to SSO resource IDs.  You should free each SSO resource ID pointer
                 and the array of pointers when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_getresources(ivadmin_ssogroup ssogroup,
                                 unsigned long *count,
                                 char ***ssoids);


/*
   IVADMIN command line equivalent:  rsrccred list user user-name

   This will return the defined resource names and resource types for the specified user.
   This information is returned within an array of pointers to ivadmin_ssocred structures.


   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   count       - Number of SSO credentials returned.
   userid      - Retrieve SSO credentials for this user.
   ssocreds    - Array of pointers to SSO credentials.  You should free each SSO credential pointer
                 and the array of pointers when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_list(ivadmin_context ctx,
                        const char *userid,
                        unsigned long *count,
                        ivadmin_ssocred **ssocreds,
                        ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrccred show resource-namersrctype {web | group} user user-name

   This will return the SSO credential specified.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   ssoid       - SSO resource ID with which the SSO credential is associated.
   ssotype     - SSO resource type (IVADMIN_SSOCRED_SSOWEB/IVADMIN_SSOCRED_SSOGROUP)
   userid      - User ID associated with the SSO credential.
   ssocred     - Returned SSO credential.  You should free this memory when no longer needed.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/

/* sso credential types */
#define IVADMIN_SSOCRED_SSOWEB     0
#define IVADMIN_SSOCRED_SSOGROUP   1
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_get(ivadmin_context ctx,
                       const char *ssoid,
                       unsigned long ssotype,
                       const char *userid,
                       ivadmin_ssocred *ssocred,
                       ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrccred create resource-namersrcuser resource-userid rsrcpwd resource-password
                                        rsrctype {web | group} user user-name

   This will create an SSO credential.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - An SSO credential already exists with the
                                       specified ssoid, ssotype, and userid.

   ctx         - Context to communicate with management server.
   ssoid       - SSO resource ID with which the SSO credential is associated.
   ssotype     - SSO resource type (IVADMIN_SSOCRED_SSOWEB/IVADMIN_SSOCRED_SSOGROUP)
   userid      - User ID associated with the SSO credential.
   ssouserid   - The user ID that this user uses to access the specified resource.
   ssopassword - The password that this user uses to access the specified resource.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_create(ivadmin_context ctx,
                          const char *ssoid,
                          unsigned long ssotype,
                          const char *userid,
                          const char *ssouserid,
                          const char *ssopassword,
                          ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrccred delete resource-namersrctype {web | group} user user-name

   This will delete an SSO credential.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   ssoid       - SSO resource ID with which the SSO credential is associated.
   ssotype     - SSO resource type (IVADMIN_SSOCRED_SSOWEB/IVADMIN_SSOCRED_SSOGROUP)
   userid      - User ID associated with the SSO credential.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_delete(ivadmin_context ctx,
                          const char *ssoid,
                          unsigned long ssotype,
                          const char *userid,
                          ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  rsrccred modify resource-namersrctype {web | group} set [-rsrcuser resource-userid]
                                        [-rsrcpwd resource-password] user user-name

   This will modify an SSO credential.

   Possible reasons returned:  None

   ctx         - Context to communicate with management server.
   ssoid       - SSO resource ID with which the SSO credential is associated.
   ssotype     - SSO resource type (IVADMIN_SSOCRED_SSOWEB/IVADMIN_SSOCRED_SSOGROUP)
   userid      - User ID associated with the SSO credential.
   ssouserid   - The user ID that this user uses to access the specified resource.
   ssopassword - The password that this user uses to access the specified resource.
   rsp         - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_set(ivadmin_context ctx,
                       const char *ssoid,
                       unsigned long ssotype,
                       const char *userid,
                       const char *ssouserid,
                       const char *ssopassword,
                       ivadmin_response *rsp);

/*
   Returns the ID of the SSO resource associated with this credential.

   Possible reasons returned:  N/A

   ssocred     - Pointer to SSO credential.

   return      - SSO resource ID associated with credential.  This should NOT be freed.
                 This is data maintained in the SSO credential structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssocred_getid(ivadmin_ssocred ssocred);

/*
   Returns the type of the SSO resource associated with this credential.

   Possible reasons returned:  N/A

   ssocred     - Pointer to SSO credential.

   return      - SSO resource type associated with credential (IVADMIN_SSOCRED_SSOWEB/IVADMIN_SSOCRED_SSOGROUP).
                 This should NOT be freed.  This is data maintained in the SSO credential structure.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_gettype(ivadmin_ssocred ssocred);

/*
   Returns the user associated with this credential.

   Possible reasons returned:  N/A

   ssocred     - Pointer to SSO credential.

   return      - User associated with credential.  This should NOT be freed.
                 This is data maintained in the SSO credential structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssocred_getuser(ivadmin_ssocred ssocred);

/*
   Returns the user associated with this credential.

   Possible reasons returned:  N/A

   ssocred     - Pointer to SSO credential.

   return      - The user ID that the credential user uses to access the resource.  This should NOT be freed.
                 This is data maintained in the SSO credential structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssocred_getssouser(ivadmin_ssocred ssocred);

/*
   Returns the password associated with this credential.

   Possible reasons returned:  N/A

   ssocred     - Pointer to SSO credential.

   return      - The password that the credential user uses to access the resource.  This should NOT be freed.
                 This is data maintained in the SSO credential structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssocred_getssopassword(ivadmin_ssocred ssocred);

/*
   Returns a boolean indicator of the success of the operation.

   Possible reasons returned:  N/A

   rsp         - Pointer to response.

   return      - Success (IVADMIN_FALSE/IVADMIN_TRUE).
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getok(ivadmin_response rsp);

/*
   Returns the number of messages contained within the response.

   Possible reasons returned:  N/A

   rsp         - Pointer to response.

   return      - Message count.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getcount(ivadmin_response rsp);

/*
   Returns the error/warning code associated with the message.

   Possible reasons returned:  N/A

   rsp         - Pointer to response.
   index       - Zero based index of mesage code requested.

   return      - Message code.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getcode(ivadmin_response rsp, unsigned long index);
/*
   Returns the reason.  A reason is not always returned.  Only
   certain api's return a reason in addition to other response
   information.  Each api identifies the possible reasons returned.
   The entire set of possible reasons is defined here.

   Possible reasons returned:  N/A

   rsp         - Pointer to response.

   return      - Reason.
*/
#define IVADMIN_REASON_ALREADY_EXISTS 1
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getreason(ivadmin_response rsp);
/*
   Returns the modifier (error, warning, info) associated with the message.

   Possible reasons returned:  N/A

   rsp         - Pointer to response.
   index       - Zero based index of mesage modifier requested.

   return      - Message modifier. (IVADMIN_RESPONSE_INFO/IVADMIN_RESPONSE_WARNING/IVADMIN_RESPONSE_ERROR).
*/
#define IVADMIN_RESPONSE_INFO     0
#define IVADMIN_RESPONSE_WARNING  1
#define IVADMIN_RESPONSE_ERROR    2
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getmodifier(ivadmin_response rsp, unsigned long index);

/*
   Returns the message text associated with the message.

   Possible reasons returned:  N/A

   rsp         - Pointer to response.
   index       - Zero based index of mesage text requested.

   return      - Message text.  This should NOT be freed.  This is data maintained in the response structure.
*/
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_response_getmessage(ivadmin_response rsp, unsigned long index);


/*
   IVADMIN command line equivalent:  svrsslcfg -config ....

   Configures an aznAPI server
     Configuration file will be updated and keyring database created.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - A server already exists.

   ctx              - Context to communicate with management server.
   cfg_file_name    - Configuration file.
   kdb_dir_name     - Keyring database directory.
   server_name      - Unique server name.
   host_name        - Host name on which the application will run.
   server_type      - Server type: local or remote.
   server_pwd       - Server password.  If specified, it must conform to password policy
                      in effect; and will be used to update bind-pwd in configuration file.
                      If NULL, a random password is generated and configuration file is not
                      updated.
   enable_listening - Sets the listening-enabled flag in the configuration file.
   listening_port   - The TCP/IP port on which the application will listen
   enable_refresh   - Enable/disable the certificate auto-refresh support.
   kdb_pwd_life     - The keyring database password life in days.  If 0, a default of 183
                      days will be used.
   ssl_timeout      - The SSL session timeout value in seconds.  If 0, a default of 7200
                      wil be used.
   appl_cert        - The name of the file that contains a base-64 encoded SSL certificate.
                      This is an optional parameter.  If specified, the certificate will be
                      stored in the keyring database using a label of APPL_LDAP_CERT.
                      Typeical use of this parameter is to store the LDAP CA certificate
                      that the application uses when it authenticates directly to LDAP.
   azn_app_host     - The host name to be written to the azn-host-name entry in the configuration
                      file. This is the host name that the application will actually use at
                      runtime.  It is only needed if the name returned by the TCP gethostbyname()
                      is incorrect or different from the host_name parameter above.
                      This parameter was added in release 3.9
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/

typedef enum {local=1,remote} ivadmin_cfg_servertype;
/*
               WARNING   WARNING  WARNING
 Deprecated API.
 ivadmin_cfg_configureserver is deprecated.  Use ivadmin_cfg_configureserver2
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_configureserver(ivadmin_context         ctx,
                               const char              *cfg_file_name,
                               const char              *kdb_dir_name,
                               const char              *server_name,
                               const char              *host_name,
                               ivadmin_cfg_servertype  server_type,
                               const char              *server_pwd,
                               int                     enable_listening,
                               int                     listening_port,
                               int                     enable_refresh,
                               int                     kdb_pwd_life,
                               int                     ssl_timeout,
                               const char              *appl_cert,
                               ivadmin_response        *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_configureserver2(ivadmin_context         ctx,
                               const char              *cfg_file_name,
                               const char              *kdb_dir_name,
                               const char              *server_name,
                               const char              *host_name,
                               ivadmin_cfg_servertype  server_type,
                               const char              *server_pwd,
                               int                     enable_listening,
                               int                     listening_port,
                               int                     enable_refresh,
                               int                     kdb_pwd_life,
                               int                     ssl_timeout,
                               const char              *appl_cert,
                               const char              *azn_app_host,
                               ivadmin_response        *rsp);

/*
   IVADMIN command line equivalent:  svrsslcfg -unconfig ....

   UnConfigures an aznAPI server
     This API never fails.  It will report success even if the server was not configured.
     However, the application must be stopped prior to invoking this API.

   Possible reasons returned:

   ctx              - Context to communicate with management server.
   cfg_file_name    - Configuration file.
   server_name      - Unique server name.
   host_name        - Host name on which the application will run.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_unconfigureserver(ivadmin_context         ctx,
                                 const char              *cfg_file_name,
                                 const char              *server_name,
                                 const char              *host_name,
                                 ivadmin_response        *rsp);

/*
   IVADMIN command line equivalent:  svrsslcfg -chgport ....

   Changes the listening port number of the application.
     The port number in the configuration file will be updated.  The server must be
     stopped and restarted to activate this change.  If the port is set to zero, the
     listen-flags will be set to disable otherwise it is unchanged.

   Possible reasons returned:

   cfg_file_name    - Configuration file.
   listening_port   - The TCP/IP port on which the application will listen.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
      ivadmin_cfg_setport(
                             const char              *cfg_file_name,
                             int                     listening_port,
                             ivadmin_response        *rsp);

/*
   IVADMIN command line equivalent:  svrsslcfg -setlisten ....

   Sets/resets the enable listening parameter in the configuration file.
   If enabling listening, the listening port in the configuration file must
   be non-zero, otherwise an invalid parameter error is returned.
   The application must be stopped and restarted to activate this change.

   Possible reasons returned:

   cfg_file_name    - Configuration file.
   enable_listening - Sets the listening-enabled flag in the configuration file.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
      ivadmin_cfg_setlistening(
                               const char              *cfg_file_name,
                               int                     enable_listening,
                               ivadmin_response        *rsp);

/*
   IVADMIN command line equivalent:  svrsslcfg -chgcert ....

   Renew the server SSL certificate.
    Use thsi API to refresh the certificate used to authenticate with the Management
    Server, if it has expired or been compromised. The application must be stopped
    prior to using this API.

   Possible reasons returned:

   ctx              - Context to communicate with management server.
   cfg_file_name    - Configuration file.
   server_name      - Unique server name.
   host_name        - Host name on which the application will run.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
      ivadmin_cfg_renewservercert(ivadmin_context          ctx,
                                  const char              *cfg_file_name,
                                  const char              *server_name,
                                  const char              *host_name,
                                  ivadmin_response        *rsp);


/*
   IVADMIN command line equivalent:  svrsslcfg -chgpwd ...

   Refresh/change the keyring database password.
     Use this API to refresh/change the keyring database random password.  A new random
     password will be created and stashed in the stash file.  The application must be
     stopped to execute this API.

   Possible reasons returned:


   cfg_file_name    - Configuration file.
   kdb_pwd_life     - The keyring database password life in days.  If 0, a default of 183
                      days will be used.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_setkeyringpwd(
                                const char              *cfg_file_name,
                                int                     kdb_pwd_life,
                                ivadmin_response        *rsp);

/*
   IVADMIN command line equivalent:  svrsslcfg -modify

   Changes the SSL timeout value in the configuration file.
   The application must be stopped and restarted to activate this change.

   Possible reasons returned:


   cfg_file_name    - Configuration file.
   ssl_timeout      - The SSL session timeout value in seconds.  If 0, a default of 7200
                      wil be used.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
      ivadmin_cfg_setssltimeout(
                                   const char              *cfg_file_name,
                                   int                     ssl_timeout,
                                   ivadmin_response        *rsp);



/*
   IVADMIN command line equivalent:  svrsslcfg -modify ....

   Replaces the optional application CA certificate
     Replace the optional SSL certificate in the keyring database.  The application must be
     stopped prior to invoking this API.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - A server already exists.

   cfg_file_name    - Configuration file.
   appl_cert        - The name of the file that contains a base-64 encoded SSL certificate.
                      This is an optional parameter.  If specified, the certificate will be
                      stored in the keyring database using a label of APPL_LDAP_CERT.
                      Typeical use of this parameter is to store the LDAP CA certificate
                      that the application uses when it authenticates directly to LDAP.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_setapplicationcert(
                                      const char              *cfg_file_name,
                                      const char              *appl_cert,
                                      ivadmin_response        *rsp);

/*
   IVADMIN command line equivalent:  svrsslcfg -addreplica ....

   Adds a replica entry to the configuration file.

   Possible reasons returned:

      IVADMIN_REASON_ALREADY_EXISTS  - The replica entry already exists

   cfg_file_name    - Configuration file.
   ivacld_host      - TCP hostname of the \"ivacld\" server.  This parameter is
                      required.
   ivacld_port      - Listening port number of the \"ivacld\" replica server.
                      This id the port number on which \"ivacld\" listens for
                      requests.  If not specified on an -add_replica action,
                      a default of 7136 will be used.
   ivacld_rank      - Replica order of preference among other replicas.  This
                      parameter defaults to 10 on the -add_replica action.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_addreplica(
                          const char              *cfg_file_name,
                          const char              *ivacld_host,
                          int                     ivacld_port,
                          int                     ivacld_rank,
                          ivadmin_response        *rsp);

/*
   IVADMIN command line equivalent:  svrsslcfg -chgreplica ....

   Changes parameters of a replica entry in the configuration file.

   Possible reasons returned:

      IVADMIN_REASON_NOT_EXISTS  - The replica entry does not exist.

   cfg_file_name    - Configuration file.
   ivacld_host      - TCP hostname of the \"ivacld\" server.  This parameter is
                      required.
   ivacld_port      - Listening port number of the \"ivacld\" replica server.
                      This id the port number on which \"ivacld\" listens for
                      requests.
   ivacld_rank      - Replica order of preference among other replicas.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_chgreplica(
                          const char              *cfg_file_name,
                          const char              *ivacld_host,
                          int                     ivacld_port,
                          int                     ivacld_rank,
                          ivadmin_response        *rsp);

/*
   IVADMIN command line equivalent:  svrsslcfg -rmvreplica ....

   Removes a replica entry from the configuration file.

   Possible reasons returned:


   cfg_file_name    - Configuration file.
   ivacld_host      - TCP hostname of the \"ivacld\" server.  This parameter is
                      required.
   rsp              - Response.  Indicates success or failure.  Free when no longer needed.
*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_rmvreplica(
                          const char              *cfg_file_name,
                          const char              *ivacld_host,
                          ivadmin_response        *rsp);
/*
   IVADMIN command line equivalent:  server task <server-name> <task-to-perform>

   Send a command to an AZN server.

   Possible reasons returned:

   IVADMIN_REASON_AZNADMINSVC_FAILURE - Although The AZN Administration
       Service implementer was successfully contacted, he failed to
       perform the requested task.  Additional information will be
       returned by the implementer in the result strings and/or outdata.

   ctx         - Context to communicate with management server.

   server      - Server name.

   task        - Command.  The command syntax can be anything but is best
                 done as pdadmin CLI format.  This allows the same command
                 to be supported in the pdadmin CLI.

   indata      - Pass through data that would allow a custom user
                 interface to communicate additional information
                 to the server.  A valid address of an azn_attrlist_h_t
                 is expected on input however if NULL is specified
                 the parameter is ignored.  It is also assumed the
                 caller has created this azn_attrlist_h_t using
                 azn_attrlist_create() prior to making this API
                 call. When the data is no longer required, the
                 caller should free the associated memory with
                 azn_attrlist_delete().

   outdata     - Pass through data that would allow a custom user
                 interface to communicate additional information
                 to the server.  A valid address of an azn_attrlist_h_t
                 is expected on input however if NULL is specified
                 the parameter is ignored.  It is also assumed the
                 caller has created this azn_attrlist_h_t using
                 azn_attrlist_create() prior to making this API
                 call. When the data is no longer required, the
                 caller should free the associated memory with
                 azn_attrlist_delete().

   resultcount - Count of result strings returned by the server.

   results     - Result strings.  These are the message strings that
                 are returned by the task.  These would typically show
                 up as output on a CLI interface or log output and
                 contain information about the success or failure
                 of the task.  You should free each result character
                 pointer and the array of pointers when  no longer
                 needed using ivadmin_free.

   rsp         - Success or failure.  This should be freed when no
                 longer needed using ivadmin_free().

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_server_performtask(ivadmin_context ctx,
                              const char *server,
                              const char *task,
                              azn_attrlist_h_t *indata,
                              azn_attrlist_h_t *outdata,
                              unsigned long *resultcount,
                              char ***results,
                              ivadmin_response *rsp);

/*
   IVADMIN command line equivalent:  server listtasks <server-name>

   Send a command to an AZN server.

   Possible reasons returned:

   IVADMIN_REASON_AZNADMINSVC_FAILURE - Although The AZN Administration
       Service implementer was successfully contacted, he failed to
       perform the requested task.  Additional information will be
       returned by the implementer in the result strings and/or outdata.

   ctx         - Context to communicate with management server.

   server      - Server name.

   indata      - Pass through data that would allow a custom user
                 interface to communicate additional information
                 to the server.  A valid address of an azn_attrlist_h_t
                 is expected on input however if NULL is specified
                 the parameter is ignored.  It is also assumed the
                 caller has created this azn_attrlist_h_t using
                 azn_attrlist_create() prior to making this API
                 call. When the data is no longer required, the
                 caller should free the associated memory with
                 azn_attrlist_delete().

   taskcount   - Number of task strings returned.

   tasks       - Task strings.  This is the list of tasks currently
                 supported by this server.  These should be in the
                 supported CLI syntax.  You should free each task
                 character pointer and the array of pointers when
                 no longer needed using ivadmin_free.

   outdata     - Pass through data that would allow a custom user
                 interface to communicate additional information
                 to the server.  A valid address of an azn_attrlist_h_t
                 is expected on input however if NULL is specified
                 the parameter is ignored.  It is also assumed the
                 caller has created this azn_attrlist_h_t using
                 azn_attrlist_create() prior to making this API
                 call. When the data is no longer required, the
                 caller should free the associated memory with
                 azn_attrlist_delete().

   resultcount - Count of result strings returned by the server.

   results     - Result strings.  These are the message strings that
                 are returned by the task.  These would typically show
                 up as output on a CLI interface or log output and
                 contain information about the success or failure
                 of the task.  You should free each result character
                 pointer and the array of pointers when  no longer
                 needed using ivadmin_free.

   rsp         - Success or failure.  This should be freed when no
                 longer needed using ivadmin_free().

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
ivadmin_server_gettasklist(ivadmin_context ctx,
                           const char *server,
                           azn_attrlist_h_t *indata,
                           unsigned long *taskcount,
                           char ***tasks,
                           azn_attrlist_h_t *outdata,
                           unsigned long *resultcount,
                           char ***results,
                           ivadmin_response *rsp);




/*
   IVADMIN command line equivalent:  server replicate <server-name>

   Notify AZN servers to receive database updates.

   ctx            - Context to communicate with management server.

   server         - Server name.  Optional name of server to notifiy
                    of database update.  If NULL is specified, all
                    servers configured to receive database update
                    notifications will be notified.
   results        - IVADMIN_TRUE - If a server was specified, this
                    will indicate the successful notification and
                    database replication by that server.  If no
                    server is specified this will indicate that
                    pdmgrd has begun notifying each AZN server but is
                    not an indication of successful notification or
                    replication to any one of those servers.
                    IVADMIN_FALSE - If a server was specified, this
                    will indicate the failure of the notification
                    and database replication by that server.  If no
                    server is specified this will indicate a failure
                    has occurred in requesting pdmgrd begin notifying
                    each AZN server.

   rsp            - Success or failure.  This should be freed when no
                    longer needed using ivadmin_free().

   authorization  - azn_operation_server_admin is required on the
                    /Management/Server object

*/
IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
ivadmin_server_replicate( ivadmin_context ctx,
                          const char *server,
                          ivadmin_response *rsp);

#ifdef __cplusplus
    }
#endif

#endif /* IVADMINAPI_H */
