/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * All Rights Reserved.
 *
 * Copyright (c) 1998, 1999, 2000 Tivoli Systems. All Rights Reserved.
 *
 * File Name: ogauthzn.h
 */

#ifndef _OGAUTHZN_H
#define _OGAUTHZN_H

#include <stddef.h>


/*
 * Function linkage modifiers for Win32
 */
#if defined(WIN32) || defined(_WIN32)
#define AZN_CALLTYPE       __cdecl
#undef AZN_DECLSPEC
#define AZN_DECLSPEC       __declspec( dllimport )
#define AZN_EXTSPEC        AZN_DECLSPEC
#define AZN_EXTSPEC_CONST  AZN_DECLSPEC extern const
#else /* WIN32 */
#define AZN_CALLTYPE
#undef AZN_DECLSPEC
#define AZN_DECLSPEC
#undef AZN_EXTSPEC
#define AZN_EXTSPEC extern
#define AZN_EXTSPEC_CONST  extern const
#endif /* WIN32 */


/*
 * Defined Constants
 */

/* for azn_boolean_t */
#ifndef TRUE
#define TRUE    1
#endif
#ifndef FALSE
#define FALSE   0
#endif

/* Permission codes returned by azn_decision_access_allowed*() */
#define  AZN_C_PERMITTED         0
#define  AZN_C_NOT_PERMITTED     1

#define  AZN_C_EMPTY_BUFFER            (NULL)
#define  AZN_C_NO_BUFFER               (NULL)

#define  AZN_C_INITIATOR_INDEX         0
#define  AZN_C_AUDIT_ID                "AZN_AUDIT_ID"
#define  AZN_C_REQUEST_TIME            "AZN_REQUEST_TIME"
#define  AZN_C_AUTHN_QUALITY           "AZN_AUTHN_QUALITY"
#define  AZN_C_REQUESTER_LOC           "AZN_REQUESTER_LOC"
#define  AZN_C_REQUEST_ROUTE_QOP       "AZN_REQUEST_ROUTE_QOP"
#define  AZN_C_VERSION                 "AZN_VERSION"
#define  AZN_C_INVALID_HANDLE          ((long)(NULL))
#define  AZN_C_AUTHN_LEVEL             "AUTHENTICATION_LEVEL"

/*
 * Default authorization weightings for participating entities
 * in the authorization process. If there was no weighting
 * specified in the service definition then the default EAS
 * weighting is one more than the core authzn engine weighting.
 * In this scenario any ruling for/against the decision by the
 * EAS will take priority over decisions from the core engine.
 */
#define  AZN_C_AUTHZN_ENGINE_WEIGHTING (100)
#define  AZN_C_DEFAULT_EAS_WEIGHTING   (AZN_C_AUTHZN_ENGINE_WEIGHTING + 1)


/* API Major Status Codes */

#define AZN_S_ERRCODE_MIN                        0x0000
#define AZN_S_COMPLETE                           0x0000
#define AZN_S_FAILURE                            0x0001
#define AZN_S_AUTHORIZATION_FAILURE              0x0002
#define AZN_S_INVALID_CREDS_HDL                  0x0003
#define AZN_S_INVALID_NEW_CREDS_HDL              0x0004
#define AZN_S_INVALID_ENTITLEMENTS_SVC           0x0005
#define AZN_S_INVALID_COMB_CREDS_HDL             0x0006
#define AZN_S_INVALID_MECHANISM_INFO             0x0007
#define AZN_S_INVALID_MECHANISM                  0x0008
#define AZN_S_INVALID_STRING_VALUE               0x0009
#define AZN_S_UNKNOWN_LABEL                      0x000a
#define AZN_S_INVALID_ADDED_CREDS_HDL            0x000b
#define AZN_S_INVALID_PROTECTED_RESOURCE         0x000c
#define AZN_S_INVALID_OPERATION                  0x000d
#define AZN_S_INVALID_PAC                        0x000e
#define AZN_S_INVALID_PAC_SVC                    0x000f
#define AZN_S_INVALID_PERMINFO_REF               0x0010
#define AZN_S_INVALID_MOD_FUNCTION               0x0011
#define AZN_S_INVALID_SUBJECT_INDEX              0x0012
#define AZN_S_UNIMPLEMENTED_FUNCTION             0x0013
#define AZN_S_INVALID_ATTRLIST_HDL               0x0014
#define AZN_S_INVALID_ATTR_NAME                  0x0015
#define AZN_S_INVALID_BUFFER                     0x0016
#define AZN_S_INVALID_BUFFER_REF                 0x0017
#define AZN_S_INVALID_STRING_REF                 0x0018
#define AZN_S_ATTR_VALUE_NOT_STRING_TYPE         0x0019
#define AZN_S_ATTR_INVALID_INDEX                 0x001a
#define AZN_S_INVALID_INTEGER_REF                0x001b
#define AZN_S_INVALID_PERMISSION_REF             0x001c
#define AZN_S_INVALID_AUTHORITY                  0x001d
#define AZN_S_INVALID_APP_CONTEXT_HDL            0x001e
#define AZN_S_INVALID_ENTITLEMENTS_HDL           0x001f
#define AZN_S_INVALID_LABELING_SCHEME            0x0020
#define AZN_S_INVALID_INIT_DATA_HDL              0x0021
#define AZN_S_INVALID_INIT_INFO_HDL              0x0022
#define AZN_S_ATTR_VALUE_NOT_BUFFER_TYPE         0x0023
#define AZN_S_API_UNINITIALIZED                  0x0024
#define AZN_S_API_ALREADY_INITIALIZED            0x0025
#define AZN_S_SVC_DEFINITION_ERROR               0x0026
#define AZN_S_SVC_SERVICE_NOT_FOUND              0x0027
#define AZN_S_SVC_INIT_FAILED                    0x0028
#define AZN_S_SVC_SHUTDOWN_FAILED                0x0029
#define AZN_S_SVC_AUTHORIZATION_FAILED           0x002A
#define AZN_S_SVC_DLL_LOAD_FAILED                0x002B
#define AZN_S_SVC_INITIALIZE_NOT_FOUND           0x002C
#define AZN_S_SVC_SHUTDOWN_NOT_FOUND             0x002D
#define AZN_S_SVC_ENT_FUNC_NOT_FOUND             0x002E
#define AZN_S_SVC_PAC_FUNC_NOT_FOUND             0x002F
#define AZN_S_SVC_EAS_FUNC_NOT_FOUND             0x0030
#define AZN_S_SVC_CRED_MOD_FUNC_NOT_FOUND        0x0031
#define AZN_S_SVC_SERVICE_IS_REGISTERED          0x0032
#define AZN_S_SVC_DISPATCHER_FAILURE             0x0033
#define AZN_S_MINOR_CODE_MESSAGE_NOT_FOUND       0x0034
#define AZN_S_INVALID_EAS_ACL_TRIGGER            0x0035
#define AZN_S_INVALID_EAS_POP_TRIGGER            0x0036
#define AZN_S_INVALID_EAS_WEIGHTING              0x0037
#define AZN_S_UNKNOWN_EAS_SVC_PARAMETER          0x0038
#define AZN_S_SVC_ADMIN_POBJ_FUNC_NOT_FOUND      0x0039
#define AZN_S_INVALID_POBJ                       0x003A
#define AZN_S_INVALID_POBJ_REF                   0x003B
#define AZN_S_ATTR_VALUE_NOT_POBJ_TYPE           0x003C
#define AZN_S_SVC_ADMIN_UNKNOWN_PARAMETER        0x003D
#define AZN_S_SVC_ADMIN_POBJ_NOT_SPECIFIED       0x003E
#define AZN_S_SVC_ADMIN_TASK_FUNC_NOT_FOUND      0x003F
#define AZN_S_SVC_ADMIN_POBJ_ALREADY_REGISTERED  0x0040
#define AZN_S_INVALID_MESSAGE_ID_REF             0x0041
#define AZN_S_MAJOR_CODE_MESSAGE_NOT_FOUND       0x0042
#define AZN_S_ATTR_VALUE_NOT_ULONG_TYPE          0x0043
#define AZN_S_SVC_ADMIN_INVALID_SVCINFO_HDL      0x0044
#define AZN_S_SVC_ADMIN_INVALID_ARG_COUNT        0x0045
#define AZN_S_SVC_ADMIN_INVALID_ARG_ARRAY        0x0046
#define AZN_S_SVC_ADMIN_OUT_OF_MEMORY            0x0047
#define AZN_S_SVC_ADMIN_INVALID_ARGUMENT         0x0048
#define AZN_S_SVC_ADMIN_INVALID_TASK             0x0049
#define AZN_S_SVC_ENT_INVALID_SVCINFO_HDL        0x004A
#define AZN_S_SVC_ENT_INVALID_ARG_COUNT          0x004B
#define AZN_S_SVC_ENT_INVALID_ARG_ARRAY          0x004C
#define AZN_S_SVC_ENT_OUT_OF_MEMORY              0x004D
#define AZN_S_SVC_ENT_INVALID_ARGUMENT           0x004E
#define AZN_S_SVC_EAS_INVALID_SVCINFO_HDL        0x004F
#define AZN_S_SVC_EAS_INVALID_ARG_COUNT          0x0050
#define AZN_S_SVC_EAS_INVALID_ARG_ARRAY          0x0051
#define AZN_S_SVC_EAS_OUT_OF_MEMORY              0x0052
#define AZN_S_SVC_EAS_INVALID_ARGUMENT           0x0053
#define AZN_S_SVC_CRED_MOD_INVALID_SVCINFO_HDL   0x0054
#define AZN_S_SVC_CRED_MOD_INVALID_ARG_COUNT     0x0055
#define AZN_S_SVC_CRED_MOD_INVALID_ARG_ARRAY     0x0056
#define AZN_S_SVC_CRED_MOD_OUT_OF_MEMORY         0x0057
#define AZN_S_SVC_CRED_MOD_INVALID_ARGUMENT      0x0058
#define AZN_S_SVC_PAC_INVALID_SVCINFO_HDL        0x0059
#define AZN_S_SVC_PAC_INVALID_ARG_COUNT          0x005A
#define AZN_S_SVC_PAC_INVALID_ARG_ARRAY          0x005B
#define AZN_S_SVC_PAC_OUT_OF_MEMORY              0x005C
#define AZN_S_SVC_PAC_INVALID_ARGUMENT           0x005D
#define AZN_S_INVALID_LISTENING_PORT             0x005E
#define AZN_S_INVALID_MAJOR_CODE                 0x005F
#define AZN_S_ATTR_READONLY                      0x0060
#define AZN_S_ERRCODE_MAX                        0x0060

/* Utility Function Major Status Codes */
#define AZN_S_U_ERRCODE_MIN                0x0100
#define AZN_S_U_INTERNAL_ERROR             0x0100 /* internal err: see minor */
#define AZN_S_U_DCE_AUTHEN_FAILED          0x0101 /* DCE authenticate failed */
#define AZN_S_U_LDAP_AUTHEN_FAILED         0x0102 /* LDAP authenticate failed*/
#define AZN_S_U_ALREADY_AUTHENTICATED      0x0103 /* API caller is logged in */
#define AZN_S_U_PASSWORD_EXPIRED           0x0104 /* user's password expired */
#define AZN_S_U_INVALID_USER               0x0105 /* user info is invalid */
#define AZN_S_U_USER_REGISTRY_UNAVAIL      0x0106 /* user registry is offline */
#define AZN_S_U_INVALID_PARAMETER          0x0107 /* invalid calling params */
#define AZN_S_U_PTHREAD_ERROR              0x0108 /* error from pthread call */
#define AZN_S_U_INVALID_PRINC_NAME         0x0109 /* invalid principal name */
#define AZN_S_U_INVALID_PASSWORD           0x010a /* invalid password string */
#define AZN_S_U_INVALID_MECH_ID_REF        0x010b /* mech id ref is invalid */
#define AZN_S_U_INVALID_KEYTAB_PATH        0x010c /* invalid keytab string */
#define AZN_S_U_UNUSED_010D                0x010d /* unused */
#define AZN_S_U_ACCOUNT_DISABLED           0x010e /* account login disabled */
#define AZN_S_U_TOD_ACCESS_DENIED          0x010f /* Time Of Day access denied */
#define AZN_S_U_ACCOUNT_LOCKEDOUT          0x0110 /* Too many bad login attempts */
#define AZN_S_U_PASSWORD_TOO_SHORT         0x0111 /* new password to short */
#define AZN_S_U_PASSWORD_HAS_SPACES        0x0112 /* new password has illegal spaces */
#define AZN_S_U_PASSWORD_TOO_MANY_REPEATED 0x0113 /* new password has too many repeated characters */
#define AZN_S_U_PASSWORD_TOO_FEW_ALPHA     0x0114 /* new password has too few alphabetic characters  */
#define AZN_S_U_PASSWORD_TOO_FEW_NONALPHA  0x0115 /* new password has too few non-alphabetic characters */
#define AZN_S_U_INSUFFICIENT_ACCESS        0x0116 /* don't have the rights to do the operation */
#define AZN_S_U_URAF_AUTHEN_FAILED         0x0117 /* URAF authenticate failed */
#define AZN_S_U_ERRCODE_MAX                0x0117


/* PD Protected Object entitlement service minor status codes  */
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_1    (azn_ent_svc_err_prefix | 0x1)
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_2    (azn_ent_svc_err_prefix | 0x2)
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_3    (azn_ent_svc_err_prefix | 0x3)
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_4    (azn_ent_svc_err_prefix | 0x4)
#define AZN_ENT_PDPOBJ_UNUSED_COMPATIBILITY_5    (azn_ent_svc_err_prefix | 0x5)
#define AZN_ENT_PDPOBJ_INVALID_OBJECT_PATH       (azn_ent_svc_err_prefix | 0x6)
#define AZN_ENT_PDPOBJ_INVALID_OPERATION_SET     (azn_ent_svc_err_prefix | 0x7)
#define AZN_ENT_PDPOBJ_INVALID_NUM_OPERATION_SET (azn_ent_svc_err_prefix | 0x8)
#define AZN_ENT_PDPOBJ_AUTHZN_ENGINE_NOT_FOUND   (azn_ent_svc_err_prefix | 0x9)
#define AZN_ENT_PDPOBJ_AZN_CLIENT_UNSUPPORTED    (azn_ent_svc_err_prefix | 0xa)

/* type definitions */

/* implementation specific status and handle definitions */

typedef unsigned long azn_status_t;
typedef long azn_creds_h_t;
typedef long azn_attrlist_h_t;
typedef unsigned long azn_boolean_t;
typedef unsigned long azn_ulong_t;

/* standard data types */

typedef struct azn_buffer_desc_struct {
  size_t         length;
  unsigned char  *value;
} azn_buffer_desc, *azn_buffer_t;

typedef char *azn_string_t;

typedef struct azn_pobj_desc_struct {
  azn_string_t    name; /* protected object path */
  unsigned int    type; /* protected object type; customer defined */
  azn_string_t    description; /* protected object description */
  azn_boolean_t   is_policy_attachable; /* indicates whether authorization
                                       policy can be attached to this object */
} azn_pobj_desc, *azn_pobj_t;


/*
 * Unauthenticated user mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain.
 *
 * Fields:
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 */
typedef struct {
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
} azn_unauth_t;


/*
 * Default user registry mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain. The default user registry is used to build the client
 * credential. This structure can be used with a NULL mechanism ID
 * in a call to azn_id_get_creds() and the client credential will
 * be built from the default user registry with which PD was installed.
 * The client does not need to know what the user registry type is.
 *
 * Fields:
 *
 *      principal       The client principal's name in the registry.
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 *      authnmech_info  Additional authentication mech info.
 *
 *  reserved    Fields reserved for future use.
 *
 */
typedef struct {
    azn_string_t        principal;
    azn_string_t        auth_method;
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
    azn_string_t        authnmech_info;
    void        *reserved[9];
} azn_authdefault_t;


/*
 * DCE specific authentication mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain.
 *
 * Fields:
 *
 *      context         A DCE login context pointer.
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 *      authnmech_info  Additional authentication mech info.
 *
 */
typedef struct {
    azn_string_t        principal;
    azn_string_t        auth_method;
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
    azn_string_t        authnmech_info;
} azn_authdce_t;


/*
 * LDAP specific authentication mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain.
 *
 * Fields:
 *
 *      ldap_dn         An LDAP distinguished name.
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 *      authnmech_info  Additional authentication mech info.
 *
 *
 */
typedef struct {
    azn_string_t        ldap_dn;
    azn_string_t        auth_method;
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
    azn_string_t        authnmech_info;
} azn_authldap_t;


/*
 * URAF specific authentication mechanism information structure.
 *
 * This structure is used by the Auth API as an input of credential
 * and other information for use within the Policy Director secure
 * domain.
 *
 * Fields:
 *
 *      uraf_user       An URAF user name.
 *
 *      ipaddr          IP address of requesting user in network
 *                      byte order.
 *
 *      qop             Quality of protection required for requests
 *                      made by this user.
 *
 *      user_info       Additional user information that may be
 *                      required for auditing.
 *
 *      browser_info    The browser (if any) employed by the user.
 *
 *      authnmech_info  Additional authentication mech info.
 *
 *  uraf_handle Pointer to a URAF registry handle context.
 *
 *
 */
typedef struct {
    azn_string_t        uraf_user;
    azn_string_t        auth_method;
    unsigned int        ipaddr;
    azn_string_t        qop;
    azn_string_t        user_info;
    azn_string_t        browser_info;
    azn_string_t        authnmech_info;
    void                *uraf_handle;
} azn_authuraf_t;



#ifdef __cplusplus
extern "C" {
#endif

/**********************************************************************
 * Constant variable declarations
 **********************************************************************/
/*
 * Mechanism IDs for azn_id_get_cred(). Each of these match
 * one of the azn_auth*_t type definitions above.
 */
AZN_EXTSPEC azn_string_t IV_UNAUTH; /* unauthenticated user */
AZN_EXTSPEC azn_string_t IV_DCE;    /* DCE authenticated user */
AZN_EXTSPEC azn_string_t IV_LDAP;   /* LDAP authenticated user */
AZN_EXTSPEC azn_string_t IV_URAF;   /* URAF authenticated user */



/**************************************************************************
 * Operation string declarations. These values are passed into the
 * azn_decision_access_allowed* and azn_entitlements_get_operations*
 * calls respectively. The actual string values of these operations
 * are an internal implementation detail and should not be relied
 * upon. The operations can be concatenated together to form complex
 * operation strings e.g., to request a read/modify operation.
 */
AZN_EXTSPEC_CONST azn_string_t azn_operation_attach;
AZN_EXTSPEC_CONST azn_string_t azn_operation_bypasstod;
AZN_EXTSPEC_CONST azn_string_t azn_operation_browse;
AZN_EXTSPEC_CONST azn_string_t azn_operation_control;
AZN_EXTSPEC_CONST azn_string_t azn_operation_traverse;
AZN_EXTSPEC_CONST azn_string_t azn_operation_delegation;
AZN_EXTSPEC_CONST azn_string_t azn_operation_view;
AZN_EXTSPEC_CONST azn_string_t azn_operation_modify;
AZN_EXTSPEC_CONST azn_string_t azn_operation_delete;
AZN_EXTSPEC_CONST azn_string_t azn_operation_server_admin;
AZN_EXTSPEC_CONST azn_string_t azn_operation_read;
AZN_EXTSPEC_CONST azn_string_t azn_operation_execute;
AZN_EXTSPEC_CONST azn_string_t azn_operation_list_directory;
AZN_EXTSPEC_CONST azn_string_t azn_operation_connect;
AZN_EXTSPEC_CONST azn_string_t azn_operation_forward;
AZN_EXTSPEC_CONST azn_string_t azn_operation_password;
AZN_EXTSPEC_CONST azn_string_t azn_operation_create;
AZN_EXTSPEC_CONST azn_string_t azn_operation_add;
AZN_EXTSPEC_CONST azn_string_t azn_operation_trace;



/**************************************************************************
 * Initialization data attribute names.
 *
 * Passed to azn_initialize. Each attribute name requires one or
 * more const azn_string_t values as outlined below.
 */

/*
 * AZN API authzn engine mode: "local" (using a policy cache replica)
 * or "remote" (making RPC requests to ivacld).
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_mode;


/*
 * AZN API authzn server name
 * First appears in PD3.8.
 */
AZN_EXTSPEC_CONST azn_string_t azn_server_name;

/*
 * AZN API application host
 */
AZN_EXTSPEC_CONST azn_string_t azn_app_host;

/*
 * Path and filename to be used to contain remaining initialization
 * parameters.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_cfg_file;


/*
 * SSL attributes that control the communications between azn calls and
 * servers.  The units for azn_init_ssl_timeout are seconds.  The
 * units for azn_init_pwd_life are days.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_keyfile;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_stashfile;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_timeout;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_pwd_life;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_mgr_config;

/* First appearance in PD3.8. */
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_keyfile_pwd;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_keyfile_label;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_listening_port;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_cert_life;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_max_worker_threads;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_auto_refresh;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_io_inactivity_timeout;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_authn_type;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_authn_user;
AZN_EXTSPEC_CONST azn_string_t azn_init_ssl_authn_pwd;


/*
 * Communications attributes, the azn_init_master_xxx attributes refer
 * to the location of ivmgrd, the replica attribute is the location of
 * an ivacld server.  There can be more than one azn_init_replica.
 *
 * The ivacld server replica's are of the format:
 *     <replica hostname>:<port>:<preference>:<replica cert dn>
 * where:
 *   <replica hostname> is the network name of the server.
 *   <port>             is the port on the server.
 *   <preference>       is a ranking for attempting contact from 1 to
 *                      10.  10 is the highest preference.
 *   <DN>               is the DN for the ivacld server.
 * For example,
 *  "rweber.bball.com:7137:5:cn=ivacld/rweber,o=Policy Director,C=US"
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_master_host;
AZN_EXTSPEC_CONST azn_string_t azn_init_master_port;
AZN_EXTSPEC_CONST azn_string_t azn_init_master_dn;
AZN_EXTSPEC_CONST azn_string_t azn_init_replica;


/*
 * The set of attributes that the caller is interested in receiving
 * from the azn_decision_access_allowed_ext() call in the permission
 * info attribute list. By default there is no information returned
 * by this call. Adding a value to this attribute at init time enables
 * the attribute to be returned as permission info if it is applicable
 * to the current decision call. This list may also include attributes
 * that are user defined. e.g., by setting an attribute on an ACL or POP
 * using "modify set attribute" command of "pdadmin".
 *
 * The following attribute constants are recognised by the authzn engine.
 * Refer below for more information on these permission info attributes:
 *
 *  azn_perminfo_all_attrs          Return all attributes.
 *  azn_perminfo_al_ulong           Audit level (unsigned long).
 *  azn_perminfo_qop                Quality of Protection (string).
 *  azn_perminfo_qop_ulong          Quality of Protection (unsigned long).
 *  azn_perminfo_wm_ulong           Warning mode
 *                                  (boolean returned in unsigned long valueg)
 *  azn_perminfo_wm_permitted_ulong Access permitted by warning mode
 *                                  (boolean retuned in unsigned long value).
 *  azn_perminfo_stepup_level       Authentication level required to access
 *                                  the object (unsigned long).
 *  azn_pop_ext_attr_loc            POP location (string)
 *  azn_acl_ext_attr_loc            ACL location (string)
 *
 *  The caller needs to use azn_attrlist_add_entry  to set these values in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call, OR set them in this configuration file.
 *
 *  See below for details on which API the caller needs to use to
 *  retrieve these attributes from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_set_perminfo_attrs;

/* Path and filename to be used to contain cached authzn policy. */
AZN_EXTSPEC_CONST azn_string_t azn_init_db_file;

/* String ID or tag to identify the client aznAPI client. Default: aznAPI. */
AZN_EXTSPEC_CONST azn_string_t azn_init_logging_client_id;

/* Audit log path and filename to collect PD audit events. */
AZN_EXTSPEC_CONST azn_string_t azn_init_audit_file;

/* Debug log path and filename to collect PD debug events. */
AZN_EXTSPEC_CONST azn_string_t azn_init_debug_file;

/*
 * Audit logging config.
 *
 * Multi-valued attribute containing entries of the format:
 *
 *  module-id[:trace-level]
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_auditcfg;

/* Debug logging config.
 *
 * Multi-valued attribute containing entries of the format:
 *
 *  module-id[:trace-level]
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_debugcfg;

/*
 * The maximum size (in bytes) for the audit and debug files. When
 * the log size reaches this value the log is backed up and a new
 * log file is started. A value of 0 will disable log rollover.
 * A negative value will perform rollover daily regardless of size.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_log_size;

/*
 * The audit/debug logging service flush interval in seconds.
 * 0 value sets default of 20 seconds. Maximum: 3600 (6 hours).
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_log_flush_interval;

/*
 * Interval in seconds for local policy cache polled updates. Values
 * can be "disable", "default" or a string time in seconds.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_cache_refresh_interval;

/*
 * Flags to enable the reception of policy cache update notifications.
 * Values can be a combination of: "disable", "enable", "use_tcp_port",
 * "use_udp_port" and "dynamic_port_selection". Multiple values are
 * accepted for this attribute name and are logically OR'd together.
 * A "disable" value overrides all others and disables the notification
 * listener.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_listen_flags;

/*
 * LDAP user registry configurations can specify the following
 * attributes to enable LDAP registry use in the AZN API.
 */

/* The LDAP server host name.  */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_host;

/*
 * The LDAP server host port (a numerical string). This will  be
 * the SSL port if SSL comms with the LDAP host is to be enabled.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_port;

/* The distinguished name to bind to the LDAP server with. */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_bind_dn;

/* The password for the bind DN. */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_bind_pwd;

/*
 * The LDAP server's SSL keyfile. (This must be non-NULL to use
 * SSL comms with the LDAP host)
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_ssl_keyfile;

/* The LDAP server's SSL keyfile distinguished name.  */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_ssl_keyfile_dn;

/* The LDAP server's SSL keyfile password.  */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_ssl_keyfile_pwd;

/*
 * The LDAP server replica's.
 *
 * Each string value is of the format <ldap-server>,<port>,<type>,<pref>
 * where:
 *   <ldap-server> is the network name of the server.
 *   <port> is the port on the ldap server.
 *   <type> is one of "readonly" or "readwrite"
 *   <pref> is a level from 1 to 10.  10 is the highest preference.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_replica;


/*
 * This enables caching of the user, group, and LDAP policy data in
 * the client to reduce the turnaround of similar LDAP transactions.
 * The string value can be "true" or "false". Defaults to "false".
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_cache;

/*
 * This makes the writable LDAP server the preferred one for
 * the client. Calls will still failover to a read-only server
 * if the write server is offline. The string value can be
 * "true" or "false". Defaults to "false".
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_prefer_rw_svr;

/*
 * Set the maximum number of entries returned from an LDAP search.
 * This is further constrained by the limits set by the LDAP server.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_max_search_size;


/*
 * Chooses whether the client will use the ldap_compare or the
 * ldap_bind call to authenticate a the users password. This
 * applies to calls to azn_util_client_authenticate() and
 * azn_util_password_authenticate() with an LDAP registry. The
 * string value can be "true" or "false". Default is "false"
 * which is to use the ldap_bind call.
 */
AZN_EXTSPEC_CONST azn_string_t azn_init_ldap_use_compare;


/*
 * aznAPI service initialization attributes.
 *
 * Multi-valued string attributes for use with azn_initialize().
 * Each string value is of the format:
 *
 *  "<service-id> = <path-to-dll> [ & params ... ]"
 *
 * The service-id is the string by which the service can be
 * identified by the aznAPI client. The path-to-dll is the
 * fully qualified path to the DLL which contains the service
 * executable code. Optionally you can specify parameters to
 * pass to the service when it is initialised by the aznAPI.
 * The parameters are considered to be all data following the
 * '&' symbol in the string.
 *
 * The external authorization service can take an additional
 * optional "-weight" parameter as follows:
 *
 *      <service-id> = <path-to-dll> [ -weight N ] [ & params ... ]
 *
 * The value N can be any unsigned long value whose absolute
 * value is used to appropriately weight the decision of an
 * EAS for or against access to the object. The default value
 * for an EAS if this is not specified is set to the value
 * AZN_C_DEFAULT_EAS_WEIGHTING. The assigned weighting of the
 * core authorization engine is AZN_C_AUTHZN_ENGINE_WEIGHTING.
 */

/* Entitlements service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_ent_svc;

/* PAC service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_pac_svc;

/* Credential modification service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_cred_mod_svc;

/* Administration service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_admin_svc;

/* External Authorization Service definition attribute */
AZN_EXTSPEC_CONST azn_string_t azn_init_extern_authzn_svc;


/**************************************************************************
 * Authzn credential attribute names.
 *
 * These attributes are returned by the azn_creds_get_attrlist_for_subject
 * call. Some of these may not be present according to the type of the
 * authenticated user and the information that was originally passed to
 * the call used to build a given cred.
 */

/* the credential version */
AZN_EXTSPEC_CONST azn_string_t azn_cred_version;

/* the mechanism ID for this cred */
AZN_EXTSPEC_CONST azn_string_t azn_cred_mech_id;

/* the UUID of the entity */
AZN_EXTSPEC_CONST azn_string_t azn_cred_principal_uuid;

/* the string name of the entity */
AZN_EXTSPEC_CONST azn_string_t azn_cred_principal_name;

/* the string group UUID memberships of this entity */
AZN_EXTSPEC_CONST azn_string_t azn_cred_group_uuids;

/* the string group name memberships of this entity */
AZN_EXTSPEC_CONST azn_string_t azn_cred_group_names;

/* the authentication ID used to build these authzn credentials */
AZN_EXTSPEC_CONST azn_string_t azn_cred_authzn_id;

/* any user info passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_user_info;

/* any authn method info passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_auth_method;

/* any authn mechanism info passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_authnmech_info;

/* the IP address information passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_ip_address;

/*
 * The Quality of Protection information passed in the mech info struct
 * One of "none", "integrity" or "privacy".
 */
AZN_EXTSPEC_CONST azn_string_t azn_cred_qop_info;

/* the browser information passed in the mech info struct */
AZN_EXTSPEC_CONST azn_string_t azn_cred_browser_info;

/* For LDAP credentials only: contains the LDAP DN of the principal */
AZN_EXTSPEC_CONST azn_string_t azn_cred_ldap_dn;



/**************************************************************************
 * Permission info attributes.
 *
 * These attributes are returned by the azn_decision_access_allowed_ext
 * call. Some of these may not be present in all permission_info attribute
 * lists returned according to the type of the access decision and the
 * information that was originally passed in with the call.
 *
 * These attribute names may also be passed in to the azn_initialize()
 * call as values for the azn_init_set_perminfo_attrs attribute to control which
 * of them are returned as permission info for any decision call. The
 * default is that no permission information will be returned.
 */

/*
 * This value is passed in the azn_init_set_perminfo_attrs attribute
 * to azn_initialize() to specify that the caller wished to receive
 * all attributes that can potentially be returned by the authzn
 * engine on an extended authzn decision call. This will override
 * any other attributes passed in the azn_init_set_perminfo_attrs
 * attribute.
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_all_attrs;

/*
 * The quality of protection required between caller and the domain
 * before access to the protected object can be granted. The QOP value
 * will be one of "none", "integrity" or "privacy".
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type: const azn_string_t
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_qop;

/*
 * This is the same as the azn_perminfo_qop attribute except it is returned as
 * an unsigned long.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type:  unsigned long
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_qop_ulong;

/* azn_perminfo_qop_ulong values */
AZN_EXTSPEC unsigned int azn_perminfo_qop_uint_none;
AZN_EXTSPEC unsigned int azn_perminfo_qop_uint_integrity;
AZN_EXTSPEC unsigned int azn_perminfo_qop_uint_privacy;

/*
 * The authentication level required to access the object, returned as an
 * unsigned long.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type:  unsigned long
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_stepup_level;

/*
 * Warning mode.  When warning mode is enabled access is always granted.
 * If access should not have been granted then the access is logged.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type:  boolean values returned in unsigned long
 */

AZN_EXTSPEC_CONST azn_string_t azn_perminfo_wm_ulong;

/*
 * Access permitted because of warning mode.  The boolean indicator is used
 * to tell the caller that access was granted because warning mode is enabled.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type:  boolean values returned in unsigned long
 */
AZN_EXTSPEC_CONST azn_string_t azn_perminfo_wm_permitted_ulong;

/*
 * Audit level.  Audit events that are performed for this authorization check.
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry_ulong() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 * type: unsigned long bitmask
 */

AZN_EXTSPEC_CONST azn_string_t azn_perminfo_al_ulong;

/* azn_perminfo_al_ulong values */
AZN_EXTSPEC unsigned int azn_perminfo_al_none;
AZN_EXTSPEC unsigned int azn_perminfo_al_permit;
AZN_EXTSPEC unsigned int azn_perminfo_al_deny;
AZN_EXTSPEC unsigned int azn_perminfo_al_error;
AZN_EXTSPEC unsigned int azn_perminfo_al_admin;
AZN_EXTSPEC unsigned int azn_perminfo_al_all;

/*
 * POP/ACL location perminfo
 *
 *  The caller needs to use azn_attrlist_add_entry  to set this value in
 *  the azn_init_set_perminfo_attrs attribute in the init_data parameter to
 *  the azn_initialize() call.
 *
 *  The caller needs to use the azn_attrlist_get_entry() API to
 *  retrieve this attribute from the perm_info output parameter of the
 *  azn_decision_access_allowed_ext() API.
 *
 */
AZN_EXTSPEC_CONST azn_string_t azn_pop_ext_attr_loc;
AZN_EXTSPEC_CONST azn_string_t azn_acl_ext_attr_loc;


/**********************************************************************
 * aznAPI service module data and attribute names.
 **********************************************************************/

/* The prefixes used by the services to define minor error codes. */
AZN_EXTSPEC unsigned int azn_ent_svc_err_prefix;
AZN_EXTSPEC unsigned int azn_mod_svc_err_prefix;
AZN_EXTSPEC unsigned int azn_pac_svc_err_prefix;
AZN_EXTSPEC unsigned int azn_eas_svc_err_prefix;
AZN_EXTSPEC unsigned int azn_admin_svc_err_prefix;

/*
 * Used to return the version string of a service module to
 * the aznAPI runtime.
 */
AZN_EXTSPEC_CONST azn_string_t azn_svc_version;

/*
 * Returns any plug-in load errors returned by the system in
 * the format:
 *
 *  "dll-path: error-string"
 *
 * Where the dll-path is the filename of the DLL that was passed
 * to the load routine and error-string is the result returned.
 */
AZN_EXTSPEC_CONST azn_string_t azn_svc_load_error;


/**********************************************************************
 * PAC service IDs.
 **********************************************************************/

/*
 * PAC Service ID: DCE EPAC service.
 *
 * Converts a DCE EPAC structure into an azn API cred.
 * This service does not perform the reverse operation
 * of building a DCE EPAC from a azn API cred. This
 * service is only for use with azn_pac_get_creds().
 *
 * This service takes one parameter: "-loadnames". If
 * this parameter is passed to the service as an argument
 * in the service definition then the EPAC PAC service
 * will attempt to load the real string names of the
 * principal and group uuids into the returned aznAPI
 * credential. This enables the return of values in the
 * azn_cred_group_names attribute for creds generated
 * from DCE EPACs. This feature is turned off by default.
 */
AZN_EXTSPEC_CONST azn_string_t azn_pac_svc_dce_epac;


/**********************************************************************
 * Entitlements service IDs.
 **********************************************************************/

/*
 * Entitlements Service ID: Policy Director Protected Object Entitlements.
 *
 * This service takes a credential, a directory within the protected
 * object tree and a set of operations and returns the list of protected
 * objects in the given directory and its subdirectories for which the
 * given aznAPI credential has permission to perform the requested set
 * of operations. The output is returned as a multi-string valued
 * attribute.
 *
 * NOTE: this is a default service. ie. when a NULL entitlements
 * service ID is specified to azn_entitlements_get_entitlements()
 * then this is the service that is called.
 */
AZN_EXTSPEC_CONST azn_string_t azn_ent_svc_pd_pobj;  /* service ID */

/* Input attribute: the protected object directory path to be searched */
AZN_EXTSPEC_CONST azn_string_t azn_ent_svc_pd_pobj_path;

/* Input attribute: the requested set of operations for the credential */
AZN_EXTSPEC_CONST azn_string_t azn_ent_svc_pd_pobj_reqd_ops;

/* Output attribute: the list of objects which match the input criteria */
AZN_EXTSPEC_CONST azn_string_t azn_ent_svc_pd_pobj_matches;

/**********************************************************************
 * Administration service attributes.
 **********************************************************************/

/* Administration service pluginstatus attribute */
/* This attribute is RESERVED for internal use by the aznAPI Admin Service */
AZN_EXTSPEC_CONST azn_string_t azn_admin_svc_pluginstatus;

/* Administration service protected object attribute */
AZN_EXTSPEC_CONST azn_string_t azn_admin_svc_pobj;

/* Administration service task attribute */
AZN_EXTSPEC_CONST azn_string_t azn_admin_svc_task;

/* Administration service results attribute */
AZN_EXTSPEC_CONST azn_string_t azn_admin_svc_results;



/**********************************************************************
 * Function Prototypes
 **********************************************************************/


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_add_entry(
  const azn_attrlist_h_t   attr_list     /* in */,
  const azn_string_t       attr_name     /* in */,
  const azn_string_t       string_value  /* in */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_add_entry_buffer(
  const azn_attrlist_h_t   attr_list     /* in */,
  const azn_string_t       attr_name     /* in */,
  const azn_buffer_t       buffer_value  /* in */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_add_entry_pobj(
  const azn_attrlist_h_t   attr_list     /* in */,
  const azn_string_t       attr_name     /* in */,
  const azn_pobj_t         pobj_value    /* in */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_add_entry_ulong(
  const azn_attrlist_h_t   attr_list     /* in */,
  const azn_string_t       attr_name     /* in */,
  const azn_ulong_t        ulong_value   /* in */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_create(
  azn_attrlist_h_t   *attr_list  /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_delete(
  azn_attrlist_h_t   *attr_list  /* in, out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_delete_entry(
  azn_attrlist_h_t   attr_list     /* in, out */,
  const azn_string_t       attr_name     /* in */
);


AZN_DECLSPEC
azn_attrlist_h_t
AZN_CALLTYPE
azn_attrlist_copy(
  const azn_attrlist_h_t   attr_list     /* in */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_name_get_num(
  const azn_attrlist_h_t   attr_list    /* in */,
  const azn_string_t       attr_name    /* in */,
  unsigned int       *num_values  /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_entry_buffer_value(
  const azn_attrlist_h_t   attr_list      /* in */,
  const azn_string_t       attr_name      /* in */,
  const unsigned int       value_index    /* in */,
  azn_buffer_t       *buffer_value   /* out */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_entry_pobj_value(
  const azn_attrlist_h_t   attr_list      /* in */,
  const azn_string_t       attr_name      /* in */,
  const unsigned int       value_index    /* in */,
  azn_pobj_t         *pobj_value   /* out */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_entry_string_value(
  const azn_attrlist_h_t   attr_list      /* in */,
  const azn_string_t       attr_name      /* in */,
  const unsigned int       value_index    /* in */,
  azn_string_t       *string_value  /* out */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_entry_ulong_value(
  const azn_attrlist_h_t   attr_list     /* in */,
  const azn_string_t       attr_name     /* in */,
  const unsigned int       value_index   /* in */,
  azn_ulong_t        *ulong_value  /* out */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_attrlist_get_names(
  const azn_attrlist_h_t   attr_list      /* in */,
  azn_string_t       *attr_names[]  /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_combine(
  const azn_creds_h_t   creds             /* in */,
  const azn_creds_h_t   creds_to_add      /* in */,
  azn_creds_h_t   *combined_creds   /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_create(
  azn_creds_h_t   *creds  /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_delete(
  azn_creds_h_t   *creds  /* in, out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_for_subject(
  const azn_creds_h_t   creds           /* in */,
  const unsigned int    subject_index   /* in */,
  azn_creds_h_t   *subject_creds  /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_get_attrlist_for_subject(
  const azn_creds_h_t      creds            /* in */,
  const unsigned int       subject_index    /* in */,
  azn_attrlist_h_t   *creds_attrlist  /* out */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_get_attr_value_string(
  const azn_creds_h_t      creds            /* in */,
  const unsigned int       subject_index    /* in */,
  const azn_string_t       attr_name        /* in */,
  azn_string_t       *attr_value      /* out */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_set_attr_value_string(
  const azn_creds_h_t      creds            /* in */,
  const unsigned int       subject_index    /* in */,
  const azn_string_t       attr_name        /* in */,
  const azn_string_t       attr_value       /* in */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_get_pac(
  const azn_creds_h_t   creds       /* in */,
  const azn_string_t    pac_svc_id  /* in */,
  azn_buffer_t    *pac        /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_modify(
  const azn_creds_h_t      creds       /* in */,
  const azn_string_t       mod_svc_id  /* in */,
  const azn_attrlist_h_t   mod_info    /* in */,
  azn_creds_h_t      *new_creds  /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_num_of_subjects(
  const azn_creds_h_t   creds              /* in */,
  unsigned int    *num_of_subjects   /* out */
);


AZN_DECLSPEC
azn_creds_h_t
AZN_CALLTYPE
azn_creds_copy(
    const azn_creds_h_t creds   /* in */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_creds_equal(
    const azn_creds_h_t cred1,     /* in */
    const azn_creds_h_t cred2,     /* in */
    azn_boolean_t       *is_equal  /* out */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_decision_access_allowed(
  const azn_creds_h_t   creds               /* in */,
  const azn_string_t    protected_resource  /* in */,
  const azn_string_t    operation           /* in */,
  int             *permission         /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_decision_access_allowed_ext(
  const azn_creds_h_t      creds               /* in */,
  const azn_string_t       protected_resource  /* in */,
  const azn_string_t       operation           /* in */,
  const azn_attrlist_h_t   app_context         /* in */,
  int                *permission         /* out */,
  azn_attrlist_h_t   *permission_info    /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_entitlement_get_entitlements(
  const azn_creds_h_t      creds                /* in */,
  const azn_string_t       entitlements_svc_id  /* in */,
  const azn_attrlist_h_t   app_context          /* in */,
  azn_attrlist_h_t   *entitlements        /* out */
);


AZN_DECLSPEC
unsigned int
AZN_CALLTYPE
azn_error_major(
  const azn_status_t   azn_status  /* in */
);


AZN_DECLSPEC
unsigned int
AZN_CALLTYPE
azn_error_minor(
  const azn_status_t   azn_status  /* in */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_error_get_string(
  const azn_status_t    aznstatus          /* in */,
  azn_string_t    *error_string      /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_error_minor_get_string(
  const unsigned int   minor_error          /* in */,
  azn_string_t   *minor_error_string  /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_id_get_creds(
  const azn_string_t    authzn_authority  /* in */,
  const azn_string_t    mechanism_id      /* in */,
  const azn_buffer_t    mechanism_info    /* in */,
  azn_creds_h_t   *authzn_creds     /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_initialize(
  const azn_attrlist_h_t  init_data           /* in */,
  azn_attrlist_h_t  *init_info          /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_pac_get_creds(
  const azn_buffer_t    pac         /* in */,
  const azn_string_t    pac_svc_id  /* in */,
  azn_creds_h_t   *new_creds  /* out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_release_buffer(
  azn_buffer_t   *buffer     /* in, out */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_release_pobj(
  azn_pobj_t   *pobj     /* in, out */
);

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_release_string(
  azn_string_t   *string     /* in, out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_release_strings(
  azn_string_t   *strings[]  /* in, out */
);


AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_shutdown(
);


#ifdef __cplusplus
}
#endif

#endif /* _OGAUTHZN_H */
