/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Copyright (C) Tivoli Systems/IBM.  2000
 *
 * NAME
 *     pdaudit - Policy Director Auditing Service header
 *
 * DESCRIPTION
 *     This header file contains the definitions of Auditing Service
 *     specific macros and the "C" API functions prototypes.
 */

#ifndef PD_AUDIT_H
#define PD_AUDIT_H

#include <pdlog.h>
#include <ogauthzn.h>

typedef enum PDAuditEnumResource {
    PD_AUDIT_RESOURCE_AZN,
    PD_AUDIT_RESOURCE_PROCESS,
    PD_AUDIT_RESOURCE_TCB,
    PD_AUDIT_RESOURCE_CREDENTIAL,
    PD_AUDIT_RESOURCE_POLICY,
    PD_AUDIT_RESOURCE_GENERAL,
    PD_AUDIT_RESOURCE_APPLICATION
} PDAuditEnumResource;

typedef enum PDAuditEnumOutcome {
    PD_AUDIT_OUTCOME_SUCCESS,
    PD_AUDIT_OUTCOME_FAILURE,
    PD_AUDIT_OUTCOME_PENDING,
    PD_AUDIT_OUTCOME_UNKNOWN
} PDAuditEnumOutcome;

typedef enum PDAuditEnumSystem {
    PD_AUDIT_SYSTEM_UNIX,
    PD_AUDIT_SYSTEM_NT
} PDAuditEnumSystem;

#ifdef __cplusplus
extern "C" {
#endif
typedef IVTbool (*PDAuditFilterFunction)(const IVTuint32, const PDAuditEnumOutcome);
#ifdef __cplusplus
}
#endif

typedef struct PDAuditCommon {
   char                 *linkage;
   PDAuditEnumResource  resource;
   char                 *originating_location;
   IVTstatus            fail_status;
   azn_creds_h_t        cred;
   char                 *protected_object;
   IVTbool              process_info_is_set;
   PDAuditEnumSystem    system;
   pid_t                pid;
   uid_t                uid;
   uid_t                eid;
   gid_t                gid;
   gid_t                egid;
   IVTbool              azn_info_is_set;
   IVTuint32            azn_permission;
   IVTuint32            azn_result;
   IVTstatus            azn_result_qualifier;
} PDAuditCommon;

/*
 * Audit Record Version
 */
#define PD_AUDIT_RECORD_REV     0x101

/*
 * Log Service API interfaces.
 */

#ifdef __cplusplus
extern "C" {
#endif
    AUDIT_DECLSPEC void *PDAuditAttachSink(char *,
                            IVTuint32,
                            PDAuditFilterFunction,
                            IVTstatus *);
    AUDIT_DECLSPEC void PDAuditDetachSink(void **,
                           IVTstatus *);
    AUDIT_DECLSPEC void *PDAuditEventStart(void *,
                            IVTuint32,
                            PDAuditEnumOutcome,
                            IVTstatus *);
    AUDIT_DECLSPEC void PDAuditEventAddText(void *event_h,
                             IVTstatus *status_p,
                             IVTuint32 hint,
                             char *fmt, ...);
    AUDIT_DECLSPEC void PDAuditEventAddCommon(void *event_h,
                               PDAuditCommon *c_p,
                               IVTstatus *);
#define PDAuditEventCommit PDLogEventCommit
#ifdef __cplusplus
}
#endif

#endif /* PD_AUDIT_H */

