/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Copyright (C) Tivoli Systems/IBM.  2000
 *
 * NAME
 *      pdlog.h - PolicyDirector Logging Service header
 *
 * DESCRIPTION
 *     This header file contains the definitions of Logging Service
 *     specific macros and the "C" API functions prototypes.
 */

#ifndef PD_LOG_H
#define PD_LOG_H

#include <auditdecl.h>
#include <stdio.h>
#include <stdarg.h>
#include <sys/types.h>

#include <IVTypes.h>

/*
 * Audit record commit options
 */

typedef enum PDLogEnumCommit {
    PD_COMMIT_QUEUE = 0,
    PD_COMMIT_SYNC = 1,
    PD_COMMIT_FLUSH = 2,
    PD_TERMINATOR = 4
} PDLogEnumCommit;

/*
 * Define tag names for common PD log files
 */
#define PD_REGISTRY_POOL "EventPool"
#define PD_AUDIT_LOG "audit"
#define PD_TRACE_LOG "trace"

typedef enum PDTraceEnumLogLevel {
    PD_TRACE_0 = 0,
        PD_TRACE_NONE = PD_TRACE_0,

    PD_TRACE_1 = 1,
        PD_TRACE_BASE = PD_TRACE_1,

        PD_TRACE_2 = 2,
    PD_TRACE_3,
    PD_TRACE_4,
    PD_TRACE_5,
    PD_TRACE_6,
        PD_TRACE_7,

        PD_TRACE_8,
        PD_TRACE_ROUTINE = PD_TRACE_8,

        PD_TRACE_9 = 9,
        PD_TRACE_DATA = PD_TRACE_9,

    PD_TRACE_MIN = PD_TRACE_1,
    PD_TRACE_MAX = PD_TRACE_9
} PDTraceEnumLogLevel;
/*
 * Log Service API interfaces.
 */

#ifdef __cplusplus
extern "C" {
#endif

    AUDIT_DECLSPEC IVTstatus PDLogPoolOpen(char *config_file_p,
                                           char *stanza_p);

    AUDIT_DECLSPEC void PDLogPoolClose();

    AUDIT_DECLSPEC void *PDLogPool(char *category_p);

    AUDIT_DECLSPEC IVTbool PDLogPoolHasObservers(void *pool_h);

    AUDIT_DECLSPEC void *PDLogConfigureAgent(char *category_p,
                                             char *adaptor_p);

    AUDIT_DECLSPEC void PDLogDestroyAgent(void **agent_h);

    AUDIT_DECLSPEC void *PDLogAttachSink(char *poolname_p);

    AUDIT_DECLSPEC void PDLogDetachSink(void **sink_h);

    AUDIT_DECLSPEC void *PDLogEventStart(void *event_sink_h);

    AUDIT_DECLSPEC char *PDLogEventGetData(void *event_sink_h);

    AUDIT_DECLSPEC void PDLogEventAddText(void *event_p,
                        IVTuint32 size,
                        char *fmt, ...);

    AUDIT_DECLSPEC void PDLogEventAddData(void *event_p,
                        size_t size,
                        void *segment);

    AUDIT_DECLSPEC void PDLogEventPokeData(void *event_p,
                        IVTuint32 offset,
                        size_t size,
                        void *segment);

    AUDIT_DECLSPEC void PDLogEventTimestamp(void *event_p);

    AUDIT_DECLSPEC void PDLogEventDiscard(void **event_p);

    AUDIT_DECLSPEC void PDLogEventCommit(void **event_p,
                          PDLogEnumCommit options,
                          IVTstatus *);

    AUDIT_DECLSPEC void PDLogEvent(void *pool_h,
                          IVTuint32 size,
                          char *fmt, ...);
/*
 * DEPRECATED PD3.7 BACKWARD COMPATIBILITY FUNCTIONS
 */
    AUDIT_DECLSPEC void *PDLogInitialize(char *logname_p,
                                         char *blade_p,
                                         char *logpath_p,
                                         IVTuint32 flush_secs,
                                         IVTuint32 logsize,
                                         IVTbool isbuffered,
                                         IVTstatus *status_p);

    AUDIT_DECLSPEC void PDLogTerminate(void **log_h_p);

#ifdef __cplusplus
}
#endif

#endif /* PD_LOG_H */
